@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -119,6 +119,12 @@ namespace DisplayPort
         ITU709,
     } ;
 
+    enum MainLinkChannelCoding
+    {
+        ChannelCoding8B10B = 0,
+        ChannelCoding128B132B,
+    };
+
     #define HDCP_BCAPS_SIZE                 (0x1)
     #define HDCP_VPRIME_SIZE                (0x14)
     #define HDCP_KSV_FIFO_SIZE              (0xF)
@@ -162,6 +168,7 @@ namespace DisplayPort
         //
         virtual bool     getLaneStatusClockRecoveryDone(int lane) = 0;                // DPCD offset 202, 203
         virtual bool     getLaneStatusSymbolLock(int lane)= 0;
+        virtual bool     getLaneStatusChannelEqualizationDone(int lane)= 0;
         virtual bool     getInterlaneAlignDone() = 0;
         virtual bool     getDownStreamPortStatusChange() = 0;
     };
@@ -219,6 +226,7 @@ namespace DisplayPort
         virtual AuxRetry::status setIgnoreMSATimingParamters(bool msaTimingParamIgnoreEn) = 0;
         virtual AuxRetry::status setLinkQualLaneSet(unsigned lane, LinkQualityPatternType linkQualPattern) = 0;
         virtual AuxRetry::status setLinkQualPatternSet(LinkQualityPatternType linkQualPattern, unsigned laneCount = 0) = 0;
+        virtual AuxRetry::status setLinkQualPatternSet(DP2xPatternInfo& patternInfo, unsigned laneCount = 0) = 0;
     };
 
     class LinkCapabilities
@@ -230,6 +238,12 @@ namespace DisplayPort
         //
         virtual NvU64    getMaxLinkRate() = 0;                                // Maximum byte-block in Hz
         virtual unsigned getMaxLaneCount() = 0;                               // DPCD offset 0x0002h
+        //
+        // Bit mask for sink side (sink and LTTPR) supported UHBR Link Rates.
+        // Defines the same as NV0073_CTRL_CMD_DP_GET_CAPS_PARAMS.UHBRSupportedByGpu and
+        //                     NV0073_CTRL_DFP_GET_INFO_PARAMS.UHBRSupportedByDfp
+        //
+        virtual NvU32    getUHBRSupported() = 0;                              // DPCD offset 0x2215h
         virtual unsigned getMaxLaneCountSupportedAtLinkRate(LinkRate linkRate) = 0;
         virtual bool     getEnhancedFraming() = 0;
         virtual bool     getSupportsNoHandshakeTraining() = 0;
@@ -331,7 +345,7 @@ namespace DisplayPort
             if (lttprGetRevisionMajor() > major)
                 return true;
 
-            if (lttprGetRevisionMinor() < major)
+            if (lttprGetRevisionMajor() < major)
                 return false;
 
             return lttprGetRevisionMinor() >= minor;
@@ -432,7 +446,7 @@ namespace DisplayPort
         //    If set to HPD mode we'll always receive an HPD whenever the topology changes.
         //    The library supports using both modes.
         //
-        virtual AuxRetry::status     setMultistreamHotplugMode(MultistreamHotplugMode notifyType) = 0;
+        virtual AuxRetry::status setMultistreamHotplugMode(MultistreamHotplugMode notifyType) = 0;
 
         //
         //  Interrupts
@@ -472,8 +486,12 @@ namespace DisplayPort
         virtual void             refreshLinkStatus() = 0;
         virtual bool             isLinkStatusValid(unsigned lanes) = 0;
 
-        virtual void getCustomTestPattern(NvU8 *testPattern) = 0;                         // DPCD offset 250 - 259
-
+        // DPCD offset 250 - 259
+        virtual void get80BitsCustomTestPattern(NvU8 *testPattern) = 0;
+        // DPCD offset 2230 - 2250
+        virtual void get264BitsCustomTestPattern(NvU8 *testPattern) = 0;
+        // DPCD offset 249
+        virtual void getSquarePatternNum(NvU8 *sqNum) = 0;
         //
         //  Message Boxes
         //
@@ -533,6 +551,7 @@ namespace DisplayPort
         virtual bool readPsrEvtIndicator(vesaPsrEventIndicator *psrErr) = 0;
         virtual bool readPrSinkDebugInfo(panelReplaySinkDebugInfo *prDbgInfo) = 0;
 
+        virtual bool     isDpInTunnelingSupported() = 0;
         virtual void     enableDpTunnelingBwAllocationSupport() = 0;
         virtual bool     isDpTunnelBwAllocationEnabled() = 0;
         virtual bool     getDpTunnelEstimatedBw(NvU8 &estimatedBw) = 0;
@@ -546,8 +565,16 @@ namespace DisplayPort
         virtual bool     clearDpTunnelingEstimatedBwStatus() = 0;
         virtual bool     clearDpTunnelingBwAllocationCapStatus() = 0;
 
+        virtual AuxRetry::status notifySDPErrDetectionCapability() = 0;
+        virtual bool isDp2xChannelCodingCapable() = 0;
+        virtual void setIgnoreCableIdCaps(bool bIgnore) = 0;
+        virtual void overrideCableIdCap(LinkRate linkRate, bool bEnable) = 0;
+        virtual void setConnectorTypeC(bool bTypeC) = 0;
+        virtual void initialize() = 0;
+        virtual AuxRetry::status setMainLinkChannelCoding(MainLinkChannelCoding channelCoding) = 0;
+        virtual void setUSBCCableIDInfo(NV0073_CTRL_DP_USBC_CABLEID_INFO *cableIDInfo) = 0;
+        virtual void setCableVconnSourceUnknown() = 0;
         virtual ~DPCDHAL() {}
-
     };
 
     //
@@ -652,7 +679,7 @@ namespace DisplayPort
                 bool      bFECSupportedRepeater[NV_DPCD14_PHY_REPEATER_CNT_MAX];
                 // If all the LTTPRs supports FEC
                 bool      bFECSupported;
-
+                bool      bAuxlessALPMSupported;                    // DPCD offset F0009
             } repeaterCaps;
 
             struct
@@ -689,8 +716,9 @@ namespace DisplayPort
             bool      streamStatusChanged;                          // DPCD offset 2005
             bool      hdmiLinkStatusChanged;                        // DPCD offset 2005
             bool      dpTunnelingIrq;                               // DPCD offset 2005
-            NvU8      eightyBitCustomPat[10];                       // DPCD offset 250 - 259
 
+            // DPCD offset 250 - 259
+            NvU8      cstm80Bits[NV_DPCD_TEST_80BIT_CUSTOM_PATTERN__SIZE];
             struct
             {
                 struct
@@ -844,6 +872,10 @@ namespace DisplayPort
 
         virtual LinkRate getMaxLinkRate();
 
+        virtual NvU32   getUHBRSupported()
+        {
+            return 0;
+        }
         // DPCD offset 2
         virtual unsigned getMaxLaneCount();
 
@@ -996,13 +1028,14 @@ namespace DisplayPort
         virtual AuxRetry::status setIgnoreMSATimingParamters(bool msaTimingParamIgnoreEn);
 
         virtual AuxRetry::status setLinkQualPatternSet(LinkQualityPatternType linkQualPattern, unsigned laneCount);
+        virtual AuxRetry::status setLinkQualPatternSet(DP2xPatternInfo& patternInfo, unsigned laneCount);
         virtual AuxRetry::status setLinkQualLaneSet(unsigned lane, LinkQualityPatternType linkQualPattern);
 
         virtual AuxRetry::status setMessagingEnable(bool _uprequestEnable, bool _upstreamIsSource);
         virtual AuxRetry::status setMultistreamLink(bool enable);
         virtual AuxRetry::status setMultistreamHotplugMode(MultistreamHotplugMode notifyType);
 
-        bool parseTestRequestTraining(NvU8 * buffer /* 0x18-0x28 valid */);
+        virtual bool parseTestRequestTraining(NvU8 * buffer /* 0x18-0x28 valid */);
         void parseAutomatedTestRequest(bool testRequestPending);
 
         virtual bool parseTestRequestPhy();
@@ -1170,6 +1203,11 @@ namespace DisplayPort
             return interrupts.laneStatusIntr.laneStatus[lane].clockRecoveryDone;
         }
 
+        virtual bool getLaneStatusChannelEqualizationDone(int lane)
+        {
+            return interrupts.laneStatusIntr.laneStatus[lane].channelEqualizationDone;
+        }
+
         virtual bool getInterlaneAlignDone()                                             // DPCD offset 204
         {
             return interrupts.laneStatusIntr.interlaneAlignDone;
@@ -1200,27 +1238,44 @@ namespace DisplayPort
             return interrupts.testPhyCompliance.testRequestPhyCompliance;
         }
 
-        virtual void getTestRequestTraining(LinkRate & rate, unsigned & lanes) // DPCD offset 219, 220
+        // DPCD offset 219, 220
+        virtual void getTestRequestTraining(LinkRate & rate, unsigned & lanes)
         {
             rate = interrupts.testTraining.testRequestLinkRate;
             lanes = interrupts.testTraining.testRequestLaneCount;
         }
 
-        virtual LinkQualityPatternType getPhyTestPattern()                            // DPCD offset 248
+        // DPCD offset 248
+        virtual LinkQualityPatternType getPhyTestPattern()
         {
             return interrupts.testPhyCompliance.phyTestPattern;
         }
 
-        virtual void getCustomTestPattern(NvU8 *testPattern)                         // DPCD offset 250 - 259
+        // DPCD offset 250 - 259
+        virtual void get80BitsCustomTestPattern(NvU8 *testPattern)
         {
             int i;
 
             for (i = 0; i < 10; i++)
             {
-                testPattern[i] = interrupts.eightyBitCustomPat[i];
+                testPattern[i] = interrupts.cstm80Bits[i];
             }
         }
 
+        // DPCD offset 2230 - 2250
+        virtual void get264BitsCustomTestPattern(NvU8 *testPattern)
+        {
+            DP_ASSERT(0 && "DP1x should never get this request.");
+            return;
+        }
+
+        // DPCD offset 249h
+        virtual void getSquarePatternNum(NvU8 *sqNum)
+        {
+            DP_ASSERT(0 && "DP1x should never get this request.");
+            return;
+        }
+
         virtual bool getBKSV(NvU8 *bKSV);
         virtual bool getBCaps(BCaps &bCaps, NvU8 * rawByte);
         virtual bool getHdcp22BCaps(BCaps &bCaps, NvU8 *rawByte);
@@ -1415,12 +1470,17 @@ namespace DisplayPort
         virtual TriState getDpTunnelBwRequestStatus();
         virtual bool     setDpTunnelBwAllocation(bool bEnable);
 
+        virtual bool     isDpInTunnelingSupported()
+        {
+            return caps.dpInTunnelingCaps.bIsSupported;
+        }
+
         virtual void     enableDpTunnelingBwAllocationSupport()
         {
             bEnableDpTunnelBwAllocationSupport = true;
         }
 
-        virtual bool isDpTunnelBwAllocationEnabled()
+        virtual bool     isDpTunnelBwAllocationEnabled()
         {
             return bIsDpTunnelBwAllocationEnabled;
         }
@@ -1433,6 +1493,18 @@ namespace DisplayPort
         bool clearDpTunnelingEstimatedBwStatus();
         bool clearDpTunnelingBwAllocationCapStatus();
 
+        virtual AuxRetry::status notifySDPErrDetectionCapability() { return AuxRetry::ack; }        
+        virtual bool isDp2xChannelCodingCapable() { return false; }
+        virtual void setIgnoreCableIdCaps(bool bIgnore) { return; }
+        virtual void overrideCableIdCap(LinkRate linkRate, bool bEnable) { return; }
+        virtual void setCableVconnSourceUnknown() { return; }
+
+        // implement this function if DPCDHALImpl needs updated state between hotunplug/plug
+        virtual void initialize(){}
+        virtual AuxRetry::status setMainLinkChannelCoding(MainLinkChannelCoding channelCoding){ return AuxRetry::ack; }
+        virtual MainLinkChannelCoding getMainLinkChannelCoding() { return ChannelCoding8B10B; }
+        virtual void setConnectorTypeC(bool bTypeC) { return; }
+        virtual void setUSBCCableIDInfo(NV0073_CTRL_DP_USBC_CABLEID_INFO *cableIDInfo) {}
     };
 
 }
