@@ -1,5 +1,5 @@
-/*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ /*
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -30,6 +30,7 @@
 #include "dp_wardatabase.h"
 #include "dp_edid.h"
 #include "dp_connectorimpl.h"
+#include "dp_connectorimpl2x.h"
 #include "dp_printf.h"
 
 using namespace DisplayPort;
@@ -93,9 +94,43 @@ void ConnectorImpl::applyOuiWARs()
     }
 }
 
+void ConnectorImpl2x::applyOuiWARs()
+{
+    switch (ouiId)
+    {
+        // Novatek Microelectronics Corp.
+        case 0x11EC38:
+            if((modelName[0] == 'N') && (modelName[1] == 'T') && (modelName[2] == '6'))
+            {
+                NvU8 portType;
+                if (hal->getSupportsMultistream() &&
+                    hal->isDp2xChannelCodingCapable() &&
+                    !hal->getDownstreamPort(&portType))
+                {
+                    bMstTimeslotBug4968411 = true;
+                    hal->setMainLinkChannelCoding(ChannelCoding128B132B);
+                }
+            }
+            break;
+        case 0xAD6000:
+            if ((modelName[0] == 'M') &&
+                (modelName[1] == 'C') &&
+                (modelName[2] == '2') &&
+                (modelName[3] == '9') &&
+                (modelName[4] == '0'))
+            {
+                bForceHeadShutdownPerMonitor = true;
+                bApplyStuffDummySymbolsWAR = true;
+                bStuffDummySymbolsFor128b132b = false;
+                bStuffDummySymbolsFor8b10b = true;
+            }
+            break;
+        
+    }
+}
+
 void Edid::applyEdidWorkArounds(NvU32 warFlag, const DpMonitorDenylistData *pDenylistData)
 {
-
     unsigned ManufacturerID = this->getManufId();
     unsigned ProductID = this->getProductId();
     unsigned YearWeek = this->getYearWeek();
@@ -185,8 +220,8 @@ void Edid::applyEdidWorkArounds(NvU32 warFlag, const DpMonitorDenylistData *pDen
             if ((ProductID >= 0x0776 ) && (ProductID <= 0x0779)) // Product id's range from decimal 1910 to 1913
             {
                 // if detailed pixel clock frequency = 106.50MHz
-                if ( (buffer.data[0x36] == 0x9A) &&
-                     (buffer.data[0x37] == 0x29) )
+                if ((buffer.data[0x36] == 0x9A) &&
+                    (buffer.data[0x37] == 0x29) )
                 {
                     // then change detailed pixel clock frequency to 106.54MHz to fix bug 343870
                     buffer.data[0x36] = 0x9E;
@@ -252,7 +287,7 @@ void Edid::applyEdidWorkArounds(NvU32 warFlag, const DpMonitorDenylistData *pDen
                 //    a "Monitor Description" of type FE = "ASCII Data String" which
                 //    has this panel's name = "LP171WX2-A4K5".
                 //
-                if ( (buffer.data[0x71] == 0x4C) &&
+                if ((buffer.data[0x71] == 0x4C) &&
                     (buffer.data[0x72] == 0x50) &&
                     (buffer.data[0x73] == 0x31) &&
                     (buffer.data[0x74] == 0x37) &&
@@ -290,7 +325,7 @@ void Edid::applyEdidWorkArounds(NvU32 warFlag, const DpMonitorDenylistData *pDen
                 //    a "Monitor Description" of type FE = "ASCII Data String" which
                 //    has this panel's name = "LP154WX4-TLC3".
                 //
-                if ( (buffer.data[0x71] == 0x4C) &&
+                if ((buffer.data[0x71] == 0x4C) &&
                     (buffer.data[0x72] == 0x50) &&
                     (buffer.data[0x73] == 0x31) &&
                     (buffer.data[0x74] == 0x35) &&
@@ -498,15 +533,6 @@ void Edid::applyEdidWorkArounds(NvU32 warFlag, const DpMonitorDenylistData *pDen
             }
             break;
 
-        // Dell
-        case 0xAC10:
-            // Dell U2713H has problem with LQA. Disable it.
-            if ((ProductID == 0xA092) || (ProductID == 0xF046))
-            {
-                this->WARFlags.reassessMaxLink = true;
-            }
-            break;
-
         // CMN
         case 0xAE0D:
             if (ProductID == 0x1747)
@@ -578,14 +604,84 @@ void Edid::applyEdidWorkArounds(NvU32 warFlag, const DpMonitorDenylistData *pDen
 
         // Asus
         case 0x6D1E:
-            if(ProductID == 0x7707)
+            if (ProductID == 0x7707)
             {
                 this->WARFlags.bIgnoreDscCap = true;
                 DP_PRINTF(DP_NOTICE, "DP-WAR> Panel incorrectly exposing DSC capability. Ignoring it.");
                 DP_PRINTF(DP_NOTICE, "DP-WAR> Bug 3543158");
             }
+            else if (ProductID == 0x5B9A)
+            {
+                this->WARFlags.bDisableDscMaxBppLimit = true;
+                DP_PRINTF(DP_NOTICE, "DP-WAR> Disable DSC max BPP limit of 16 for DSC.");
+            }
+            else if (ProductID == 0x5CA7 || ProductID == 0x9E9D || ProductID == 0x9EA0 || ProductID == 0x9EA5 || ProductID == 0x5CB7 ||
+                     ProductID == 0x9EA8 || ProductID == 0x9EAF || ProductID == 0x7846 || ProductID == 0x7849 || ProductID == 0x5CB5 ||
+                     ProductID == 0x77E0 || ProductID == 0x9EB9)
+            {
+                this->WARFlags.bForceHeadShutdownOnModeTransition = true;
+                DP_PRINTF(DP_NOTICE, "DP-WAR> Force head shutdown on Mode transition.");
+            }
+            break;
+        case 0xB306:
+            if (ProductID == 0x3228)
+            {
+                // ASUS PG32UQXR does not set DPCD 0x2217 to reflect correct CableID.
+                this->WARFlags.bSkipCableIdCheck = true;
+                DP_PRINTF(DP_NOTICE, "DP-WAR> Panel does not expose cable capability. Ignoring it. Bug 4968411");
+            }
+            else if(ProductID == 0x24b5 || ProductID == 0x32f2 || ProductID == 0x27BC)
+            {
+                //
+                // Asus ROG PG248QP  (0x24b5) Bug 5100062
+                // Asus ROG PG32UCDM (0x32f2) Bug 5088957
+                // Asus ROG PG27AQN  (0x27BC) Bug 5300665
+
+                this->WARFlags.bForceHeadShutdown = true;
+                DP_PRINTF(DP_NOTICE, "DP-WAR> Force head shutdown.");
+            }
             break;
 
+        // Samsung
+        case 0x2D4C:
+            if (ProductID == 0x7474)
+            {
+                //
+                // Samsung Odyssey Neo 57 G9 needs manual allocation of timeslots when directly connected to upstream source
+                // Do not allocate manual timeslot when under a separate branch. This is checked with branch OUI.
+                //
+                this->WARFlags.bAllocateManualTimeslots = true;
+                this->WARFlags.bDP2XPreferNonDSCForLowPClk = true;
+                DP_PRINTF(DP_NOTICE, "DP-WAR> Panel needs allocation of manual timeslot. Bug 4958974");
+            }
+            if (ProductID == 0x7256)
+            {
+                this->WARFlags.bDisableDownspread = true;
+            }
+            break;
+        case 0x545A: // VRT Varjo XR3
+            if (ProductID == 0x5841 || ProductID == 0x5842 || ProductID == 0x5843)
+            {
+                this->WARFlags.bDisableDownspread = true;
+                DP_PRINTF(DP_NOTICE, "DP-WAR> VRT monitor does not work with GB20x when downspread is enabled. Disabling downspread.");
+            }
+            break;
+
+        case 0xD94D: // Sony
+            if (ProductID == 0x07EE) // Sony SDM27Q10S
+            {
+                this->WARFlags.bSkipResetMSTMBeforeLt = true;
+                DP_PRINTF(DP_NOTICE, "DP-WAR> Sony SDM27Q10S needs to skip reset MST_EN before LT");
+            }
+            break;
+        case 0xAC10:
+            if (ProductID == 0x42AD || ProductID == 0x42AC)
+            {
+                this->WARFlags.bApplyStuffDummySymbolsWAR   = true;
+                this->WARData.bStuffDummySymbolsFor128b132b = true;
+                this->WARData.bStuffDummySymbolsFor8b10b    = false;
+            }
+            break;
         default:
             break;
     }
