@@ -183,7 +183,8 @@ nvlink_lib_discover_and_get_remote_conn_info
 (
     nvlink_link      *end,
     nvlink_conn_info *conn_info,
-    NvU32             flags
+    NvU32             flags,
+    NvBool            bForceDiscovery
 )
 {
     NvlStatus      status                            = NVL_SUCCESS;
@@ -258,7 +259,8 @@ nvlink_lib_discover_and_get_remote_conn_info
     conn_info->bConnected = NV_FALSE;
 
     // Get the remote_end of the link
-    nvlink_core_discover_and_get_remote_end(end, &remote_end, flags);
+    nvlink_core_discover_and_get_remote_end(end, &remote_end, flags,
+                                            bForceDiscovery);
 
     if (remote_end)
     {
