@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2019-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2019-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -83,13 +83,20 @@
 #include <gpu/kern_gpu_power.h>
 #include <gpu_mgr/gpu_mgr.h>
 #include <core/locks.h>
+#include <pex.h>
+#include <acpidsmguids.h>
 #include "kernel/gpu/intr/intr.h"
 
 #include <gpu/mem_sys/kern_mem_sys.h>
 #include <gpu/subdevice/subdevice.h>
+#include <ctrl/ctrl0080/ctrl0080gpu.h>
 #include <ctrl/ctrl2080/ctrl2080unix.h>
 #include <gpu/timer/objtmr.h>
 
+
+// mask holding dynamic power supported GPUs
+static NvU32 dynamicPowerSupportGpuMask = 0;
+
 //
 // Schedule timer based callback, to check for the complete GPU Idleness.
 // Windows has idle time from 70msec to 10sec, we opted for present duration
@@ -126,6 +133,12 @@
 //
 #define GCOFF_DYNAMIC_PM_MAX_FB_SIZE_MB      1024
 
+//
+// Safe/Fallback value of PEX reset delay for GCOFF/GPUOFF.
+// Used when programming upstream port as per sku specific PEX reset delay from vbios fails
+//
+#define DEFAULT_GCOFF_PEXRST_DELAY 0
+
 static void RmScheduleCallbackForIdlePreConditions(OBJGPU *);
 static void RmScheduleCallbackForIdlePreConditionsUnderGpuLock(OBJGPU *);
 static void RmScheduleCallbackToIndicateIdle(OBJGPU *);
@@ -251,21 +264,28 @@ static NvBool RmCanEnterGcxUnderGpuLock(
     OBJGPU *pGpu
 )
 {
+    nv_state_t    *nv  = NV_GET_NV_STATE(pGpu);
+    nv_priv_t     *nvp = NV_GET_NV_PRIV(nv);
+
     NV_ASSERT(rmDeviceGpuLockIsOwner(pGpu->gpuInstance));
 
     /*
-     * If GPU does not support GC6 and the actual FB utilization is higher than the threshold,
-     * then the GPU can neither enter GC6 nor GCOFF. So, return from here.
+     * If GC6 cannot be achieved (either GC6 is unsupported or the upstream port is not configured),
+     * Check for GCOFF prerequisites
      */
-    if (!pGpu->getProperty(pGpu, PDB_PROP_GPU_RTD3_GC6_SUPPORTED))
+    if (!pGpu->getProperty(pGpu, PDB_PROP_GPU_RTD3_GC6_SUPPORTED) ||
+        !nvp->gc6_upstream_port_configured)
     {
         NvU64          usedFbSize     = 0;
-        nv_state_t    *nv             = NV_GET_NV_STATE(pGpu);
-        nv_priv_t     *nvp            = NV_GET_NV_PRIV(nv);
         MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
 
+        /*
+         * In order to enter GCOFF, the FB memory threshold should be less than gcoff_max_fb_size
+         * and clients of RM should not hold the refcount to prevent GCOFF.
+         */
         if (!((memmgrGetUsedRamSize(pGpu, pMemoryManager, &usedFbSize) == NV_OK) &&
-            (usedFbSize <= nvp->dynamic_power.gcoff_max_fb_size)))
+              (usedFbSize <= nvp->dynamic_power.gcoff_max_fb_size) &&
+              (nvp->dynamic_power.clients_gcoff_disallow_refcount == 0)))
         {
             return NV_FALSE;
         }
@@ -422,7 +442,10 @@ NV_STATUS NV_API_CALL rm_schedule_gpu_wakeup(
 
     NV_ENTER_RM_RUNTIME(sp, fp);
 
-    ret = osQueueWorkItem(pGpu, RmForceGpuNotIdle, NULL);
+    ret = osQueueWorkItem(pGpu,
+                          RmForceGpuNotIdle,
+                          NULL,
+                          OS_QUEUE_WORKITEM_FLAGS_NONE);
 
     NV_EXIT_RM_RUNTIME(sp, fp);
 
@@ -732,7 +755,7 @@ rmReadAndParseDynamicPowerRegkey
         return NV_OK;
     }
 
-    chipId = gpuGetChipIdFromPmcBoot42(pNvp->pmc_boot_42);
+    chipId = decodePmcBoot42ChipId(pNvp->pmc_boot_42);
 
     // From GA102+, we enable RTD3 only if system is found to be Notebook
     if ((chipId >= NV_PMC_BOOT_42_CHIP_ID_GA102) &&
@@ -742,11 +765,57 @@ rmReadAndParseDynamicPowerRegkey
         return NV_OK;
     }
 
+    // For Tegra PCI iGPU, support GPU Rail-Gating feature
+    if (pNv->supports_tegra_igpu_rg)
+    {
+        *pOption = NV_REG_DYNAMIC_POWER_MANAGEMENT_FINE;
+        return NV_OK;
+    }
+
     *pOption = NV_REG_DYNAMIC_POWER_MANAGEMENT_NEVER;
     return NV_OK;
 }
 #undef NV_PMC_BOOT_42_CHIP_ID_GA102
 
+/*!
+ * @brief Initialize state related to Tegra iGPU dynamic power management.
+ * Called once per GPU during driver initialization.
+ *
+ * @param[in]   sp  nvidia_stack_t pointer.
+ * @param[in]   nv  nv_state_t pointer.
+ */
+void NV_API_CALL rm_init_tegra_dynamic_power_management(
+    nvidia_stack_t *sp,
+    nv_state_t *nv
+)
+{
+    nv_priv_t *nvp = NV_GET_NV_PRIV(nv);
+    void *fp;
+
+    if (!nv->is_tegra_pci_igpu || !nv->supports_tegra_igpu_rg)
+        return;
+
+    NV_ENTER_RM_RUNTIME(sp,fp);
+
+    portMemSet(&nvp->dynamic_power, 0, sizeof(nvp->dynamic_power));
+
+    nv->is_tegra_pci_igpu_rg_enabled = NV_FALSE;
+
+    NV_PRINTF(LEVEL_INFO,
+                "NVRM: Tegra PCI iGPU Rail-Gating is supported.\n");
+    NV_EXIT_RM_RUNTIME(sp,fp);
+}
+
+void rm_check_s0ix_regkey_and_platform_support(void)
+{
+    OBJSYS *pSys = SYS_GET_INSTANCE();
+    NvU32 data;
+    NvBool status = ((nv_platform_supports_s0ix()) &&
+                     ((osReadRegistryDword(NULL, NV_REG_ENABLE_S0IX_POWER_MANAGEMENT, &data) == NV_OK) && (data == 1)));
+
+    pSys->setProperty(pSys, PDB_PROP_SYS_SUPPORTS_S0IX, status);
+}
+
 /*!
  * @brief Initialize state related to dynamic power management.
  * Called once per GPU during driver initialization.
@@ -770,8 +839,6 @@ void NV_API_CALL rm_init_dynamic_power_management(
 
     NV_ENTER_RM_RUNTIME(sp,fp);
 
-    portMemSet(&nvp->dynamic_power, 0, sizeof(nvp->dynamic_power));
-
     /*
      * Program an impossible value so that we show correct status
      * during procfs read of runtime D3 status.
@@ -794,8 +861,9 @@ void NV_API_CALL rm_init_dynamic_power_management(
         nvp->dynamic_power.dynamic_power_regkey = regkeyValue;
     }
 
-    if (!nv_dynamic_power_available(nv) || !bPr3AcpiMethodPresent ||
-        (status != NV_OK))
+    nvp->pr3_acpi_method_present = bPr3AcpiMethodPresent;
+    if (!nv_dynamic_power_available(nv) || (status != NV_OK) ||
+        (!bPr3AcpiMethodPresent && !nv->supports_tegra_igpu_rg))
     {
         NV_PRINTF(LEVEL_NOTICE,
                   "%s: Disabling dynamic power management either due to lack"
@@ -847,6 +915,7 @@ void NV_API_CALL rm_init_dynamic_power_management(
         // fallthrough
     case NV_REG_DYNAMIC_POWER_MANAGEMENT_NEVER:
         nvp->dynamic_power.mode = NV_DYNAMIC_PM_NEVER;
+        nv->supports_tegra_igpu_rg = NV_FALSE;
         break;
     }
 
@@ -854,7 +923,23 @@ void NV_API_CALL rm_init_dynamic_power_management(
     if ((nvp->dynamic_power.mode == NV_DYNAMIC_PM_FINE) &&
         (nvp->dynamic_power.dynamic_power_regkey == NV_REG_DYNAMIC_POWER_MANAGEMENT_DEFAULT))
     {
-        nv_allow_runtime_suspend(nv);
+        // Enable Dynamic Rail-Gating for Tegra PCI iGPU
+        if (nv->supports_tegra_igpu_rg)
+        {
+            if (nv_pci_tegra_pm_init(nv) == NV_TRUE)
+            {
+                nv_printf(NV_DBG_INFO, "NVRM: Tegra PCI iGPU railgating is enabled\n");
+                nv->is_tegra_pci_igpu_rg_enabled = NV_TRUE;
+
+                // Allow runtime suspend for Tegra PCI iGPU device with RG successfully enabled.
+                nv_allow_runtime_suspend(nv);
+            }
+        }
+        else
+        {
+            // Allow runtime suspend for the GPU which is not Tegra PCI iGPU device.
+            nv_allow_runtime_suspend(nv);
+        }
     }
 
     // Legacy case: check if device is primary and driven by VBIOS or fb driver.
@@ -902,10 +987,19 @@ void NV_API_CALL rm_cleanup_dynamic_power_management(
     NV_ENTER_RM_RUNTIME(sp,fp);
 
     // Disable RTD3 infrastructure from OS side.
-    if ((nvp->dynamic_power.mode == NV_DYNAMIC_PM_FINE) &&
-        (nvp->dynamic_power.dynamic_power_regkey == NV_REG_DYNAMIC_POWER_MANAGEMENT_DEFAULT))
+    if (((nvp->dynamic_power.mode == NV_DYNAMIC_PM_FINE) &&
+        (nvp->dynamic_power.dynamic_power_regkey == NV_REG_DYNAMIC_POWER_MANAGEMENT_DEFAULT)) ||
+        nv->is_tegra_pci_igpu_rg_enabled == NV_TRUE)
     {
         nv_disallow_runtime_suspend(nv);
+
+        // Disable Dynamic Rail-Gating for Tegra PCI iGPU
+        if (nv->supports_tegra_igpu_rg)
+        {
+            nv_pci_tegra_pm_deinit(nv);
+            nv->is_tegra_pci_igpu_rg_enabled = NV_FALSE;
+            nv_printf(NV_DBG_INFO, "NVRM: Tegra PCI iGPU railgating is disabled\n");
+        }
     }
 
     nv_dynamic_power_state_t old_state = nvp->dynamic_power.state;
@@ -1073,6 +1167,63 @@ os_ref_dynamic_power(
     return status;
 }
 
+/*
+ * @brief This function queries the dynamic boost support.
+ * This function returns -1 in case it failed to query the
+ * dynamic boost support. It returns 1 if dynamic boost is
+ * supported and returns 0 if dynamic boost is not supported.
+ *
+ * The caller must hold the API lock before calling this
+ * function.
+ *
+ * @param[in]   nv      nv_state_t pointer.
+ */
+static int os_get_dynamic_boost_support(
+    nv_state_t *nv    
+)
+{
+    NV_STATUS status = NV_OK;
+    RM_API *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
+    NV0000_CTRL_SYSTEM_NVPCF_GET_POWER_MODE_INFO_PARAMS *pNvpcfParams;
+    OBJGPU *pGpu;
+    int ret;
+
+    status = os_alloc_mem((void**)&pNvpcfParams, sizeof(*pNvpcfParams));
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Failed to allocate memory\n");
+        ret = -1;
+        goto end;
+    }
+
+    pGpu = NV_GET_NV_PRIV_PGPU(nv);
+    pNvpcfParams->gpuId = pGpu->gpuId;
+    pNvpcfParams->subFunc = 
+        NVPCF0100_CTRL_CONFIG_DSM_2X_FUNC_GET_SUPPORTED_CASE;
+    status = pRmApi->Control(pRmApi,
+                           nv->rmapi.hClient,
+                           nv->rmapi.hClient,
+                           NV0000_CTRL_CMD_SYSTEM_NVPCF_GET_POWER_MODE_INFO,
+                           (void*)pNvpcfParams,
+                           sizeof(*pNvpcfParams));
+    if (status == NV_OK)
+    {
+        ret = 1;
+    }
+    else if (status == NV_ERR_NOT_SUPPORTED)
+    {
+        ret = 0;
+    }
+    else
+    {
+        ret = -1;
+    }
+
+    os_free_mem(pNvpcfParams);
+end:
+    return ret;
+}
+
 /*!
  * @brief Wrapper around os_ref_dynamic_power() suitable for use from the
  * per-OS layers.
@@ -1300,10 +1451,10 @@ static void timerCallbackToRemoveIdleHoldoff(
     NV_STATUS  status = NV_OK;
     OBJGPU    *pGpu   = reinterpretCast(pCallbackData, OBJGPU *);
 
-    status = osQueueWorkItemWithFlags(pGpu,
-                                      RmRemoveIdleHoldoff,
-                                      NULL,
-                                      OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_SUBDEVICE);
+    status = osQueueWorkItem(pGpu,
+                             RmRemoveIdleHoldoff,
+                             NULL,
+                             OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_SUBDEVICE);
 
     if (status != NV_OK)
     {
@@ -1333,7 +1484,10 @@ static void timerCallbackToIndicateIdle(
         nvp->dynamic_power.state == NV_DYNAMIC_POWER_STATE_IDLE_SUSTAINED &&
         RmCanEnterGcxUnderGpuLock(pGpu))
     {
-        osQueueWorkItem(pGpu, RmIndicateIdle, NULL);
+        osQueueWorkItem(pGpu,
+                        RmIndicateIdle,
+                        NULL,
+                        OS_QUEUE_WORKITEM_FLAGS_NONE);
     }
     else
     {
@@ -1541,39 +1695,33 @@ static void RmDestroyDynamicPowerCallbacks(
 }
 
 /*
- * @brief Adds a GPU to OBJOS::dynamicPowerSupportGpuMask
+ * @brief Adds a GPU to dynamicPowerSupportGpuMask
  *
  * @param[in]                  instance
  */
-void osAddGpuDynPwrSupported
+static void AddGpuDynamicPowerSupported
 (
     NvU32 instance
 )
 {
-    OBJSYS *pSys = SYS_GET_INSTANCE();
-    OBJOS *pOS = SYS_GET_OS(pSys);
-
-    pOS->dynamicPowerSupportGpuMask |= (1 << instance);
+    dynamicPowerSupportGpuMask |= NVBIT(instance);
 }
 
 /*
- * @brief Removes a GPU from OBJOS::dynamicPowerSupportGpuMask
+ * @brief Removes a GPU from dynamicPowerSupportGpuMask
  *
  * @param[in]                  instance
  */
-void osRemoveGpuDynPwrSupported
+static void RemoveGpuDynamicPowerSupported
 (
     NvU32 instance
 )
 {
-    OBJSYS *pSys = SYS_GET_INSTANCE();
-    OBJOS *pOS = SYS_GET_OS(pSys);
-
-    pOS->dynamicPowerSupportGpuMask &= ~(1 << instance);
+    dynamicPowerSupportGpuMask &= ~NVBIT(instance);
 }
 
 /*
- * @brief queries  OBJOS::dynamicPowerSupportGpuMask
+ * @brief queries  dynamicPowerSupportGpuMask
  *
  * @param[in]                  void
  */
@@ -1582,10 +1730,7 @@ NvU32 osGetDynamicPowerSupportMask
     void
 )
 {
-    OBJSYS *pSys = SYS_GET_INSTANCE();
-    OBJOS *pOS = SYS_GET_OS(pSys);
-
-    return  pOS->dynamicPowerSupportGpuMask;
+    return dynamicPowerSupportGpuMask;
 }
 
 /*!
@@ -1623,7 +1768,7 @@ void RmDestroyDeferredDynamicPowerManagement(
 
     RmCancelDynamicPowerCallbacks(pGpu);
     RmDestroyDynamicPowerCallbacks(pGpu);
-    osRemoveGpuDynPwrSupported(gpuGetInstance(pGpu));
+    RemoveGpuDynamicPowerSupported(gpuGetInstance(pGpu));
 }
 
 /*!
@@ -1907,6 +2052,12 @@ static NvBool RmCheckRtd3GcxSupport(
         return NV_FALSE;
     }
 
+    if (!nvp->pr3_acpi_method_present)
+    {
+        NV_PRINTF(LEVEL_NOTICE, "RTD3 ACPI support is not available.\n");
+        return NV_FALSE;
+    }
+
     return NV_TRUE;
 }
 
@@ -1935,7 +2086,7 @@ static void RmInitDeferredDynamicPowerManagement(
              NV_PRINTF(LEVEL_NOTICE, "RTD3 is not supported.\n");
              return;
         }
-        osAddGpuDynPwrSupported(gpuGetInstance(pGpu));
+        AddGpuDynamicPowerSupported(gpuGetInstance(pGpu));
         nvp->dynamic_power.b_fine_not_supported = NV_FALSE;
         status = CreateDynamicPowerCallbacks(pGpu);
 
@@ -2029,7 +2180,7 @@ RmUpdateFixedFbsrModes(OBJGPU *pGpu)
 }
 
 static NV_STATUS
-RmPowerManagementInternal(
+RmPowerManagement(
     OBJGPU *pGpu,
     nv_pm_action_t pmAction
 )
@@ -2045,8 +2196,8 @@ RmPowerManagementInternal(
         nv_priv_t  *nvp = NV_GET_NV_PRIV(nv);
         NvBool bcState = gpumgrGetBcEnabledStatus(pGpu);
         Intr *pIntr = GPU_GET_INTR(pGpu);
-
         MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
+
         if ((pmAction == NV_PM_ACTION_HIBERNATE) || (pmAction == NV_PM_ACTION_STANDBY))
         {
             //
@@ -2113,25 +2264,13 @@ RmPowerManagementInternal(
                 rmStatus = NV_ERR_INVALID_ARGUMENT;
                 break;
         }
+
         pMemoryManager->fixedFbsrModesMask = 0;
     }
 
     return rmStatus;
 }
 
-static NV_STATUS
-RmPowerManagement(
-    OBJGPU *pGpu,
-    nv_pm_action_t pmAction
-)
-{
-    NV_STATUS   rmStatus;
-
-    rmStatus = RmPowerManagementInternal(pGpu, pmAction);
-
-    return rmStatus;
-}
-
 /*!
  * @brief Call core RM to perform the GCx (GC6/GCOF) entry sequence (before
  * powering off the GPU) or exit sequence (after powering the GPU back on).
@@ -2155,6 +2294,7 @@ NV_STATUS RmGcxPowerManagement(
 {
     KernelMemorySystem *pKernelMemorySystem = GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu);
     nv_state_t         *nv                  = NV_GET_NV_STATE(pGpu);
+    nv_priv_t          *nvp                 = NV_GET_NV_PRIV(nv);
     NV_STATUS           status              = NV_OK;
 
     if (pGpu->acpiMethodData.jtMethodData.bSBIOSCaps &&
@@ -2185,7 +2325,8 @@ NV_STATUS RmGcxPowerManagement(
         // 2. For system PM with s2idle, GC6 can be used if it is
         //    supported by the GPU.
         //
-        if (pGpu->getProperty(pGpu, PDB_PROP_GPU_RTD3_GC6_SUPPORTED))
+        if (pGpu->getProperty(pGpu, PDB_PROP_GPU_RTD3_GC6_SUPPORTED) &&
+            nvp->gc6_upstream_port_configured)
         {
             bCanUseGc6 = bIsDynamicPM ? NV_TRUE : nv_s2idle_pm_configured();
         }
@@ -2202,10 +2343,14 @@ NV_STATUS RmGcxPowerManagement(
             pGpu->setProperty(pGpu, PDB_PROP_GPU_GCOFF_STATE_ENTERING, NV_TRUE);
 
             //
-            // Set 'bPreserveComptagBackingStoreOnSuspendDef' so that comptag
+            // Set 'bPreserveComptagBackingStoreOnSuspend' so that comptag
             // related handling can be done during state unload/load.
             //
-            pKernelMemorySystem->bPreserveComptagBackingStoreOnSuspend = NV_TRUE;
+            if (!pMemoryManager->bUseVirtualCopyOnSuspend)
+            {
+                pKernelMemorySystem->bPreserveComptagBackingStoreOnSuspend = NV_TRUE;
+            }
+
             status = RmPowerManagement(pGpu, NV_PM_ACTION_STANDBY);
             pGpu->setProperty(pGpu, PDB_PROP_GPU_GCOFF_STATE_ENTERING, NV_FALSE);
 
@@ -2324,11 +2469,12 @@ NV_STATUS NV_API_CALL rm_power_management(
     NV_STATUS rmStatus = NV_OK;
     void *fp;
     NvBool bTryAgain = NV_FALSE;
+    NvBool bConsoleDisabled = NV_FALSE;
 
     NV_ENTER_RM_RUNTIME(sp,fp);
-    threadStateInit(&threadState, THREAD_STATE_FLAGS_NONE);
+    threadStateInit(&threadState, THREAD_STATE_FLAGS_DEVICE_INIT);
 
-    NV_ASSERT_OK(os_flush_work_queue(pNv->queue));
+    NV_ASSERT_OK(os_flush_work_queue(pNv->queue, pmAction != NV_PM_ACTION_RESUME));
 
     // LOCK: acquire API lock
     if ((rmStatus = rmapiLockAcquire(API_LOCK_FLAGS_NONE, RM_LOCK_MODULES_DYN_POWER)) == NV_OK)
@@ -2342,38 +2488,78 @@ NV_STATUS NV_API_CALL rm_power_management(
                 // LOCK: acquire GPUs lock
                 if ((rmStatus = rmGpuLocksAcquire(GPUS_LOCK_FLAGS_NONE, RM_LOCK_MODULES_DYN_POWER)) == NV_OK)
                 {
-                    nv_priv_t *nvp = NV_GET_NV_PRIV(pNv);
-
-                    //
-                    // Before going to S3 or S4, remove idle holdoff which was
-                    // applied during gc6 exit.
-                    //
-                    if ((pmAction != NV_PM_ACTION_RESUME) &&
-                        (nvp->dynamic_power.b_idle_holdoff == NV_TRUE))
+                    if (pGpu->getProperty(pGpu, PDB_PROP_GPU_TEGRA_SOC_NVDISPLAY))
                     {
-                        nv_indicate_idle(pNv);
-                        RmCancelCallbackToRemoveIdleHoldoff(pGpu);
-                        nvp->dynamic_power.b_idle_holdoff = NV_FALSE;
-                    }
-
-                    //
-                    // Use GCx (GCOFF/GC6) power management if S0ix-based PM is
-                    // enabled and the request is for system suspend/resume.
-                    // Otherwise, use the existing mechanism.
-                    //
-                    if (nvp->s0ix_pm_enabled &&
-                        (pmAction == NV_PM_ACTION_STANDBY ||
-                        (pmAction == NV_PM_ACTION_RESUME &&
-                         !nvp->pm_state.InHibernate)))
-                    {
-                        rmStatus = RmGcxPowerManagement(pGpu,
-                                        pmAction == NV_PM_ACTION_STANDBY,
-                                        NV_FALSE, &bTryAgain);
+                        rmStatus = RmPowerManagementTegra(pGpu, pmAction);
 
+                        //
+                        // RmPowerManagementTegra() is most likely to fail due to
+                        // gpuStateUnload() failures deep in the RM's GPU power
+                        // management paths.  However, those paths make no
+                        // attempt to unwind in case of errors.  Rather, they
+                        // soldier on and simply report an error at the very end.
+                        // GPU software state meanwhile will indicate the GPU
+                        // has been suspended.
+                        //
+                        // Sadly, in case of an error during suspend/hibernate,
+                        // the only path forward here is to attempt to resume the
+                        // GPU, accepting that the odds of success will vary.
+                        //
+                        if (rmStatus != NV_OK && pmAction != NV_PM_ACTION_RESUME)
+                        {
+                            RmPowerManagementTegra(pGpu, NV_PM_ACTION_RESUME);
+                        }
                     }
                     else
                     {
-                        rmStatus = RmPowerManagement(pGpu, pmAction);
+                        //
+                        // For GPU driving console, disable console access here, to ensure no console
+                        // writes through BAR1 can interfere with physical RM's setup of BAR1
+                        //
+                        if (pNv->client_managed_console)
+                        {
+                            os_disable_console_access();
+                            bConsoleDisabled = NV_TRUE;
+                        }
+
+                        nv_priv_t *nvp = NV_GET_NV_PRIV(pNv);
+
+                        //
+                        // Before going to S3 or S4, remove idle holdoff which was
+                        // applied during gc6 exit.
+                        //
+                        if ((pmAction != NV_PM_ACTION_RESUME) &&
+                            (nvp->dynamic_power.b_idle_holdoff == NV_TRUE))
+                        {
+                            nv_indicate_idle(pNv);
+                            RmCancelCallbackToRemoveIdleHoldoff(pGpu);
+                            nvp->dynamic_power.b_idle_holdoff = NV_FALSE;
+                        }
+
+                        //
+                        // Use GCx (GCOFF/GC6) power management if S0ix-based PM is
+                        // enabled and the request is for system suspend/resume.
+                        // Otherwise, use the existing mechanism.
+                        //
+                        if (nvp->s0ix_pm_enabled &&
+                            (pmAction == NV_PM_ACTION_STANDBY ||
+                            (pmAction == NV_PM_ACTION_RESUME &&
+                             !nvp->pm_state.InHibernate)))
+                        {
+                            rmStatus = RmGcxPowerManagement(pGpu,
+                                            pmAction == NV_PM_ACTION_STANDBY,
+                                            NV_FALSE, &bTryAgain);
+
+                        }
+                        else
+                        {
+                            rmStatus = RmPowerManagement(pGpu, pmAction);
+                        }
+
+                        if (bConsoleDisabled)
+                        {
+                            os_enable_console_access();
+                        }
                     }
 
                     // UNLOCK: release GPUs lock
@@ -2386,7 +2572,7 @@ NV_STATUS NV_API_CALL rm_power_management(
         rmapiLockRelease();
     }
 
-    NV_ASSERT_OK(os_flush_work_queue(pNv->queue));
+    NV_ASSERT_OK(os_flush_work_queue(pNv->queue, pmAction != NV_PM_ACTION_RESUME));
 
     threadStateFree(&threadState, THREAD_STATE_FLAGS_NONE);
     NV_EXIT_RM_RUNTIME(sp,fp);
@@ -2450,6 +2636,11 @@ NV_STATUS NV_API_CALL rm_transition_dynamic_power(
     THREAD_STATE_NODE   threadState;
     void               *fp;
 
+    if (NV_IS_SOC_DISPLAY_DEVICE(nv))
+    {
+        return NV_OK;
+    }
+
     NV_ENTER_RM_RUNTIME(sp,fp);
 
     threadStateInit(&threadState, THREAD_STATE_FLAGS_NONE);
@@ -2536,6 +2727,38 @@ void RmHandleDisplayChange(
     }
 }
 
+/*!
+ * @brief RmGetDynamicBoostSupport() gets the notebook DB
+ * support. DB support is static information for vast majority
+ * of notebooks and hence we fetch it only once (during the first 
+ * call to this function). This way, the GPU remains in RTD3 state
+ * when procfs read is done.
+ *
+ *  @param[in]   nv    nv_state_t pointer.
+ */
+static const char* RmGetDynamicBoostSupport(
+    nv_state_t *pNv
+)
+{
+    nv_priv_t *pNvp = NV_GET_NV_PRIV(pNv);
+    const char* DbStatus = "?";
+
+    if (pNvp->db_supported == -1)
+    {
+        pNvp->db_supported = os_get_dynamic_boost_support(pNv);
+    }
+    
+    if (pNvp->db_supported == 0)
+    {
+        DbStatus = "Not Supported";
+    }
+    else if (pNvp->db_supported == 1)
+    { 
+        DbStatus = "Supported";
+    }
+    return DbStatus;
+}
+
 /*!
  * @brief: Function to query Dynamic Power Management
  *
@@ -2602,10 +2825,10 @@ static void RmQueueIdleSustainedWorkitem(
 
     if (!nvp->dynamic_power.b_idle_sustained_workitem_queued)
     {
-        status = osQueueWorkItemWithFlags(pGpu,
-                                          RmHandleIdleSustained,
-                                          NULL,
-                                          OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_SUBDEVICE);
+        status = osQueueWorkItem(pGpu,
+            RmHandleIdleSustained,
+            NULL,
+            OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_SUBDEVICE);
         if (status != NV_OK)
         {
             NV_PRINTF(LEVEL_WARNING,
@@ -2665,6 +2888,104 @@ RmInitS0ixPowerManagement(
     }
 }
 
+/*!
+ * @brief: Helper function to abstract the execution and return value verification
+ *         of PEX DSM methods, necessary for GC6 to work in Desktops.
+ *
+ * This function is capable to invoke three _DSM methods, as listed below:
+ *   0x0: _DSM for "Query supported functions"
+ *        (for more details, refer to section 9.1.1 of the ACPI 6.3 Specification).
+ *   0xA: _DSM for "Requesting D3cold Aux Power Limit"
+ *        (for more details, refer to section 4.6.10 of PCI Firmware Specification v3.3)
+ *   0xB: _DSM for "Adding PERST# Assertion Delay"
+ *        (for more details, refer to section 4.6.11 of PCI Firmware Specification v3.3)
+ *
+ * @param[in]   nv                 nv_state_t pointer.
+ * @param[in]   dsmSubFunc         subfunction to invoke.
+ * @param[in]   data               input data for dsmSubFunc.
+ *
+ * @return      TRUE if dsmSubFunc executed successfully and return value is as expected;
+ *              FALSE otherwise.
+ */
+static NvBool RmAcpiD3ColdDsm
+(
+    nv_state_t *nv,
+    NvU32       dsmSubFunc,
+    NvU32       data
+)
+{
+    NvU32 inData = data;
+
+    /*
+     * Third argument (i.e. acpiDsmRev) is sent as 4 because PEX_FUNC_AUXPOWERLIMIT and
+     * PEX_FUNC_PEXRST_DELAY sub-functions were introduced in PCI Firmware Specification 3.2
+     * and they are supported only for _DSM revision 4 onwards. But, RM still use _DSM
+     * revision 2 (see drivers/common/inc/pex.h). Hence, the need to hardcode 4 here.
+     */
+    if (nv_acpi_d3cold_dsm_for_upstream_port(nv, (NvU8 *)&PEX_DSM_GUID, 4, dsmSubFunc, &data) != NV_OK)
+    {
+        NV_PRINTF(LEVEL_NOTICE, "%s: PEX _DSM subfunction: 0x%X failed.\n", __FUNCTION__, dsmSubFunc);
+        return NV_FALSE;
+    }
+
+    switch (dsmSubFunc)
+    {
+        case NV_ACPI_ALL_FUNC_SUPPORT:
+            return ((data & NVBIT(PEX_FUNC_PEXRST_DELAY)) &&
+                    (data & NVBIT(PEX_FUNC_AUXPOWERLIMIT)));
+
+        case PEX_FUNC_PEXRST_DELAY:
+            return (inData == data);
+
+        case PEX_FUNC_AUXPOWERLIMIT:
+            return ((data == NV_AUX_POWER_REQUEST_STATUS_GRANTED_WITH_12V_POWER) ||
+                    (data == NV_AUX_POWER_REQUEST_STATUS_GRANTED_WITHOUT_12V_POWER));
+
+        default:
+            return NV_FALSE;
+    }
+
+    return NV_FALSE;
+}
+
+static NvBool RmConfigureUpstreamPortForRTD3(
+    nv_state_t *nv,
+    NvBool      bConfigureForGC6
+)
+{
+    nv_priv_t *nvp  = NV_GET_NV_PRIV(nv);
+    OBJGPU    *pGpu = NV_GET_NV_PRIV_PGPU(nv);
+
+    if (nvp->b_mobile_config_enabled)
+        return NV_TRUE;
+
+    if (!RmAcpiD3ColdDsm(nv, NV_ACPI_ALL_FUNC_SUPPORT, 0))
+    {
+        return NV_FALSE;
+    }
+
+    if (!RmAcpiD3ColdDsm(nv, PEX_FUNC_PEXRST_DELAY,
+                         bConfigureForGC6 ? pGpu->gc6State.GC6PerstDelay : DEFAULT_GCOFF_PEXRST_DELAY))
+    {
+        return NV_FALSE;
+    }
+
+    if (!RmAcpiD3ColdDsm(nv, PEX_FUNC_AUXPOWERLIMIT,
+                         bConfigureForGC6 ? pGpu->gc6State.GC6TotalBoardPower : 0))
+    {
+        if (bConfigureForGC6)
+            RmAcpiD3ColdDsm(nv, PEX_FUNC_PEXRST_DELAY, DEFAULT_GCOFF_PEXRST_DELAY);
+
+        return NV_FALSE;
+    }
+
+    NV_PRINTF(LEVEL_NOTICE,
+              "Aux Power and Pex delay settings %s successfully.\n",
+              bConfigureForGC6? "applied": "cleared");
+
+    return NV_TRUE;
+}
+
 void RmInitPowerManagement(
     nv_state_t *nv
 )
@@ -2675,10 +2996,20 @@ void RmInitPowerManagement(
         NvBool bGC6Support   = NV_FALSE;
         NvBool bGCOFFSupport = NV_FALSE;
         NvBool bRtd3Support  = RmCheckRtd3GcxSupport(nv, &bGC6Support, &bGCOFFSupport);
+        nv_priv_t *nvp       = NV_GET_NV_PRIV(nv);
+        OBJGPU *pGpu         = NV_GET_NV_PRIV_PGPU(nv);
 
         RmInitDeferredDynamicPowerManagement(nv, bRtd3Support);
         RmInitS0ixPowerManagement(nv, bRtd3Support, bGC6Support);
 
+        if (bRtd3Support && bGC6Support &&
+            (nvp->s0ix_pm_enabled || (nvp->dynamic_power.mode == NV_DYNAMIC_PM_FINE)))
+        {
+            nvp->gc6_upstream_port_configured = RmConfigureUpstreamPortForRTD3(nv, NV_TRUE);
+        }
+
+        nv->is_pm_unsupported = pGpu->getProperty(pGpu, PDB_PROP_GPU_POWER_MANAGEMENT_UNSUPPORTED);
+
         // UNLOCK: release GPUs lock
         rmGpuLocksRelease(GPUS_LOCK_FLAGS_NONE, NULL);
     }
@@ -2688,6 +3019,13 @@ void RmDestroyPowerManagement(
     nv_state_t *nv
 )
 {
+    nv_priv_t *nvp = NV_GET_NV_PRIV(nv);
+
+    if (nvp->gc6_upstream_port_configured)
+    {
+        RmConfigureUpstreamPortForRTD3(nv, NV_FALSE);
+    }
+
     RmDestroyDeferredDynamicPowerManagement(nv);
 }
 
@@ -2705,6 +3043,7 @@ void NV_API_CALL rm_get_power_info(
     const char       *pGcoffSupported = "?";
     const char       *pDynamicPowerStatus = "?";
     const char       *pS0ixStatus = "?";
+    const char       *pDbStatus = "?";
 
     NV_ENTER_RM_RUNTIME(sp,fp);
     threadStateInit(&threadState, THREAD_STATE_FLAGS_NONE);
@@ -2728,7 +3067,7 @@ void NV_API_CALL rm_get_power_info(
             pGc6Supported = RmGetGpuGcxSupport(pGpu, NV_TRUE);
             pGcoffSupported = RmGetGpuGcxSupport(pGpu, NV_FALSE);
             pS0ixStatus = pNvp->s0ix_pm_enabled ? "Enabled" : "Disabled";
-
+            pDbStatus = RmGetDynamicBoostSupport(pNv);
             // UNLOCK: release per device lock
             rmGpuGroupLockRelease(gpuMask, GPUS_LOCK_FLAGS_NONE);
         }
@@ -2745,5 +3084,6 @@ void NV_API_CALL rm_get_power_info(
     powerInfo->gc6_support = pGc6Supported;
     powerInfo->gcoff_support = pGcoffSupported;
     powerInfo->s0ix_status = pS0ixStatus;
+    powerInfo->db_support = pDbStatus;
 }
 
