@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1999-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1999-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -45,8 +45,8 @@
 #include <gpu/timer/objtmr.h>
 #include "gpu/bus/kern_bus.h"
 #include "nverror.h"
-#include <gpu/bif/kernel_bif.h>
 #include <nv-nb-regs.h>
+#include <gpu/bif/kernel_bif.h>
 #include <gpu/mem_mgr/mem_mgr.h>
 #include <gpu/mem_sys/kern_mem_sys.h>
 #include "kernel/gpu/intr/intr.h"
@@ -58,7 +58,7 @@
 #include <nv-firmware-chip-family-select.h>
 #include <gpu/gsp/kernel_gsp.h>
 #include "liblogdecode.h"
-#include <gpu/fsp/kern_fsp.h>
+#include  <gpu/gsp/kernel_gsp.h>
 
 #include <mem_mgr/virt_mem_mgr.h>
 #include <virtualization/kernel_vgpu_mgr.h>
@@ -141,20 +141,6 @@ typedef struct {
                                      (status).line = __LINE__; }
 
 
-//
-// GPU architectures support DMA addressing up to a certain address width,
-// above which all other bits in any given DMA address must not vary
-// (e.g., all 0). This value is the minimum of the DMA addressing
-// capabilities, in number of physical address bits, for all supported
-// GPU architectures.
-//
-#define NV_GPU_MIN_SUPPORTED_DMA_ADDR_WIDTH                36
-
-//
-// All GPU architectures with GSP support at least 47 physical address bits.
-//
-#define NV_GSP_GPU_MIN_SUPPORTED_DMA_ADDR_WIDTH            47
-
 static inline NvU64 nv_encode_pci_info(nv_pci_info_t *pci_info)
 {
     return gpuEncodeDomainBusDevice(pci_info->domain, pci_info->bus, pci_info->slot);
@@ -165,6 +151,18 @@ static inline NvU32 nv_generate_id_from_pci_info(nv_pci_info_t *pci_info)
     return gpuGenerate32BitId(pci_info->domain, pci_info->bus, pci_info->slot);
 }
 
+static void nv_set_probed_gpu_flags(nv_state_t *nv)
+{
+    NvU32 flags = 0;
+
+    if (NV_IS_SOC_DISPLAY_DEVICE(nv))
+    {
+        flags |= DRF_DEF(0000, _CTRL_GPU_PROBED_ID_FLAGS, _SOC_DISPLAY, _TRUE);
+    }
+
+    gpumgrSetProbedFlags(nv->gpu_id, flags);
+}
+
 static inline void nv_os_map_kernel_space(nv_state_t *nv, nv_aperture_t *aperture)
 {
     NV_ASSERT(aperture->map == NULL);
@@ -271,6 +269,8 @@ NV_STATUS osRmInitRm(void)
         return status;
     }
 
+    rm_check_s0ix_regkey_and_platform_support();
+
     // Setup any ThreadState defaults
     threadStateInitSetupFlags(THREAD_STATE_SETUP_FLAGS_ENABLED |
                               THREAD_STATE_SETUP_FLAGS_TIMEOUT_ENABLED |
@@ -364,7 +364,8 @@ osHandleGpuLost
         nvErrorLog_va((void *)pGpu, ROBUST_CHANNEL_GPU_HAS_FALLEN_OFF_THE_BUS,
                       "GPU has fallen off the bus.");
 
-        gpuNotifySubDeviceEvent(pGpu, NV2080_NOTIFIERS_RC_ERROR, NULL, 0, ROBUST_CHANNEL_GPU_HAS_FALLEN_OFF_THE_BUS, 0);
+        gpuNotifySubDeviceEvent(pGpu, NV2080_NOTIFIERS_GPU_UNAVAILABLE, NULL,
+                                0, ROBUST_CHANNEL_GPU_HAS_FALLEN_OFF_THE_BUS, 0);
 
         NV_DEV_PRINTF(NV_DBG_ERRORS, nv, "GPU has fallen off the bus.\n");
 
@@ -377,35 +378,22 @@ osHandleGpuLost
 
         gpuSetDisconnectedProperties(pGpu);
 
-        // Trigger the OS's PCI recovery mechanism
-        if (nv_pci_trigger_recovery(nv) != NV_OK)
+        if (IS_GSP_CLIENT(pGpu))
         {
-            //
-            // Initiate a crash dump immediately, since the OS doesn't appear
-            // to have a mechanism wired up for attempted recovery.
-            //
-            (void) RmLogGpuCrash(pGpu);
-        }
-        else
-        {
-            //
-            // Make the SW state stick around until the recovery can start, but
-            // don't change the PDB property: this is only used to report to
-            // clients whether or not persistence mode is enabled, and we'll
-            // need it after the recovery callbacks to restore the correct
-            // persistence mode for the GPU.
-            //
-            osModifyGpuSwStatePersistence(pGpu->pOsGpuInfo, NV_TRUE);
+            // Notify all channels of the error so that UVM can fail gracefully
+            KernelGsp *pKernelGsp = GPU_GET_KERNEL_GSP(pGpu);
+            kgspRcAndNotifyAllChannels(pGpu, pKernelGsp, ROBUST_CHANNEL_GPU_HAS_FALLEN_OFF_THE_BUS, NV_FALSE);
         }
 
-        OBJSYS *pSys = SYS_GET_INSTANCE();
-        OBJCL  *pCl  = SYS_GET_CL(pSys);
+        // Initiate a crash dump immediately.
+        RmLogGpuCrash(pGpu);
+
         // Set SURPRISE_REMOVAL flag for eGPU to help in device removal.
-        if ((pCl != NULL) &&
-            pCl->getProperty(pCl, PDB_PROP_CL_IS_EXTERNAL_GPU))
+        if (pGpu->getProperty(pGpu, PDB_PROP_GPU_IS_EXTERNAL_GPU))
         {
             nv->flags |= NV_FLAG_IN_SURPRISE_REMOVAL;
         }
+
         DBG_BREAKPOINT();
     }
 
@@ -567,10 +555,11 @@ RmInitGpuInfoWithRmApi
 
     portMemSet(pGpuInfoParams, 0, sizeof(*pGpuInfoParams));
 
-    pGpuInfoParams->gpuInfoListSize = 3;
+    pGpuInfoParams->gpuInfoListSize = 4;
     pGpuInfoParams->gpuInfoList[0].index = NV2080_CTRL_GPU_INFO_INDEX_4K_PAGE_ISOLATION_REQUIRED;
     pGpuInfoParams->gpuInfoList[1].index = NV2080_CTRL_GPU_INFO_INDEX_MOBILE_CONFIG_ENABLED;
     pGpuInfoParams->gpuInfoList[2].index = NV2080_CTRL_GPU_INFO_INDEX_DMABUF_CAPABILITY;
+    pGpuInfoParams->gpuInfoList[3].index = NV2080_CTRL_GPU_INFO_INDEX_COHERENT_GPU_MEMORY_MODE;
 
     status = pRmApi->Control(pRmApi, nv->rmapi.hClient,
                              nv->rmapi.hSubDevice,
@@ -590,7 +579,30 @@ RmInitGpuInfoWithRmApi
              NV2080_CTRL_GPU_INFO_INDEX_DMABUF_CAPABILITY_YES);
     }
 
-    nv->coherent = pGpu->getProperty(pGpu, PDB_PROP_GPU_COHERENT_CPU_MAPPING);
+    nv->coherent =
+        (pGpu->getProperty(pGpu, PDB_PROP_GPU_COHERENT_CPU_MAPPING) ||
+         pGpu->getProperty(pGpu, PDB_PROP_GPU_ZERO_FB));
+
+    //
+    // If coherent GPU memory mode is NONE, then GPU memory has struct page
+    // on coherent platforms and no struct page on non-coherent ones.
+    // If the mode is enabled and _NUMA, then GPU memory has struct page.
+    // If the mode is enabled and _DRIVER, then GPU memory doesn't have struct page.
+    // Tegra iGPU also falls under struct page category, although
+    // COHERENT_GPU_MEMORY_MODE doesn't apply there.
+    //
+    if (pGpuInfoParams->gpuInfoList[3].data ==
+        NV2080_CTRL_GPU_INFO_INDEX_COHERENT_GPU_MEMORY_MODE_NONE)
+    {
+        nv->mem_has_struct_page = nv->coherent;
+    }
+    else
+    {
+        // If mode is not _NONE, we're already on a PDB_PROP_GPU_COHERENT_CPU_MAPPING platform.
+        nv->mem_has_struct_page = pGpu->getProperty(pGpu, PDB_PROP_GPU_ZERO_FB) ||
+                                  (pGpuInfoParams->gpuInfoList[3].data ==
+                                   NV2080_CTRL_GPU_INFO_INDEX_COHERENT_GPU_MEMORY_MODE_NUMA);
+    }
 
     portMemFree(pGpuInfoParams);
 
@@ -723,6 +735,13 @@ osInitNvMapping(
 
         gpuAttachArg->iovaspaceId     = nv->iovaspace_id;
         gpuAttachArg->cpuNumaNodeId   = nv->cpu_numa_node_id;
+
+        if (nv->iovaspace_id != NV_IOVA_DOMAIN_NONE)
+        {
+            // Default - PCIe GPUs are connected via NISO IOMMU
+            nv->iommus.iso_iommu_present  = NV_FALSE;
+            nv->iommus.niso_iommu_present = NV_TRUE;
+        }
     }
 
     //
@@ -757,15 +776,6 @@ osInitNvMapping(
 
     pGpu->busInfo.IntLine = nv->interrupt_line;
 
-    //
-    // Set the DMA address size as soon as we have the HAL to call to
-    // determine the precise number of physical address bits supported
-    // by the architecture. DMA allocations should not be made before
-    // this point.
-    //
-    nv_set_dma_address_size(nv, gpuGetPhysAddrWidth_HAL(pGpu, ADDR_SYSMEM));
-
-    pGpu->dmaStartAddress = (RmPhysAddr)nv_get_dma_start_address(nv);
     if (nv->fb != NULL)
     {
         pGpu->registerAccess.gpuFbAddr = (GPUHWREG*) nv->fb->map;
@@ -812,7 +822,12 @@ osInitNvMapping(
         MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
 
         memmgrSetPmaForcePersistence(pMemoryManager, NV_TRUE);
-        pKernelMemorySystem->bPreserveComptagBackingStoreOnSuspend = NV_TRUE;
+
+        // Monolithic uses Fifolite channel which doesn't support Virtual mode
+        if (!pMemoryManager->bUseVirtualCopyOnSuspend)
+        {
+            pKernelMemorySystem->bPreserveComptagBackingStoreOnSuspend = NV_TRUE;
+        }
 
         nv->preserve_vidmem_allocations = NV_TRUE;
     }
@@ -827,6 +842,16 @@ osInitNvMapping(
     {
         pGpu->setProperty(pGpu, PDB_PROP_GPU_DISP_PB_REQUIRES_SMMU_BYPASS, NV_TRUE);
     }
+
+    if (pGpu->getProperty(pGpu, PDB_PROP_GPU_TRIGGER_PCIE_FLR))
+    {
+        nv->flags |= NV_FLAG_TRIGGER_FLR;
+    }
+
+    if (pGpu->getProperty(pGpu, PDB_PROP_GPU_IS_SOC_SDM))
+    {
+        nv->flags |= NV_FLAG_HAS_CONSOLE_IN_SYSMEM_CARVEOUT;
+    }
 }
 
 void osInitScalabilityOptions
@@ -870,6 +895,25 @@ osTeardownScalability(
     return clTeardownPcie(pGpu, pCl);
 }
 
+static NV_STATUS
+RmInitNvHal(
+    nv_state_t *nv,
+    NvU32 deviceReference,
+    UNIX_STATUS *status
+)
+{
+    OBJGPU             *pGpu      = gpumgrGetGpu(deviceReference);
+    nv_priv_t          *nvp       = NV_GET_NV_PRIV(nv);
+
+    PORT_UNREFERENCED_VARIABLE(pGpu);
+
+    nvp->flags |= NV_INIT_FLAG_HAL;
+
+    nvp->flags |= NV_INIT_FLAG_HAL_COMPONENTS;
+
+    return NV_OK;
+}
+
 #define NV_DBG_PRINT_VGA_STATUS(nv, src)    \
     NV_DEV_PRINTF(NV_DBG_SETUP, nv, "%s reports GPU is %s VGA\n", \
               src, NV_PRIMARY_VGA(nv) ? "primary" : "not primary");
@@ -911,7 +955,11 @@ static void
 RmDeterminePrimaryDevice(OBJGPU *pGpu)
 {
     nv_state_t *nv = NV_GET_NV_STATE(pGpu);
-    NvBool bFrameBufferConsoleDevice = NV_FALSE;
+
+    if (pGpu->getProperty(pGpu, PDB_PROP_GPU_TEGRA_SOC_NVDISPLAY))
+    {
+        return;
+    }
 
     // Skip updating nv->primary_vga while RM is recovering after GPU reset
     if (nv->flags & NV_FLAG_IN_RECOVERY)
@@ -944,15 +992,15 @@ RmDeterminePrimaryDevice(OBJGPU *pGpu)
 
     //
     // If GPU is driving any frame buffer console(vesafb, efifb etc)
-    // mark the GPU as Primary.
+    // mark the console as client driven and GPU as Primary.
     //
-    bFrameBufferConsoleDevice = rm_get_uefi_console_status(nv);
+    nv->client_managed_console = rm_get_uefi_console_status(nv);
 
     NV_DEV_PRINTF(NV_DBG_SETUP, nv, " is %s UEFI console device\n",
-              bFrameBufferConsoleDevice ? "primary" : "not primary");
+              nv->client_managed_console ? "primary" : "not primary");
 
     pGpu->setProperty(pGpu, PDB_PROP_GPU_PRIMARY_DEVICE,
-                      (bFrameBufferConsoleDevice || !!nv->primary_vga));
+                      (nv->client_managed_console || !!nv->primary_vga));
 }
 
 static void
@@ -975,6 +1023,21 @@ RmSetConsolePreservationParams(OBJGPU *pGpu)
         return;
     }
 
+    //
+    // If this is the zero FB SOC GPU with a console in system carveout, then
+    // the console memory is not mapped onto the BAR. The kernel can directly
+    // access the console memory, while the GPU accesses system carveout memory
+    // through the SMMU.
+    // The carveout is a reserved region of system memory that doesn’t get used
+    // in standard memory allocation.
+    //
+    // In this scenario, you don’t need to reserve console memory or BAR mapping.
+    //
+    if (NV_HAS_CONSOLE_IN_SYSMEM_CARVEOUT(nv))
+    {
+        return;
+    }
+
     //
     // Check the OS layer for any video memory used by a console
     // driver that should be reserved.
@@ -1058,6 +1121,52 @@ RmTeardownDeviceDma(
     }
 }
 
+static void
+RmEnableDeviceClks(nv_state_t *nv)
+{
+    NvU32 freqKHz = 0;
+    NvU32 i;
+    NV_STATUS ret;
+
+    for (i = TEGRASOC_WHICH_CLK_GPU_FIRST; i <= TEGRASOC_WHICH_CLK_GPU_LAST; i++)
+    {
+        ret = nv_get_max_freq(nv, i, &freqKHz);
+        if (ret != NV_OK)
+        {
+            NV_PRINTF(LEVEL_INFO, "NVRM: Max Freq fetch failed for Clk:%d\n", i);
+            continue;
+        }
+
+        ret = nv_enable_clk(nv, i);
+        if (ret != NV_OK)
+        {
+            NV_PRINTF(LEVEL_INFO, "NVRM: Clk prepare enable failed for Clk:%d\n", i);
+            continue;
+        }
+
+        ret = nv_set_freq(nv, i, freqKHz);
+        if (ret != NV_OK)
+        {
+            NV_PRINTF(LEVEL_INFO, "NVRM: Set Freq failed for Clk:%d\n", i);
+        }
+        else
+        {
+            NV_PRINTF(LEVEL_INFO, "NVRM: Set Freq:%d for Clk:%d\n", freqKHz, i);
+        }
+    }
+}
+
+static void
+RmDisableDeviceClks(nv_state_t *nv)
+{
+    NvU32 i;
+
+    for (i = TEGRASOC_WHICH_CLK_GPU_FIRST; i <= TEGRASOC_WHICH_CLK_GPU_LAST; i++)
+    {
+        nv_disable_clk(nv, i);
+    }
+}
+
 static void
 RmInitNvDevice(
     NvU32 deviceReference,
@@ -1088,7 +1197,7 @@ RmInitNvDevice(
     // Configure eGPU setting
     if (RmCheckForExternalGpu(pGpu, pCl))
     {
-        pCl->setProperty(pCl, PDB_PROP_CL_IS_EXTERNAL_GPU, NV_TRUE);
+        pGpu->setProperty(pGpu, PDB_PROP_GPU_IS_EXTERNAL_GPU, NV_TRUE);
         nv->is_external_gpu = NV_TRUE;
     }
     status->rmStatus = gpumgrStateInitGpu(pGpu);
@@ -1279,13 +1388,11 @@ RmSetupRegisters(
 
     if (nv->fb != NULL)
     {
-        NV_DEV_PRINTF(NV_DBG_SETUP, nv, "   fb        looks like: 0x%" NvU64_fmtx " 0x%" NvU64_fmtx,
+        NV_DEV_PRINTF(NV_DBG_SETUP, nv, "   fb        looks like: 0x%" NvU64_fmtx " 0x%" NvU64_fmtx "\n",
                 nv->fb->cpu_address, nv->fb->size);
     }
 
-    {
-        nv_os_map_kernel_space(nv, nv->regs);
-    }
+    nv_os_map_kernel_space(nv, nv->regs);
 
     if (nv->regs->map == NULL)
     {
@@ -1336,13 +1443,17 @@ NvBool RmInitPrivateState(
 {
     nv_priv_t *nvp;
     NvU32 gpuId;
+    NvU32 socChipId0 = 0;
     NvU32 pmc_boot_0 = 0;
     NvU32 pmc_boot_1 = 0;
     NvU32 pmc_boot_42 = 0;
+    NvU32 dmaAddrWidth = 0;
 
     NV_SET_NV_PRIV(pNv, NULL);
 
-    if (!NV_IS_SOC_DISPLAY_DEVICE(pNv) && !NV_IS_SOC_IGPU_DEVICE(pNv))
+    if (NV_IS_SOC_DISPLAY_DEVICE(pNv))
+        socChipId0 = pNv->disp_sw_soc_chip_id;
+    else
     {
         pNv->regs->map_u = os_map_kernel_space(pNv->regs->cpu_address,
                                                os_page_size,
@@ -1381,23 +1492,41 @@ NvBool RmInitPrivateState(
 
     pNv->gpu_id = gpuId;
 
+    nv_set_probed_gpu_flags(pNv);
+
     pNv->iovaspace_id = nv_requires_dma_remap(pNv) ? gpuId :
                                                      NV_IOVA_DOMAIN_NONE;
     pNv->cpu_numa_node_id = NV0000_CTRL_NO_NUMA_NODE;
 
-    kvgpumgrAttachGpu(pNv->gpu_id);
+    // Get the GpuArch instance for this architecture to determine the DMA address width.
+    GpuArch *pGpuArch = gpumgrGetGpuArch(pmc_boot_42, socChipId0, TEGRA_CHIP_TYPE_DEFAULT);
+    if (pGpuArch == NULL)
+    {
+        NV_PRINTF(LEVEL_ERROR, "failed to get GpuArch for 0x%x/0x%x.\n",
+                  pmc_boot_42, socChipId0);
+        gpumgrUnregisterGpuId(gpuId);
+        os_free_mem(nvp);
+        return NV_FALSE;
+    }
 
-    //
-    // Set up a reasonable default DMA address size, based on the minimum
-    // possible on currently supported GPUs.
-    //
-    nv_set_dma_address_size(pNv, NV_GPU_MIN_SUPPORTED_DMA_ADDR_WIDTH);
+    dmaAddrWidth = gpuarchGetDmaAddrWidth(pGpuArch);
+    if (dmaAddrWidth == 0)
+    {
+        dmaAddrWidth = gpuarchGetSystemPhysAddrWidth(pGpuArch);
+    }
+    nv_set_dma_address_size(pNv, dmaAddrWidth);
+
+    pNv->is_tegra_pci_igpu = !NV_IS_SOC_DISPLAY_DEVICE(pNv) && gpuarchIsZeroFb(pGpuArch);
+    pNv->supports_tegra_igpu_rg = pNv->is_tegra_pci_igpu && gpuarchSupportsIgpuRg(pGpuArch);
+
+    kvgpumgrAttachGpu(pNv->gpu_id);
 
     os_mem_set(nvp, 0, sizeof(*nvp));
     nvp->status = NV_ERR_INVALID_STATE;
     nvp->pmc_boot_0 = pmc_boot_0;
     nvp->pmc_boot_1 = pmc_boot_1;
     nvp->pmc_boot_42 = pmc_boot_42;
+    nvp->db_supported = -1;
     NV_SET_NV_PRIV(pNv, nvp);
 
     return NV_TRUE;
@@ -1416,6 +1545,8 @@ void RmClearPrivateState(
     nv_dynamic_power_t dynamicPowerCopy;
     NvU32 x = 0;
     NvU32 pmc_boot_0, pmc_boot_1, pmc_boot_42;
+    NvBool pr3_acpi_method_present = 0;
+    int db_supported;
 
     //
     // Do not clear private state after GPU resets, it is used while
@@ -1435,6 +1566,8 @@ void RmClearPrivateState(
     pmc_boot_0 = nvp->pmc_boot_0;
     pmc_boot_1 = nvp->pmc_boot_1;
     pmc_boot_42 = nvp->pmc_boot_42;
+    pr3_acpi_method_present = nvp->pr3_acpi_method_present;
+    db_supported = nvp->db_supported;
 
     for (x = 0; x < MAX_I2C_ADAPTERS; x++)
     {
@@ -1451,6 +1584,8 @@ void RmClearPrivateState(
     nvp->pmc_boot_0 = pmc_boot_0;
     nvp->pmc_boot_1 = pmc_boot_1;
     nvp->pmc_boot_42 = pmc_boot_42;
+    nvp->pr3_acpi_method_present = pr3_acpi_method_present;
+    nvp->db_supported = db_supported;
 
     for (x = 0; x < MAX_I2C_ADAPTERS; x++)
     {
@@ -1663,7 +1798,7 @@ static NV_STATUS RmFetchGspRmImages
 {
     nv_firmware_chip_family_t chipFamily;
     nv_priv_t *nvp = NV_GET_NV_PRIV(nv);
-    NvU32 gpuArch = (gpuGetArchitectureFromPmcBoot42(nvp->pmc_boot_42) <<
+    NvU32 gpuArch = (decodePmcBoot42Architecture(nvp->pmc_boot_42) <<
                      GPU_ARCH_SHIFT);
     NvU32 gpuImpl = DRF_VAL(_PMC, _BOOT_42, _IMPLEMENTATION, nvp->pmc_boot_42);
 
@@ -1706,6 +1841,22 @@ static NV_STATUS RmFetchGspRmImages
     return NV_OK;
 }
 
+static void _checkP2pChipsetSupport(
+    nv_state_t *nv
+)
+{
+    OBJSYS *pSys = SYS_GET_INSTANCE();
+    OBJCL  *pCl  = SYS_GET_CL(pSys);
+
+    // Grace and Ampere Computing chipsets are incapable of PCIe P2P
+    if ((pCl->Chipset == CS_NVIDIA_TH500) ||
+        (pCl->Chipset == CS_AMPERE_ALTRA) ||
+        (pCl->Chipset == CS_AMPERE_AMPEREONE160))
+    {
+        nv->flags |= NV_FLAG_PCI_P2P_UNSUPPORTED_CHIPSET;
+    }
+}
+
 NvBool RmInitAdapter(
     nv_state_t *nv
 )
@@ -1728,6 +1879,7 @@ NvBool RmInitAdapter(
     NV_DEV_PRINTF(NV_DBG_SETUP, nv, "RmInitAdapter\n");
 
     nv->flags &= ~NV_FLAG_PASSTHRU;
+    nv->flags &= ~NV_FLAG_PCI_P2P_UNSUPPORTED_CHIPSET;
 
     RmSetupRegisters(nv, &status);
     if (! RM_INIT_SUCCESS(status.initStatus) )
@@ -1754,13 +1906,18 @@ NvBool RmInitAdapter(
     //
     if (nv->request_firmware)
     {
-        nv_set_dma_address_size(nv, NV_GSP_GPU_MIN_SUPPORTED_DMA_ADDR_WIDTH);
-
-        status.rmStatus = RmFetchGspRmImages(nv, &gspFw, &gspFwHandle, &gspFwLogHandle);
-        if (status.rmStatus != NV_OK)
+        if (!NV_IS_SOC_DISPLAY_DEVICE(nv))
         {
-            RM_SET_ERROR(status, RM_INIT_FIRMWARE_FETCH_FAILED);
-            goto shutdown;
+            status.rmStatus = RmFetchGspRmImages(nv, &gspFw, &gspFwHandle, &gspFwLogHandle);
+            if (status.rmStatus != NV_OK)
+            {
+                RM_SET_ERROR(status, RM_INIT_FIRMWARE_FETCH_FAILED);
+                goto shutdown;
+            }
+        }
+        else
+        {
+            nv->request_fw_client_rm = NV_TRUE;
         }
     }
 
@@ -1808,13 +1965,13 @@ NvBool RmInitAdapter(
         goto shutdown;
     }
 
-    KernelFsp *pKernelFsp = GPU_GET_KERNEL_FSP(pGpu);
-    if ((pKernelFsp != NULL) && !IS_GSP_CLIENT(pGpu) && !IS_VIRTUAL(pGpu))
+    // Boot GSP-RM proxy through COT command either via FSP or SEC2
+    if (!IS_GSP_CLIENT(pGpu) && !IS_VIRTUAL(pGpu))
     {
-        status.rmStatus = kfspPrepareAndSendBootCommands_HAL(pGpu, pKernelFsp);
+        status.rmStatus = gpuBootGspRmProxy(pGpu);
         if (status.rmStatus != NV_OK)
         {
-            NV_PRINTF(LEVEL_ERROR, "FSP boot command failed.\n");
+            NV_PRINTF(LEVEL_ERROR, "GSP-RM proxy boot command failed.\n");
             RM_SET_ERROR(status, RM_INIT_FIRMWARE_INIT_FAILED);
             goto shutdown;
         }
@@ -1830,12 +1987,19 @@ NvBool RmInitAdapter(
     // For GPU driving console, disable console access here, to ensure no console
     // writes through BAR1 can interfere with physical RM's setup of BAR1
     //
-    if (rm_get_uefi_console_status(nv))
+    if (nv->client_managed_console)
     {
         os_disable_console_access();
         consoleDisabled = NV_TRUE;
     }
 
+    // This needs to run before GSP-RM is booted, or else it will timeout
+    if (pGpu->getProperty(pGpu, PDB_PROP_GPU_CLKS_IN_TEGRA_SOC))
+    {
+        NV_PRINTF(LEVEL_INFO, "Enable Clocks to Max\n");
+        RmEnableDeviceClks(nv);
+    }
+
     //
     // If GSP fw RM support is enabled then start the GSP microcode
     // (including the task running the full instance of the RM) and
@@ -1852,6 +2016,16 @@ NvBool RmInitAdapter(
             goto shutdown;
         }
     }
+    else if (IS_DCE_CLIENT(pGpu))
+    {
+        status.rmStatus = dceclientDceRmInit(pGpu, GPU_GET_DCECLIENTRM(pGpu), NV_TRUE);
+        if (status.rmStatus != NV_OK)
+        {
+            NV_PRINTF(LEVEL_ERROR, "Cannot initialize DCE firmware RM\n");
+            RM_SET_ERROR(status, RM_INIT_FIRMWARE_INIT_FAILED);
+            goto shutdown;
+        }
+    }
     else if (nv->request_fw_client_rm)
     {
         // We were expecting to enable GSP-RM but something went wrong.
@@ -1875,6 +2049,14 @@ NvBool RmInitAdapter(
     if (IS_PASSTHRU(pGpu))
         nv->flags |= NV_FLAG_PASSTHRU;
 
+    RmInitNvHal(nv, devicereference, &status);
+    if (!RM_INIT_SUCCESS(status.initStatus))
+    {
+        NV_PRINTF(LEVEL_ERROR,
+                  "RmInitNvHal() failed, bailing out of RmInitAdapter!\n");
+        goto shutdown;
+    }
+
     status.rmStatus = RmInitX86Emu(pGpu);
     if (status.rmStatus != NV_OK)
     {
@@ -1884,8 +2066,10 @@ NvBool RmInitAdapter(
         goto shutdown;
     }
 
+
     initVendorSpecificRegistry(pGpu, nv->pci_info.device_id);
-    if (!IS_VIRTUAL(pGpu))
+
+    if (!IS_VIRTUAL(pGpu) && !NV_IS_SOC_DISPLAY_DEVICE(nv))
     {
         initNbsiTable(pGpu);
     }
@@ -2034,7 +2218,7 @@ NvBool RmInitAdapter(
 
     RmInitPowerManagement(nv);
 
-    if (!NV_IS_SOC_DISPLAY_DEVICE(nv) && !NV_IS_SOC_IGPU_DEVICE(nv))
+    if (!NV_IS_SOC_DISPLAY_DEVICE(nv))
     {
         status.rmStatus = RmRegisterGpudb(pGpu);
         if (status.rmStatus != NV_OK)
@@ -2064,6 +2248,8 @@ NvBool RmInitAdapter(
         }
     }
 
+    _checkP2pChipsetSupport(nv);
+
     NV_DEV_PRINTF(NV_DBG_SETUP, nv, "RmInitAdapter succeeded!\n");
 
     retVal = NV_TRUE;
@@ -2133,8 +2319,6 @@ void RmShutdownAdapter(
 
                 freeNbsiTable(pGpu);
 
-                gpuFreeEventHandle(pGpu);
-
                 OBJCL          *pCl  = SYS_GET_CL(pSys);
                 OBJOS          *pOS  = GPU_GET_OS(pGpu);
                 if (pCl != NULL)
@@ -2166,8 +2350,23 @@ void RmShutdownAdapter(
                     NV_ASSERT(rmStatus == NV_OK);
                 }
 
+                if (IS_DCE_CLIENT(pGpu))
+                {
+                    rmStatus = dceclientDceRmInit(pGpu, GPU_GET_DCECLIENTRM(pGpu), NV_FALSE);
+                    if (rmStatus != NV_OK)
+                    {
+                        NV_PRINTF(LEVEL_ERROR, "DCE firmware RM Shutdown failure\n");
+                    }
+                }
+
                 os_enable_console_access();
 
+                if (pGpu->getProperty(pGpu, PDB_PROP_GPU_CLKS_IN_TEGRA_SOC))
+                {
+                    NV_PRINTF(LEVEL_INFO, "Disable Clocks\n");
+                    RmDisableDeviceClks(nv);
+                }
+
                 //if (nvp->flags & NV_INIT_FLAG_HAL)
                 //  destroyHal(pDev);
 
@@ -2356,6 +2555,9 @@ static void initVendorSpecificRegistry(
     if (!pGpu)
         return;
 
+    if (pGpu->bIsSOC)
+        return;
+
     rmStatus = GPU_BUS_CFG_RD32(pGpu,
             NV_CONFIG_PCI_NV_11, &subsystem_id);
     if (rmStatus != NV_OK)
