@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -78,7 +78,7 @@
 #define MC_ENGINE_IDX_NVENC                         38
 #define MC_ENGINE_IDX_NVENC1                        39
 #define MC_ENGINE_IDX_NVENC2                        40
-#define MC_ENGINE_IDX_RESERVED41                    41
+#define MC_ENGINE_IDX_NVENC3                        41
 #define MC_ENGINE_IDX_C2C                           42
 #define MC_ENGINE_IDX_LTC                           43
 #define MC_ENGINE_IDX_FBHUB                         44
@@ -152,12 +152,18 @@
 #define MC_ENGINE_IDX_PXUC                          168
 #define MC_ENGINE_IDX_SYSLTC                        169
 #define MC_ENGINE_IDX_LRCC                          170
-#define MC_ENGINE_IDX_RESERVED171                   171
-#define MC_ENGINE_IDX_RESERVED172                   172
-#define MC_ENGINE_IDX_RESERVED173                   173
-#define MC_ENGINE_IDX_RESERVED174                   174
+#define MC_ENGINE_IDX_GSPLITE                       171
+#define MC_ENGINE_IDX_GSPLITE0                      MC_ENGINE_IDX_GSPLITE
+#define MC_ENGINE_IDX_GSPLITE1                      172
+#define MC_ENGINE_IDX_GSPLITE2                      173
+#define MC_ENGINE_IDX_GSPLITE3                      174
+#define MC_ENGINE_IDX_GSPLITE_MAX                   MC_ENGINE_IDX_GSPLITE3
+#define MC_ENGINE_IDX_DPAUX                         175
+#define MC_ENGINE_IDX_GPIO                          176
+#define MC_ENGINE_IDX_DISP_LOW                      177
+#define MC_ENGINE_IDX_HFRP                          178
 // This must be kept as the max bit if we need to add more engines
-#define MC_ENGINE_IDX_MAX                           175
+#define MC_ENGINE_IDX_MAX                           179
 
 // Index GR reference
 #define MC_ENGINE_IDX_GRn(x)            (MC_ENGINE_IDX_GR0 + (x))
@@ -184,6 +190,9 @@
 // Index OFA reference
 #define MC_ENGINE_IDX_OFA(x)            (MC_ENGINE_IDX_OFA0 + (x))
 
+// Index GSPLITE reference
+#define MC_ENGINE_IDX_GSPLITEn(x)       (MC_ENGINE_IDX_GSPLITE + (x))
+
 MAKE_BITVECTOR(MC_ENGINE_BITVECTOR, MC_ENGINE_IDX_MAX);
 typedef MC_ENGINE_BITVECTOR *PMC_ENGINE_BITVECTOR;
 
