@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -246,7 +246,7 @@ NvU32 threadStateGetSetupFlags(void)
 static void _threadStateSetNextCpuYieldTime(THREAD_STATE_NODE *pThreadNode)
 {
     NvU64 timeInNs;
-    osGetCurrentTick(&timeInNs);
+    timeInNs = osGetMonotonicTimeNs();
 
     pThreadNode->timeout.nextCpuYieldTime = timeInNs +
         (TIMEOUT_DEFAULT_OS_RESCHEDULE_INTERVAL_SECS) * 1000000 * 1000;
@@ -261,7 +261,7 @@ void threadStateYieldCpuIfNecessary(OBJGPU *pGpu, NvBool bQuiet)
     rmStatus = threadStateGetCurrent(&pThreadNode, pGpu);
     if ((rmStatus == NV_OK) && pThreadNode )
     {
-        osGetCurrentTick(&timeInNs);
+        timeInNs = osGetMonotonicTimeNs();
         if (timeInNs >= pThreadNode->timeout.nextCpuYieldTime)
         {
             if (NV_OK == osSchedule())
@@ -306,11 +306,11 @@ static NV_STATUS _threadNodeInitTime(THREAD_STATE_NODE *pThreadNode)
         // Note that MODS does not have interrupt timeout requirements and there are
         // existing code paths that violates the timeout
         //
-        computeTimeoutMsecs = 500;
-        nonComputeTimeoutMsecs = 500;
+        computeTimeoutMsecs = TIMEOUT_DPC_ISR_INTERVAL_MS;
+        nonComputeTimeoutMsecs = TIMEOUT_DPC_ISR_INTERVAL_MS;
     }
 
-    osGetCurrentTick(&timeInNs);
+    timeInNs = osGetMonotonicTimeNs();
 
     if (firstInit)
     {
@@ -327,23 +327,22 @@ static NV_STATUS _threadNodeInitTime(THREAD_STATE_NODE *pThreadNode)
         computeTimeoutMsecs = pThreadNode->timeout.overrideTimeoutMsecs;
     }
 
-    NvBool deviceInit = ((pThreadNode->flags & THREAD_STATE_FLAGS_DEVICE_INIT) != 0);
-    if (deviceInit && hypervisorIsType(OS_HYPERVISOR_HYPERV))
+    if ((pThreadNode->flags & THREAD_STATE_FLAGS_DEVICE_INIT) != 0)
     {
         //
-        // Hyper-V intercepts MMIO accesses to the GPU adding a lot of extra
-        // latency. This causes RM device init that is very heavy in MMIO
-        // accesses to take much longer than on baremetal. To WAR this, use a
-        // long 60 secs timeout for threads performing device init. This should
-        // leave us with a comfortable buffer as the longest init observed so
-        // far was 20 seconds on K80.
+        // Even on platforms with strict timing requirements (e.g. WDDM) there
+        // is an exception for initialization. While init time is an important
+        // performance metric, we do not want to functionally fail because of
+        // an arbitrary deadline. Thus, we set the timeout to give plenty of
+        // buffer room for some of the slower platforms:
+        // - P40 can take ~30 seconds when booting in passthrough due to
+        //   Hyper-V intercepting all MMIO accesses (bug 1900927)
+        // - Hopper+ can take 3+ seconds due to memory link initialization
         //
-        // See bug 1900927 for more details.
-        //
-        const NvU32 HYPER_V_INIT_TIMEOUT_MS = 60 * 1000;
+        const NvU32 DEVICE_INIT_TIMEOUT_MS = 60 * 1000;
 
-        computeTimeoutMsecs = HYPER_V_INIT_TIMEOUT_MS;
-        nonComputeTimeoutMsecs = HYPER_V_INIT_TIMEOUT_MS;
+        computeTimeoutMsecs = NV_MAX(computeTimeoutMsecs, DEVICE_INIT_TIMEOUT_MS);
+        nonComputeTimeoutMsecs = NV_MAX(nonComputeTimeoutMsecs, DEVICE_INIT_TIMEOUT_MS);
     }
 
     _threadStateSetNextCpuYieldTime(pThreadNode);
@@ -423,7 +422,7 @@ static NV_STATUS _threadNodeCheckTimeout(OBJGPU *pGpu, THREAD_STATE_NODE *pThrea
         return NV_ERR_INVALID_STATE;
     }
 
-    osGetCurrentTick(&timeInNs);
+    timeInNs = osGetMonotonicTimeNs();
     if (pElapsedTimeUs)
     {
         *pElapsedTimeUs = (timeInNs - pThreadNode->timeout.enterTime) / 1000;
@@ -509,27 +508,21 @@ static void _threadStateLogInitCaller(THREAD_STATE_NODE *pThreadNode, NvU64 func
 }
 
 /**
- * @brief Initialize a threadState for regular threads (non-interrupt context)
- *
- * @param[in/out] pThreadNode
- * @param[in] flags
- *
+ * @brief Common initialization logic for both stack and heap thread state nodes
+ * 
+ * @param[in/out] pThreadNode The node to initialize 
+ * @param[in] flags Thread state flags
+ * @param[in] bUsingHeap NV_TRUE if heap-allocated, NV_FALSE if stack-allocated
+ * 
+ * @return NV_OK on success, error code on failure
  */
-void threadStateInit(THREAD_STATE_NODE *pThreadNode, NvU32 flags)
+static NV_STATUS _threadStateInitCommon(THREAD_STATE_NODE *pThreadNode, NvU32 flags, NvBool bUsingHeap)
 {
     NV_STATUS rmStatus;
     NvU64 funcAddr;
 
-    // Isrs should be using threadStateIsrInit().
-    NV_ASSERT((flags & (THREAD_STATE_FLAGS_IS_ISR_LOCKLESS |
-                        THREAD_STATE_FLAGS_IS_ISR |
-                        THREAD_STATE_FLAGS_DEFERRED_INT_HANDLER_RUNNING)) == 0);
-
-    // Check to see if ThreadState is enabled
-    if (!(threadStateDatabase.setupFlags & THREAD_STATE_SETUP_FLAGS_ENABLED))
-        return;
-
     portMemSet(pThreadNode, 0, sizeof(*pThreadNode));
+    pThreadNode->bUsingHeap = bUsingHeap;
     pThreadNode->threadSeqId = portAtomicIncrementU32(&threadStateDatabase.threadSeqCntr);
     pThreadNode->cpuNum = osGetCurrentProcessorNumber();
     pThreadNode->flags = flags;
@@ -547,9 +540,10 @@ void threadStateInit(THREAD_STATE_NODE *pThreadNode, NvU32 flags)
 
     rmStatus = osGetCurrentThread(&pThreadNode->threadId);
     if (rmStatus != NV_OK)
-        return;
+        return rmStatus;
 
-    NV_ASSERT_OR_RETURN_VOID(pThreadNode->cpuNum < threadStateDatabase.maxCPUs);
+    NV_ASSERT_OR_RETURN(pThreadNode->cpuNum < threadStateDatabase.maxCPUs, 
+                        NV_ERR_INVALID_STATE);
 
     funcAddr = (NvU64) (NV_RETURN_ADDRESS());
 
@@ -559,27 +553,23 @@ void threadStateInit(THREAD_STATE_NODE *pThreadNode, NvU32 flags)
         // Reset the threadId as insertion failed. bValid is already NV_FALSE
         pThreadNode->threadId = 0;
         portSyncSpinlockRelease(threadStateDatabase.spinlock);
-        return;
-    }
-    else
-    {
-        pThreadNode->bValid = NV_TRUE;
-        rmStatus = NV_OK;
+        return NV_ERR_GENERIC;
     }
 
+    pThreadNode->bValid = NV_TRUE;
     _threadStateLogInitCaller(pThreadNode, funcAddr);
 
     portSyncSpinlockRelease(threadStateDatabase.spinlock);
 
     _threadStatePrintInfo(pThreadNode);
 
-    NV_ASSERT(rmStatus == NV_OK);
     threadPriorityStateAlloc();
 
     if (TLS_MIRROR_THREADSTATE)
     {
         THREAD_STATE_NODE **pTls = (THREAD_STATE_NODE **)tlsEntryAcquire(TLS_ENTRY_ID_THREADSTATE);
-        NV_ASSERT_OR_RETURN_VOID(pTls != NULL);
+        NV_ASSERT_OR_RETURN(pTls != NULL, NV_ERR_INVALID_STATE);
+
         if (*pTls != NULL)
         {
             NV_PRINTF(LEVEL_WARNING,
@@ -588,6 +578,66 @@ void threadStateInit(THREAD_STATE_NODE *pThreadNode, NvU32 flags)
         }
         *pTls = pThreadNode;
     }
+    return NV_OK;
+}
+
+/**
+ * @brief Initialize a threadState for regular threads (non-interrupt context)
+ *  Use the new UAF-safe API for new code, threadStateAlloc().
+ * @param[in/out] pThreadNode
+ * @param[in] flags
+ *
+ */
+void threadStateInit(THREAD_STATE_NODE *pThreadNode, NvU32 flags)
+{
+    // Isrs should be using threadStateIsrInit().
+    NV_ASSERT_OR_RETURN_VOID((flags & (THREAD_STATE_FLAGS_IS_ISR_LOCKLESS |
+        THREAD_STATE_FLAGS_IS_ISR |
+        THREAD_STATE_FLAGS_DEFERRED_INT_HANDLER_RUNNING)) == 0);
+
+    // Check to see if ThreadState is enabled
+    if (!(threadStateDatabase.setupFlags & THREAD_STATE_SETUP_FLAGS_ENABLED))
+        return;
+
+    // Use common initialization logic (stack-allocated)
+    // Note: Legacy void API ignores errors for backward compatibility
+    _threadStateInitCommon(pThreadNode, flags, NV_FALSE);
+}
+
+/**
+ * @brief Allocate a heap-based threadState
+ * @param[in] flags Thread state flags
+ *
+ * @return Heap-allocated THREAD_STATE_NODE* on success, NULL on failure
+ */
+THREAD_STATE_NODE* threadStateAlloc(NvU32 flags)
+{
+    THREAD_STATE_NODE *pHeapNode;
+    NV_STATUS rmStatus;
+
+    // Isrs should be using threadStateIsrInit().
+    NV_ASSERT_OR_RETURN((flags & (THREAD_STATE_FLAGS_IS_ISR_LOCKLESS |
+        THREAD_STATE_FLAGS_IS_ISR |
+        THREAD_STATE_FLAGS_DEFERRED_INT_HANDLER_RUNNING)) == 0, NULL);
+
+    // Check to see if ThreadState is enabled
+    if (!(threadStateDatabase.setupFlags & THREAD_STATE_SETUP_FLAGS_ENABLED))
+        return NULL;
+
+    // Allocate heap node directly
+    pHeapNode = portMemAllocNonPaged(sizeof(THREAD_STATE_NODE));
+    if (pHeapNode == NULL)
+        return NULL;
+
+    rmStatus = _threadStateInitCommon(pHeapNode, flags, NV_TRUE);
+    if (rmStatus != NV_OK)
+        goto cleanup_heap;
+
+    return pHeapNode;
+
+cleanup_heap:
+    portMemFree(pHeapNode);
+    return NULL;
 }
 
 /**
@@ -640,7 +690,9 @@ TlsMirror_Exit:
     if (rmStatus != NV_OK)
         return;
 
+    portSyncSpinlockAcquire(threadStateDatabase.spinlock);
     threadStateDatabase.ppISRDeferredIntHandlerThreadNode[pGpu->gpuInstance] = pThreadNode;
+    portSyncSpinlockRelease(threadStateDatabase.spinlock);
 }
 
 /**
@@ -691,6 +743,8 @@ TlsMirror_Exit:
 
     NV_ASSERT_OR_RETURN_VOID(pThreadNode->cpuNum < threadStateDatabase.maxCPUs);
 
+    portSyncSpinlockAcquire(threadStateDatabase.spinlock);
+
     //
     // We use a cpu/gpu indexed structure to store the threadNode pointer
     // instead of a tree indexed by threadId because threadId is no longer
@@ -701,17 +755,27 @@ TlsMirror_Exit:
     pThreadStateIsrLockless = &threadStateDatabase.pIsrlocklessThreadNode[pThreadNode->cpuNum];
     NV_ASSERT(pThreadStateIsrLockless->ppIsrThreadStateGpu[pGpu->gpuInstance] == NULL);
     pThreadStateIsrLockless->ppIsrThreadStateGpu[pGpu->gpuInstance] = pThreadNode;
+    portSyncSpinlockRelease(threadStateDatabase.spinlock);
 }
 
-/**
- * @brief Free the thread state for locked ISR and bottom-half
- *
- * @param[in/out] pThreadNode
- * @param[in] pGpu
- * @param[in] flags THREAD_STATE_FLAGS_IS_ISR or THREAD_STATE_FLAGS_DEFERRED_INT_HANDLER_RUNNING
- *
- */
-void threadStateFreeISRAndDeferredIntHandler
+void threadStateOnlyProcessWorkISRAndDeferredIntHandler
+(
+    THREAD_STATE_NODE *pThreadNode,
+    OBJGPU *pGpu,
+    NvU32 flags
+)
+{
+    NV_ASSERT_OR_RETURN_VOID(pGpu &&
+        (flags & (THREAD_STATE_FLAGS_IS_ISR | THREAD_STATE_FLAGS_DEFERRED_INT_HANDLER_RUNNING)));
+
+    if (!(threadStateDatabase.setupFlags & THREAD_STATE_SETUP_FLAGS_ENABLED))
+        return;
+
+    // Process any work needed before exiting.
+    _threadStateFreeProcessWork(pThreadNode);
+}
+
+void threadStateOnlyFreeISRAndDeferredIntHandler
 (
     THREAD_STATE_NODE *pThreadNode,
     OBJGPU *pGpu,
@@ -726,16 +790,15 @@ void threadStateFreeISRAndDeferredIntHandler
     if (!(threadStateDatabase.setupFlags & THREAD_STATE_SETUP_FLAGS_ENABLED))
         return;
 
-    // Process any work needed before exiting.
-    _threadStateFreeProcessWork(pThreadNode);
-
     if (threadStateDatabase.setupFlags & THREAD_STATE_SETUP_FLAGS_CHECK_TIMEOUT_AT_FREE_ENABLED)
     {
         rmStatus = _threadNodeCheckTimeout(NULL /*pGpu*/, pThreadNode, NULL /*pElapsedTimeUs*/);
         NV_ASSERT(rmStatus == NV_OK);
     }
 
+    portSyncSpinlockAcquire(threadStateDatabase.spinlock);
     threadStateDatabase.ppISRDeferredIntHandlerThreadNode[pGpu->gpuInstance] = NULL;
+    portSyncSpinlockRelease(threadStateDatabase.spinlock);
 
     if (TLS_MIRROR_THREADSTATE)
     {
@@ -758,6 +821,26 @@ void threadStateFreeISRAndDeferredIntHandler
     }
 }
 
+/**
+ * @brief Free the thread state for locked ISR and bottom-half
+ *
+ * @param[in/out] pThreadNode
+ * @param[in] pGpu
+ * @param[in] flags THREAD_STATE_FLAGS_IS_ISR or THREAD_STATE_FLAGS_DEFERRED_INT_HANDLER_RUNNING
+ *
+ */
+void threadStateFreeISRAndDeferredIntHandler
+(
+    THREAD_STATE_NODE *pThreadNode,
+    OBJGPU *pGpu,
+    NvU32 flags
+)
+{
+    threadStateOnlyProcessWorkISRAndDeferredIntHandler(pThreadNode, pGpu, flags);
+
+    threadStateOnlyFreeISRAndDeferredIntHandler(pThreadNode, pGpu, flags);
+}
+
 /**
  * @brief Free the thread state for a regular thread
  *
@@ -845,6 +928,12 @@ void threadStateFree(THREAD_STATE_NODE *pThreadNode, NvU32 flags)
                      r);
         }
     }
+
+    // Free heap memory if this node was heap-allocated
+    if (pThreadNode->bUsingHeap)
+    {
+        portMemFree(pThreadNode);
+    }
 }
 
 /**
@@ -876,9 +965,11 @@ void threadStateFreeISRLockless(THREAD_STATE_NODE *pThreadNode, OBJGPU *pGpu, Nv
         NV_ASSERT(rmStatus == NV_OK);
     }
 
+    portSyncSpinlockAcquire(threadStateDatabase.spinlock);
     pThreadStateIsrlockless = &threadStateDatabase.pIsrlocklessThreadNode[pThreadNode->cpuNum];
     NV_ASSERT(pThreadStateIsrlockless->ppIsrThreadStateGpu[pGpu->gpuInstance] != NULL);
     pThreadStateIsrlockless->ppIsrThreadStateGpu[pGpu->gpuInstance] = NULL;
+    portSyncSpinlockRelease(threadStateDatabase.spinlock);
 
     if (TLS_MIRROR_THREADSTATE)
     {
@@ -1169,16 +1260,9 @@ NV_STATUS threadStateCheckTimeout(OBJGPU *pGpu, NvU64 *pElapsedTimeUs)
     return rmStatus;
 }
 
-//
-// Set override timeout value for specified thread
-//
-void threadStateSetTimeoutOverride(THREAD_STATE_NODE *pThreadNode, NvU64 newTimeoutMs)
+static void _threadStateSetTimeoutOverride(THREAD_STATE_NODE *pThreadNode, NvU64 newTimeoutMs)
 {
-    NvU64 timeInNs;
-
-    pThreadNode->timeout.overrideTimeoutMsecs = newTimeoutMs;
-
-    osGetCurrentTick(&timeInNs);
+    NvU64 timeInNs = osGetMonotonicTimeNs();
 
     _threadStateSetNextCpuYieldTime(pThreadNode);
 
@@ -1195,6 +1279,24 @@ void threadStateSetTimeoutOverride(THREAD_STATE_NODE *pThreadNode, NvU64 newTime
     }
 }
 
+//
+// Set override timeout value for specified thread
+//
+void threadStateSetTimeoutOverride(THREAD_STATE_NODE *pThreadNode, NvU64 newTimeoutMs)
+{
+    pThreadNode->timeout.overrideTimeoutMsecs = newTimeoutMs;
+    _threadStateSetTimeoutOverride(pThreadNode, newTimeoutMs);
+}
+
+//
+// One-time override timeout for specified thread; does not apply across timeout resets
+//
+void threadStateSetTimeoutSingleOverride(THREAD_STATE_NODE *pThreadNode, NvU64 newTimeoutMs)
+{
+    // Does not cache override in overrideTimeoutMsecs, so it is not re-applied upon reset.
+    _threadStateSetTimeoutOverride(pThreadNode, newTimeoutMs);
+}
+
 NV_STATUS threadStateEnqueueCallbackOnFree
 (
     THREAD_STATE_NODE          *pThreadNode,
