@@ -27,6 +27,7 @@
 #include "nverror.h"
 #include "gpu/bif/kernel_bif.h"
 #include "gpu/fsp/kern_fsp.h"
+#include "gpu_mgr/gpu_mgr.h"
 #include "platform/chipset/chipset.h"
 #include "ctrl/ctrl2080/ctrl2080bus.h"
 #include "kernel/gpu/nvlink/kernel_nvlink.h"
@@ -42,11 +43,12 @@
 
 #include "os/os.h"
 
-static NV_STATUS    _kbifSavePcieConfigRegisters_GH100(OBJGPU *pGpu, KernelBif *pKernelBif, const PKBIF_XVE_REGMAP_REF pRegmapRef);
-static NV_STATUS    _kbifRestorePcieConfigRegisters_GH100(OBJGPU *pGpu, KernelBif *pKernelBif, const PKBIF_XVE_REGMAP_REF pRegmapRef);
+static NV_STATUS _kbifSavePcieConfigRegisters_GH100(OBJGPU *pGpu, KernelBif *pKernelBif, const PKBIF_XVE_REGMAP_REF pRegmapRef, NvU32 maxSize);
+static NV_STATUS _kbifRestorePcieConfigRegisters_GH100(OBJGPU *pGpu, KernelBif *pKernelBif, const PKBIF_XVE_REGMAP_REF pRegmapRef, NvU32 maxSize);
 
 // Size of PCIe config space
-#define PCIE_CONFIG_SPACE_MAX_SIZE     0x2FC
+#define PCIE_CONFIG_SPACE_FN0_MAX_SIZE     0x2FC
+#define PCIE_CONFIG_SPACE_FN1_MAX_SIZE     0x15C
 
 // Devinit completion timeout after FLR
 #define BIF_FLR_DEVINIT_COMPLETION_TIMEOUT_DEFAULT      900000
@@ -68,6 +70,12 @@ kbifIsMSIEnabledInHW_GH100
 {
     NvU32 data32;
 
+    if (IS_VIRTUAL(pGpu))
+    {
+        // SR-IOV guests only support MSI-X
+        return !IS_VIRTUAL_WITH_SRIOV(pGpu);
+    }
+
     if (GPU_BUS_CFG_CYCLE_RD32(pGpu, NV_EP_PCFG_GPU_MSI_64_HEADER,
                              &data32) != NV_OK)
     {
@@ -114,35 +122,6 @@ kbifIsMSIXEnabledInHW_GH100
     }
 }
 
-/*!
- * @brief Check if access to PCI config space is enabled or not
- *
- * @param[in] pGpu        GPU object pointer
- * @param[in] pKernelBif  Kernel BIF object pointer
- *
- * @return NV_TRUE Pci IO access is enabled
- */
-NvBool
-kbifIsPciIoAccessEnabled_GH100
-(
-    OBJGPU    *pGpu,
-    KernelBif *pKernelBif
-)
-{
-    NvU32   data = 0;
-
-    if (GPU_BUS_CFG_CYCLE_RD32(pGpu,
-                             NV_EP_PCFG_GPU_CTRL_CMD_AND_STATUS, &data) == NV_OK)
-    {
-        if (FLD_TEST_DRF(_EP_PCFG_GPU, _CTRL_CMD_AND_STATUS, _CMD_IO_SPACE, _ENABLE, data))
-        {
-            return NV_TRUE;
-        }
-    }
-
-    return NV_FALSE;
-}
-
 /*!
  * @brief Check if device is a 3D controller
  *
@@ -764,7 +743,8 @@ _kbifSavePcieConfigRegisters_GH100
 (
     OBJGPU    *pGpu,
     KernelBif *pKernelBif,
-    const PKBIF_XVE_REGMAP_REF pRegmapRef
+    const PKBIF_XVE_REGMAP_REF pRegmapRef,
+    NvU32      maxSize
 )
 {
     NV_STATUS status;
@@ -772,11 +752,11 @@ _kbifSavePcieConfigRegisters_GH100
     NvU32     bufOffset = 0;
 
     // Read and save entire config space
-    for (regOffset = 0x0; regOffset < PCIE_CONFIG_SPACE_MAX_SIZE; regOffset+=0x4)
+    for (regOffset = 0x0; regOffset < maxSize; regOffset+=0x4)
     {
         status = GPU_BUS_CFG_CYCLE_RD32(pGpu, regOffset,
                                         &pRegmapRef->bufBootConfigSpace[bufOffset]);
-        if (status != NV_OK)
+        if (status == NV_ERR_INVALID_STATE)
         {
             NV_PRINTF(LEVEL_ERROR, "Config read failed.\n");
             return status;
@@ -803,7 +783,8 @@ _kbifRestorePcieConfigRegisters_GH100
 (
     OBJGPU    *pGpu,
     KernelBif *pKernelBif,
-    const PKBIF_XVE_REGMAP_REF pRegmapRef
+    const PKBIF_XVE_REGMAP_REF pRegmapRef,
+    NvU32      maxSize
 )
 {
     NvU32     domain    = gpuGetDomain(pGpu);
@@ -818,11 +799,11 @@ _kbifRestorePcieConfigRegisters_GH100
     NV_ASSERT_OR_RETURN(pHandle, NV_ERR_INVALID_POINTER);
 
     // Restore entire config space
-    for (regOffset = 0x0; regOffset < PCIE_CONFIG_SPACE_MAX_SIZE; regOffset+=0x4)
+    for (regOffset = 0x0; regOffset < maxSize; regOffset+=0x4)
     {
         status = GPU_BUS_CFG_CYCLE_WR32(pGpu, regOffset,
                      pRegmapRef->bufBootConfigSpace[bufOffset]);
-        if (status != NV_OK)
+        if (status == NV_ERR_INVALID_STATE)
         {
             NV_PRINTF(LEVEL_ERROR, "Config write failed.\n");
             NV_ASSERT(0);
@@ -839,11 +820,9 @@ _kbifRestorePcieConfigRegisters_GH100
  *
  * @param[in]  pGpu         GPU object pointer
  * @param[in]  pKernelBif   KernelBif object pointer
- *
- * @return  'NV_OK' if successful, an RM error code otherwise.
  */
-NV_STATUS
-kbifSaveMsixTable_GH100
+static void
+_kbifSaveMsixTable
 (
     OBJGPU    *pGpu,
     KernelBif *pKernelBif
@@ -860,8 +839,6 @@ kbifSaveMsixTable_GH100
         pKernelBif->xveRegmapRef[0].bufMsixTable[(i*4) + 2] = GPU_VREG_RD32(pGpu, NV_VIRTUAL_FUNCTION_PRIV_MSIX_TABLE_DATA(i));
         pKernelBif->xveRegmapRef[0].bufMsixTable[(i*4) + 3] = GPU_VREG_RD32(pGpu, NV_VIRTUAL_FUNCTION_PRIV_MSIX_TABLE_VECTOR_CONTROL(i));
     }
-
-    return NV_OK;
 }
 
 /*!
@@ -869,11 +846,9 @@ kbifSaveMsixTable_GH100
  *
  * @param[in]  pGpu         GPU object pointer
  * @param[in]  pKernelBif   KernelBif object pointer
- *
- * @return  'NV_OK' if successful, an RM error code otherwise.
  */
-NV_STATUS
-kbifRestoreMsixTable_GH100
+static void
+_kbifRestoreMsixTable
 (
     OBJGPU    *pGpu,
     KernelBif *pKernelBif
@@ -894,8 +869,6 @@ kbifRestoreMsixTable_GH100
         osGpuWriteReg032(pGpu, vRegOffset + NV_VIRTUAL_FUNCTION_PRIV_MSIX_TABLE_DATA(i),           pKernelBif->xveRegmapRef[0].bufMsixTable[(i*4) + 2]);
         osGpuWriteReg032(pGpu, vRegOffset + NV_VIRTUAL_FUNCTION_PRIV_MSIX_TABLE_VECTOR_CONTROL(i), pKernelBif->xveRegmapRef[0].bufMsixTable[(i*4) + 3]);
     }
-
-    return NV_OK;
 }
 
 /*!
@@ -972,6 +945,8 @@ kbifDoFunctionLevelReset_GH100
     NvU32      flrDevInitTimeout;
     NvU32      flrDevInitTimeoutScale = pKernelBif->flrDevInitTimeoutScale;
 
+    KernelFsp *pKernelFsp = GPU_GET_KERNEL_FSP(pGpu);
+
     // If this is non-windows platform or non-ESXi, we already use OS based interface
     {
         pKernelBif->bInFunctionLevelReset = NV_TRUE;
@@ -995,12 +970,7 @@ kbifDoFunctionLevelReset_GH100
 
     if (bMSIXEnabled)
     {
-        status = kbifSaveMsixTable_HAL(pGpu, pKernelBif);
-        if (status != NV_OK)
-        {
-            NV_ASSERT_FAILED("MSIX Table save failed!\n");
-            goto kbifDoFunctionLevelReset_GH100_exit;
-        }
+        _kbifSaveMsixTable(pGpu, pKernelBif);
     }
 
     // Once we save required registers, we are done with prep. phase for FLR
@@ -1065,7 +1035,15 @@ kbifDoFunctionLevelReset_GH100
             NV_PRINTF(LEVEL_ERROR, "Entering secure boot completion wait.\n");
         }
 
-        status = NV_ERR_NOT_SUPPORTED;
+        if (pKernelFsp == NULL ||
+            pKernelFsp->getProperty(pKernelFsp, PDB_PROP_KFSP_IS_MISSING))
+        {
+            status = NV_ERR_NOT_SUPPORTED;
+        }
+        else
+        {
+            status = kfspWaitForSecureBoot_HAL(pGpu, pKernelFsp);
+        }
         if (status != NV_OK)
         {
             DBG_BREAKPOINT();
@@ -1083,11 +1061,7 @@ kbifDoFunctionLevelReset_GH100
     if (bMSIXEnabled)
     {
         // TODO-NM: Check if this needed for other NVPM flows like GC6
-        status = kbifRestoreMsixTable_HAL(pGpu, pKernelBif);
-        if (status != NV_OK)
-        {
-            NV_ASSERT_FAILED("MSIX Table save failed!\n");
-        }
+        _kbifRestoreMsixTable(pGpu, pKernelBif);
     }
 
 kbifDoFunctionLevelReset_GH100_exit:
@@ -1368,7 +1342,6 @@ kbifSavePcieConfigRegisters_GH100
     KernelBif *pKernelBif
 )
 {
-
     NV_STATUS status = NV_OK;
 
     // Skip config save on FMODEL (200684952), move to regmap (200700271)
@@ -1394,7 +1367,7 @@ kbifSavePcieConfigRegisters_GH100
 
     // Save pcie config space for function 0
     status = _kbifSavePcieConfigRegisters_GH100(pGpu, pKernelBif,
-                                                &pKernelBif->xveRegmapRef[0]);
+                                                &pKernelBif->xveRegmapRef[0], PCIE_CONFIG_SPACE_FN0_MAX_SIZE);
     if (status != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR, "Saving PCIe config space failed for gpu.\n");
@@ -1402,6 +1375,34 @@ kbifSavePcieConfigRegisters_GH100
         return status;
     }
 
+    // No need to save/restore azalia config space if gpu is in GC6 cycle or if it is in FLR
+    if (IS_GPU_GC6_STATE_ENTERING(pGpu) ||
+        pKernelBif->bPreparingFunctionLevelReset)
+    {
+        return NV_OK;
+    }
+
+    // Return early if device is not multifunction (azalia is disabled or not present)
+    if (!pKernelBif->getProperty(pKernelBif, PDB_PROP_KBIF_DEVICE_IS_MULTIFUNCTION))
+    {
+        return NV_OK;
+    }
+
+    if (!IS_SILICON(pGpu))
+    {
+        NV_PRINTF(LEVEL_INFO, "Skipping PCIe Fn1 config space save.\n");
+        return NV_OK;
+    }
+
+    // Save pcie config space for function 1
+    status = kbifSavePcieConfigRegistersFn1_HAL(pGpu, pKernelBif,
+                                                &pKernelBif->xveRegmapRef[1], PCIE_CONFIG_SPACE_FN1_MAX_SIZE);
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Saving PCIe config space failed for azalia.\n");
+        NV_ASSERT(0);
+    }
+
     return status;
 }
 
@@ -1421,6 +1422,9 @@ kbifRestorePcieConfigRegisters_GH100
 )
 {
     NV_STATUS status = NV_OK;
+    RMTIMEOUT timeout;
+    NvU64     timeStampStart;
+    NvU64     timeStampEnd;
 
     // Skip config restore on FMODEL (200684952), move to regmap (200700271)
     if (IS_FMODEL(pGpu))
@@ -1428,9 +1432,16 @@ kbifRestorePcieConfigRegisters_GH100
         return NV_OK;
     }
 
+    if (pKernelBif->xveRegmapRef[0].bufBootConfigSpace == NULL)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Config space buffer is NULL!\n");
+        NV_ASSERT(0);
+        return NV_ERR_OBJECT_NOT_FOUND;
+    }
+
     // Restore pcie config space for function 0
     status = _kbifRestorePcieConfigRegisters_GH100(pGpu, pKernelBif,
-                                                   &pKernelBif->xveRegmapRef[0]);
+                                                   &pKernelBif->xveRegmapRef[0], PCIE_CONFIG_SPACE_FN0_MAX_SIZE);
     if (status != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR, "Restoring PCIe config space failed for gpu.\n");
@@ -1438,6 +1449,69 @@ kbifRestorePcieConfigRegisters_GH100
         return status;
     }
 
+    // No need to save/restore azalia config space if gpu is in GC6 cycle or if it is in FLR
+    if (IS_GPU_GC6_STATE_EXITING(pGpu) ||
+        pKernelBif->bInFunctionLevelReset)
+    {
+        //
+        // Check that GPU is really accessible.
+        // Skip on pre-silicon because there can be timing issues in the test between device ready and this code.
+        // Todo: find a safe timeout for pre-silicon runs
+        //
+        if (IS_SILICON(pGpu))
+        {
+            // Check if GPU is actually accessible before continue
+            osGetPerformanceCounter(&timeStampStart);
+            gpuSetTimeout(pGpu, GPU_TIMEOUT_DEFAULT, &timeout, 0);
+            NvU32 pmcBoot0 = GPU_REG_RD32(pGpu, NV_PMC_BOOT_0);
+
+            while (pmcBoot0 != pGpu->chipId0)
+            {
+                NV_PRINTF(LEVEL_INFO,
+                          "GPU not back on the bus after %s, 0x%x != 0x%x!\n",
+                          pKernelBif->bInFunctionLevelReset?"FLR":"GC6 exit", pmcBoot0, pGpu->chipId0);
+                pmcBoot0 = GPU_REG_RD32(pGpu, NV_PMC_BOOT_0);
+                NV_ASSERT(0);
+                status = gpuCheckTimeout(pGpu, &timeout);
+                if (status == NV_ERR_TIMEOUT)
+                {
+                    NV_PRINTF(LEVEL_ERROR,
+                              "Timeout GPU not back on the bus after %s,\n", pKernelBif->bInFunctionLevelReset?"FLR":"GC6 exit");
+                    DBG_BREAKPOINT();
+                    return status;
+                }
+            }
+
+            osGetPerformanceCounter(&timeStampEnd);
+            NV_PRINTF(LEVEL_ERROR,
+                      "Time spend on GPU back on bus is 0x%x ns,\n",
+                      (NvU32)NV_MIN(NV_U32_MAX, timeStampEnd - timeStampStart));
+        }
+
+        return NV_OK;
+    }
+
+    // Return early if device is not multifunction (azalia is disabled or not present)
+    if (!pKernelBif->getProperty(pKernelBif, PDB_PROP_KBIF_DEVICE_IS_MULTIFUNCTION))
+    {
+        return NV_OK;
+    }
+
+    if (!IS_SILICON(pGpu))
+    {
+        NV_PRINTF(LEVEL_INFO, "Skipping PCIe Fn1 config space restore.\n");
+        return NV_OK;
+    }
+
+    // Restore pcie config space for function 1
+    status = kbifRestorePcieConfigRegistersFn1_HAL(pGpu, pKernelBif,
+                                                   &pKernelBif->xveRegmapRef[1], PCIE_CONFIG_SPACE_FN1_MAX_SIZE);
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Restoring PCIe config space failed for azalia.\n");
+        NV_ASSERT(0);
+    }
+
     return status;
 }
 
@@ -1714,3 +1788,79 @@ kbifDoSecondaryBusHotReset_GH100
     return kbifDoSecondaryBusHotReset_GM107(pGpu, pKernelBif);
 }
 
+/*!
+ * @brief Find the presence of C2C connection
+ *
+ * @param[in] pGpu0            First GPU object pointer
+ * @param[in] pKernelBif0      KernelBIF object pointer of first GPU
+ * @param[in] pGpu1            second GPU object pointer
+ *
+ * @return  NV_TRUE if C2C connection is present.
+ *          NV_FALSE otherwise.
+ */
+NvBool
+kbifIsC2CP2PSupported_GH100
+(
+    OBJGPU    *pGpu0,
+    KernelBif *pKernelBif0,
+    OBJGPU    *pGpu1
+)
+{
+    KernelBif *pKernelBif1  = GPU_GET_KERNEL_BIF(pGpu1);
+    NV2080_CTRL_CMD_BUS_GET_C2C_INFO_PARAMS c2cInfoParamsGpu0 = {0};
+    NV2080_CTRL_CMD_BUS_GET_C2C_INFO_PARAMS c2cInfoParamsGpu1 = {0};
+    RM_API *pRmApiGpu0 = GPU_GET_PHYSICAL_RMAPI(pGpu0);
+    RM_API *pRmApiGpu1 = GPU_GET_PHYSICAL_RMAPI(pGpu1);
+    NvU32 gpuAttachCnt = 0U;
+    NV_STATUS status;
+
+    if (pGpu0 != pGpu1)
+    {
+        if (gpumgrGetGpuAttachInfo(&gpuAttachCnt, NULL) != NV_OK ||
+            gpuAttachCnt > 2)
+        {
+            return NV_FALSE;
+        }
+    }
+
+    if (!pKernelBif0->getProperty(pKernelBif0, PDB_PROP_KBIF_IS_C2C_LINK_UP) ||
+        !pKernelBif1->getProperty(pKernelBif1, PDB_PROP_KBIF_IS_C2C_LINK_UP))
+    {
+        return NV_FALSE;
+    }
+
+    status = pRmApiGpu0->Control(pRmApiGpu0,
+                                 pGpu0->hInternalClient,
+                                 pGpu0->hInternalSubdevice,
+                                 NV2080_CTRL_CMD_BUS_GET_C2C_INFO,
+                                 &c2cInfoParamsGpu0,
+                                 sizeof(c2cInfoParamsGpu0));
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "GPU0 NV2080_CTRL_CMD_BUS_GET_C2C_INFO failed %s (0x%x)\n",
+                nvstatusToString(status), status);
+        return NV_FALSE;
+    }
+
+    status = pRmApiGpu1->Control(pRmApiGpu1,
+                                 pGpu1->hInternalClient,
+                                 pGpu1->hInternalSubdevice,
+                                 NV2080_CTRL_CMD_BUS_GET_C2C_INFO,
+                                 &c2cInfoParamsGpu1,
+                                 sizeof(c2cInfoParamsGpu1));
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "GPU1 NV2080_CTRL_CMD_BUS_GET_C2C_INFO failed %s (0x%x)\n",
+                nvstatusToString(status), status);
+        return NV_FALSE;
+    }
+
+    if ((c2cInfoParamsGpu0.remoteType == NV2080_CTRL_BUS_GET_C2C_INFO_REMOTE_TYPE_GPU) &&
+        (c2cInfoParamsGpu1.remoteType == NV2080_CTRL_BUS_GET_C2C_INFO_REMOTE_TYPE_GPU))
+    {
+        return NV_TRUE;
+    }
+
+    return NV_FALSE;
+}
+
