@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2014-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2014-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -27,7 +27,12 @@
 #include "gpu/bus/kern_bus.h"
 #include "gpu/bus/p2p_api.h"
 #include "gpu/mem_mgr/mem_mgr.h"
+#include "gpu/mem_mgr/fermi_dma.h"
 #include "kernel/gpu/nvlink/kernel_nvlink.h"
+#include "vgpu/vgpu_events.h"
+#include "platform/sli/sli.h"
+
+#include "published/pascal/gp100/dev_ram.h"
 
 /*!
  * @brief Create a P2P mapping to a given peer GPU
@@ -265,11 +270,11 @@ kbusCreateP2PMappingForNvlink_GP100
         NV_PRINTF(LEVEL_INFO, "- P2P: Using Default RM mapping for P2P.\n");
     }
 
+    if (bEgmPeer)
+    {
+        NV_PRINTF(LEVEL_INFO, "EGM peer\n");
+    }
 
-        if (bEgmPeer)
-        {
-            NV_PRINTF(LEVEL_INFO, "EGM peer\n");
-        }
     //
     // Does the mapping already exist between the given pair of GPUs using the peerIDs
     // *peer0 and *peer1 respectively ?
@@ -369,7 +374,7 @@ kbusCreateP2PMappingForNvlink_GP100
     {
         KernelNvlink *pKernelNvlink0 = GPU_GET_KERNEL_NVLINK(pGpu0);
         KernelNvlink *pKernelNvlink1 = GPU_GET_KERNEL_NVLINK(pGpu1);
-        NV2080_CTRL_NVLINK_ENABLE_NVLINK_PEER_PARAMS params;
+        NV2080_CTRL_INTERNAL_NVLINK_ENABLE_NVLINK_PEER_PARAMS params;
 
         if (pKernelNvlink0 == NULL || pKernelNvlink1 == NULL)
         {
@@ -382,7 +387,7 @@ kbusCreateP2PMappingForNvlink_GP100
 
         // Set the NVLink USE_NVLINK_PEER fields in the LTCS registers for GPU0
         status = knvlinkExecGspRmRpc(pGpu0, pKernelNvlink0,
-                                     NV2080_CTRL_CMD_NVLINK_ENABLE_NVLINK_PEER,
+                                     NV2080_CTRL_CMD_INTERNAL_NVLINK_ENABLE_NVLINK_PEER,
                                      (void *)&params, sizeof(params));
         if (status != NV_OK)
         {
@@ -399,7 +404,7 @@ kbusCreateP2PMappingForNvlink_GP100
 
         // Set the NVLink USE_NVLINK_PEER fields in the LTCS registers for GPU1
         status = knvlinkExecGspRmRpc(pGpu1, pKernelNvlink1,
-                                     NV2080_CTRL_CMD_NVLINK_ENABLE_NVLINK_PEER,
+                                     NV2080_CTRL_CMD_INTERNAL_NVLINK_ENABLE_NVLINK_PEER,
                                      (void *)&params, sizeof(params));
         if (status != NV_OK)
         {
@@ -477,6 +482,7 @@ _kbusRemoveNvlinkPeerMapping
     //    P2P between all the peers is destroyed.
     //    busNvlinkMappingRefcountPerGpu == 0 check is done in this case to remove
     //    the peer id from busNvlinkPeerNumberMask[peerGpuInst]
+    //
     //    Two peer ids are used to reach the same GPU, one for HBM and one for
     //    EGM. In that case busNvlinkMappingRefcountPerGpu isn't going to
     //    reach 0 until both the peer ids are removed. In this case,
@@ -534,7 +540,7 @@ _kbusRemoveNvlinkPeerMapping
         else
         {
             KernelNvlink *pKernelNvlink0 = GPU_GET_KERNEL_NVLINK(pGpu0);
-            NV2080_CTRL_NVLINK_ENABLE_NVLINK_PEER_PARAMS params;
+            NV2080_CTRL_INTERNAL_NVLINK_ENABLE_NVLINK_PEER_PARAMS params;
 
             NV_ASSERT_OR_RETURN(pKernelNvlink0 != NULL, NV_ERR_NOT_SUPPORTED);
 
@@ -544,7 +550,7 @@ _kbusRemoveNvlinkPeerMapping
 
             // Unset the NVLink USE_NVLINK_PEER fields in the LTCS registers for GPU0
             status = knvlinkExecGspRmRpc(pGpu0, pKernelNvlink0,
-                                         NV2080_CTRL_CMD_NVLINK_ENABLE_NVLINK_PEER,
+                                         NV2080_CTRL_CMD_INTERNAL_NVLINK_ENABLE_NVLINK_PEER,
                                          (void *)&params, sizeof(params));
             if (status != NV_OK)
             {
@@ -727,9 +733,10 @@ kbusGetPeerId_GP100
 {
     extern NvU32 kbusGetPeerId_GM107(OBJGPU *pGpu, KernelBus *pKernelBus, OBJGPU *pPeerGpu);
     NvU32 gpuPeerInst = gpuGetInstance(pGpuPeer);
-    NvU32 peerId = pKernelBus->p2p.busNvlinkPeerNumberMask[gpuPeerInst];
+    NvU32 peerMask = pKernelBus->p2p.busNvlinkPeerNumberMask[gpuPeerInst];
+    NvU32 peerId;
 
-    if (peerId == 0)
+    if (peerMask == 0)
     {
         NV_PRINTF(LEVEL_INFO,
                   "NVLINK P2P not set up between GPU%u and GPU%u, checking for PCIe P2P...\n",
@@ -737,8 +744,16 @@ kbusGetPeerId_GP100
         return kbusGetPeerId_GM107(pGpu, pKernelBus, pGpuPeer);
     }
 
-    LOWESTBITIDX_32(peerId);
-    return peerId;
+    FOR_EACH_INDEX_IN_MASK(32, peerId, peerMask)
+    {
+        if (!pKernelBus->p2p.bEgmPeer[peerId])
+        {
+            return peerId;
+        }
+    }
+    FOR_EACH_INDEX_IN_MASK_END;
+
+    return BUS_INVALID_PEER;
 }
 
 /**
@@ -800,6 +815,22 @@ kbusGetNvlinkP2PPeerId_GP100
 
     *nvlinkPeer = BUS_INVALID_PEER;
 
+    //
+    // If running in forced-config use pre-determined
+    // PeerIDs from the static table
+    //
+    if (knvlinkIsForcedConfig(pGpu0, pKernelNvlink0) ||
+        knvlinkAreLinksRegistryOverriden(pGpu0, pKernelNvlink0))
+    {
+        *nvlinkPeer = kbusGetPeerIdFromTable_HAL(pGpu0, pKernelBus0,
+                                                 pGpu0->gpuInstance,
+                                                 pGpu1->gpuInstance);
+
+        if (*nvlinkPeer == BUS_INVALID_PEER)
+        {
+            return NV_ERR_INVALID_REQUEST;
+        }
+    }
     //
     // Use the NVLINK-specific unique ID of the GPU (related to link ID) for
     // the peer ID. We expect that this will remain the same across multiple
@@ -810,50 +841,35 @@ kbusGetNvlinkP2PPeerId_GP100
     // no PCIe P2P will be used.
     //
     if ((pKernelNvlink0 != NULL) &&
-        (knvlinkGetPeersNvlinkMaskFromHshub(pGpu0, pKernelNvlink0) != 0))
+        (knvlinkGetNumLinksToPeer(pGpu0, pKernelNvlink0, pGpu1) != 0))
     {
-        if (knvlinkIsForcedConfig(pGpu0, pKernelNvlink0) ||
-            knvlinkAreLinksRegistryOverriden(pGpu0, pKernelNvlink0))
+        *nvlinkPeer = kbusGetPeerId_HAL(pGpu0, pKernelBus0, pGpu1);
+        if (*nvlinkPeer != BUS_INVALID_PEER)
         {
-            *nvlinkPeer = kbusGetPeerIdFromTable_HAL(pGpu0, pKernelBus0,
-                                                     pGpu0->gpuInstance,
-                                                     pGpu1->gpuInstance);
-
-            if (*nvlinkPeer == BUS_INVALID_PEER)
-            {
-                return NV_ERR_INVALID_REQUEST;
-            }
+            return NV_OK;
         }
-        else
+
+        // Reserve GPU0 peer IDs for NVLINK use
+        if (!pKernelBus0->p2p.bNvlinkPeerIdsReserved)
         {
-            *nvlinkPeer = kbusGetPeerId_HAL(pGpu0, pKernelBus0, pGpu1);
-            if (*nvlinkPeer != BUS_INVALID_PEER)
-            {
-                return NV_OK;
-            }
+            NvU32 idMask = knvlinkGetUniquePeerIdMask_HAL(pGpu0, pKernelNvlink0);
 
-            // Reserve GPU0 peer IDs for NVLINK use
-            if (!pKernelBus0->p2p.bNvlinkPeerIdsReserved)
+            //
+            // If NVLINK is topology is not forced, idMask will be non-zero
+            // if nvlinks are detected during topology discovery in core lib
+            //
+            if (idMask != 0)
             {
-                NvU32 idMask = knvlinkGetUniquePeerIdMask_HAL(pGpu0, pKernelNvlink0);
-
-                //
-                // If NVLINK is topology is not forced, idMask will be non-zero
-                // if nvlinks are detected during topology discovery in core lib
-                //
-                if (idMask != 0)
+                // Reserve GPU0 peer IDs for NVLINK use
+                status = kbusReserveP2PPeerIds_HAL(pGpu0, pKernelBus0, idMask);
+                if (status != NV_OK)
                 {
-                    // Reserve GPU0 peer IDs for NVLINK use
-                    status = kbusReserveP2PPeerIds_HAL(pGpu0, pKernelBus0, idMask);
-                    if (status != NV_OK)
-                    {
-                        return status;
-                    }
-                    pKernelBus0->p2p.bNvlinkPeerIdsReserved = NV_TRUE;
+                    return status;
                 }
+                pKernelBus0->p2p.bNvlinkPeerIdsReserved = NV_TRUE;
             }
-            *nvlinkPeer = knvlinkGetUniquePeerId_HAL(pGpu0, pKernelNvlink0, pGpu1);
         }
+        *nvlinkPeer = knvlinkGetUniquePeerId_HAL(pGpu0, pKernelNvlink0, pGpu1);
     }
 
     return status;
@@ -937,3 +953,248 @@ kbusGetNvlinkPeerNumberMask_GP100
 
     return (pKernelBus->p2p.busNvlinkPeerNumberMask[peerId]);
 }
+
+/*!
+ * @brief Initialize NV_RAMIN_ADR_LIMIT at the given location
+ *
+ * @param[in] pGpu            OBJGPU
+ * @param[in] pKernelBus      KernelBus
+ * @param[in] pBar0Wr         Controls whether to MEM_WR using pMap or GPU_REG_WR at addr
+ * @param[in] instBlockAddr   physical address of the instance block to be written
+ * @param[in] pMap            CPU mapped address to start from
+ * @param[in] vaLimit         Virtual address limit to program
+ */
+void
+kbusInstBlkWriteAddrLimit_GP100
+(
+    OBJGPU    *pGpu,
+    KernelBus *pKernelBus,
+    NvBool     bBar0Wr,
+    NvU64      instBlockAddr,
+    NvU8      *pMap,
+    NvU64      vaLimit
+)
+{
+    NV_STATUS         status     = NV_OK;
+    NvU32             adrLimitLo = (NvU64_LO32(vaLimit) | 0xfff);
+    NvU32             adrLimitHi = SF_NUM(_RAMIN_ADR_LIMIT, _HI, NvU64_HI32(vaLimit));
+
+    if (bBar0Wr)
+    {
+        status = kbusMemAccessBar0Window_HAL(pGpu, pKernelBus,
+                              (instBlockAddr + SF_OFFSET(NV_RAMIN_ADR_LIMIT_LO)),
+                              &adrLimitLo,
+                              sizeof(NvU32),
+                              NV_FALSE,
+                              pKernelBus->InstBlkAperture);
+        NV_ASSERT_OR_RETURN_VOID(NV_OK == status);
+
+        status = kbusMemAccessBar0Window_HAL(pGpu, pKernelBus,
+                              (instBlockAddr + SF_OFFSET(NV_RAMIN_ADR_LIMIT_HI)),
+                              &adrLimitHi,
+                              sizeof(NvU32),
+                              NV_FALSE,
+                              pKernelBus->InstBlkAperture);
+        NV_ASSERT_OR_RETURN_VOID(NV_OK == status);
+    }
+    else
+    {
+        MEM_WR32(pMap + SF_OFFSET(NV_RAMIN_ADR_LIMIT_LO), adrLimitLo);
+        MEM_WR32(pMap + SF_OFFSET(NV_RAMIN_ADR_LIMIT_HI), adrLimitHi);
+    }
+}
+
+NV_STATUS
+kbusInitInstBlk_GP100
+(
+    OBJGPU            *pGpu,
+    KernelBus         *pKernelBus,
+    PMEMORY_DESCRIPTOR pInstBlkMemDesc, // NULL if BAR2 aperture not set up yet: BAR2 instance block using BAR0 window
+    PMEMORY_DESCRIPTOR pPDB,
+    NvU64              vaLimit,
+    NvU64              bigPageSize,
+    OBJVASPACE        *pVAS
+)
+{
+    NvU8              *pMap;
+    NvBool             newPteFormat      = NV_FALSE;
+    NvU32              pageDirBaseHi     = 0;
+    NvU32              pageDirBaseTarget = 0;
+    NvU32              gfid;
+
+    NV_ASSERT_OK_OR_RETURN(vgpuGetCallingContextGfid(pGpu, &gfid));
+
+    NV_ASSERT_OR_RETURN(pKernelBus->bar2[gfid].pFmt != NULL, NV_ERR_INVALID_STATE);
+    newPteFormat = (GMMU_FMT_VERSION_2 == pKernelBus->bar2[gfid].pFmt->version);
+
+    if (pInstBlkMemDesc == NULL)
+    {
+        if (kbusIsPhysicalBar2InitPagetableEnabled(pKernelBus))
+        {
+            return kbusSetupBar2InstBlkAtBottomOfFb_HAL(pGpu, pKernelBus, pPDB, vaLimit, bigPageSize, gfid);
+        }
+
+        // Initialize NV_RAMIN_ADR_LIMIT at the given offset
+        kbusInstBlkWriteAddrLimit_HAL(pGpu, pKernelBus, NV_TRUE,
+                                      pKernelBus->bar2[gfid].instBlockBase,
+                                      NULL,
+                                      vaLimit);
+
+        pageDirBaseHi = SF_NUM(_RAMIN_PAGE_DIR_BASE, _HI, NvU64_HI32(memdescGetPhysAddr(pPDB, AT_GPU, 0)));
+        NV_ASSERT_OK_OR_RETURN(kbusMemAccessBar0Window_HAL(pGpu, pKernelBus,
+                               (pKernelBus->bar2[gfid].instBlockBase + SF_OFFSET(NV_RAMIN_PAGE_DIR_BASE_HI)),
+                               &pageDirBaseHi,
+                               sizeof(NvU32),
+                               NV_FALSE,
+                               pKernelBus->InstBlkAperture));
+
+        //
+        // Set up the big page size in the memdesc for this address space
+        //
+        if (bigPageSize == FERMI_BIG_PAGESIZE_128K)
+        {
+            pageDirBaseTarget = SF_NUM(_RAMIN_PAGE_DIR_BASE, _TARGET, kgmmuGetHwPteApertureFromMemdesc(GPU_GET_KERNEL_GMMU(pGpu), pPDB)) |
+                         (newPteFormat ?
+                         SF_DEF(_RAMIN, _USE_NEW_PT_FORMAT, _TRUE) :
+                         SF_DEF(_RAMIN, _USE_NEW_PT_FORMAT, _FALSE)) |
+                         SF_DEF(_RAMIN, _BIG_PAGE_SIZE, _128KB) |
+                         SF_NUM(_RAMIN_PAGE_DIR_BASE, _VOL, memdescGetVolatility(pPDB)) |
+                         SF_NUM(_RAMIN_PAGE_DIR_BASE, _LO, NvU64_LO32(memdescGetPhysAddr(pPDB, AT_GPU, 0) >> PDB_SHIFT_FERMI));
+        }
+        else if (bigPageSize == FERMI_BIG_PAGESIZE_64K)
+        {
+            pageDirBaseTarget = SF_NUM(_RAMIN_PAGE_DIR_BASE, _TARGET, kgmmuGetHwPteApertureFromMemdesc(GPU_GET_KERNEL_GMMU(pGpu), pPDB)) |
+                         (newPteFormat ?
+                         SF_DEF(_RAMIN, _USE_NEW_PT_FORMAT, _TRUE) :
+                         SF_DEF(_RAMIN, _USE_NEW_PT_FORMAT, _FALSE)) |
+                         SF_DEF(_RAMIN, _BIG_PAGE_SIZE, _64KB) |
+                         SF_NUM(_RAMIN_PAGE_DIR_BASE, _VOL, memdescGetVolatility(pPDB)) |
+                         SF_NUM(_RAMIN_PAGE_DIR_BASE, _LO, NvU64_LO32(memdescGetPhysAddr(pPDB, AT_GPU, 0) >> PDB_SHIFT_FERMI));
+        }
+        NV_ASSERT_OK_OR_RETURN(kbusMemAccessBar0Window_HAL(pGpu, pKernelBus,
+                               (pKernelBus->bar2[gfid].instBlockBase + SF_OFFSET(NV_RAMIN_PAGE_DIR_BASE_TARGET)),
+                               &pageDirBaseTarget,
+                               sizeof(NvU32),
+                               NV_FALSE,
+                               pKernelBus->InstBlkAperture));
+    }
+    else
+    {
+        SLI_LOOP_START(SLI_LOOP_FLAGS_BC_ONLY)
+        pKernelBus = GPU_GET_KERNEL_BUS(pGpu);
+        pMap = kbusMapRmAperture_HAL(pGpu, pInstBlkMemDesc);
+        if (pMap == NULL)
+        {
+            SLI_LOOP_RETURN(NV_ERR_INSUFFICIENT_RESOURCES);
+        }
+        kbusBar2InstBlkWrite_HAL(pGpu, pKernelBus, pMap, pPDB, vaLimit, bigPageSize);
+        kbusUnmapRmAperture_HAL(pGpu, pInstBlkMemDesc, &pMap, NV_TRUE);
+        if (pKernelBus->bar2[gfid].bMigrating)
+        {
+            //
+            // Remove memdesc from cached mappings as the page tables backing
+            // this mapping will be discarded and the old VA to PA translation
+            // won't be valid.
+            //
+            kbusReleaseRmAperture_HAL(pGpu, pKernelBus, pInstBlkMemDesc);
+        }
+        SLI_LOOP_END
+        pKernelBus = GPU_GET_KERNEL_BUS(pGpu);
+    }
+    return NV_OK;
+}
+
+void
+kbusBar2InstBlkWrite_GP100
+(
+    OBJGPU            *pGpu,
+    KernelBus         *pKernelBus,
+    NvU8              *pMap,
+    PMEMORY_DESCRIPTOR pPDB,
+    NvU64              vaLimit,
+    NvU64              bigPageSize
+)
+{
+    NvBool            newPteFormat = NV_FALSE;
+
+    NV_ASSERT_OR_RETURN_VOID(NULL != pMap);
+    NV_ASSERT_OR_RETURN_VOID(NULL != pKernelBus->bar2[GPU_GFID_PF].pFmt);
+
+    newPteFormat = (GMMU_FMT_VERSION_2 == pKernelBus->bar2[GPU_GFID_PF].pFmt->version);
+
+    // Initialize NV_RAMIN_ADR_LIMIT in the mapped instblk
+    kbusInstBlkWriteAddrLimit_HAL(pGpu, pKernelBus, NV_FALSE, 0x0, pMap, vaLimit);
+    MEM_WR32(pMap + SF_OFFSET(NV_RAMIN_PAGE_DIR_BASE_HI),
+                    SF_NUM(_RAMIN_PAGE_DIR_BASE, _HI,
+                           NvU64_HI32(memdescGetPhysAddr(pPDB, AT_GPU, 0))));
+
+    if (bigPageSize == FERMI_BIG_PAGESIZE_128K)
+    {
+        MEM_WR32(pMap + SF_OFFSET(NV_RAMIN_PAGE_DIR_BASE_TARGET),
+                        SF_NUM(_RAMIN_PAGE_DIR_BASE, _TARGET, kgmmuGetHwPteApertureFromMemdesc(GPU_GET_KERNEL_GMMU(pGpu), pPDB)) |
+                        (newPteFormat ?
+                        SF_DEF(_RAMIN, _USE_NEW_PT_FORMAT, _TRUE) :
+                        SF_DEF(_RAMIN, _USE_NEW_PT_FORMAT, _FALSE)) |
+                        SF_DEF(_RAMIN, _BIG_PAGE_SIZE, _128KB) |
+                        SF_NUM(_RAMIN_PAGE_DIR_BASE, _VOL, memdescGetVolatility(pPDB)) |
+                        SF_NUM(_RAMIN_PAGE_DIR_BASE, _LO,
+                               NvU64_LO32(memdescGetPhysAddr(pPDB, AT_GPU, 0) >> PDB_SHIFT_FERMI)));
+    }
+    else if (bigPageSize == FERMI_BIG_PAGESIZE_64K)
+    {
+        MEM_WR32(pMap + SF_OFFSET(NV_RAMIN_PAGE_DIR_BASE_TARGET),
+                        SF_NUM(_RAMIN_PAGE_DIR_BASE, _TARGET, kgmmuGetHwPteApertureFromMemdesc(GPU_GET_KERNEL_GMMU(pGpu), pPDB)) |
+                        (newPteFormat ?
+                        SF_DEF(_RAMIN, _USE_NEW_PT_FORMAT, _TRUE) :
+                        SF_DEF(_RAMIN, _USE_NEW_PT_FORMAT, _FALSE)) |
+                        SF_DEF(_RAMIN, _BIG_PAGE_SIZE, _64KB) |
+                        SF_NUM(_RAMIN_PAGE_DIR_BASE, _VOL, memdescGetVolatility(pPDB)) |
+                        SF_NUM(_RAMIN_PAGE_DIR_BASE, _LO,
+                               NvU64_LO32(memdescGetPhysAddr(pPDB, AT_GPU, 0) >> PDB_SHIFT_FERMI)));
+    }
+}
+
+/*!
+ * @brief Calculate the base and limit of BAR2 cpu-invisible range
+ *
+ * @param[in]  pGpu        OBJGPU pointer
+ * @param[in]  pKernelBus  KernelBus pointer
+ */
+void
+kbusCalcCpuInvisibleBar2Range_GP100
+(
+    OBJGPU    *pGpu,
+    KernelBus *pKernelBus,
+    NvU32      gfid
+)
+{
+    NvU32 cpuInvisibleSize = 0;
+
+    if (pKernelBus->bar2[gfid].cpuInvisibleLimit == 0)
+    {
+        cpuInvisibleSize = kbusCalcCpuInvisibleBar2ApertureSize_HAL(pGpu, pKernelBus);
+        if (cpuInvisibleSize != 0)
+        {
+            cpuInvisibleSize--;
+
+            if (pKernelBus->bar2[gfid].cpuInvisibleBase == 0)
+            {
+                if (!RMCFG_FEATURE_PLATFORM_GSP)
+                {
+                    if (pKernelBus->bar2[gfid].cpuVisibleLimit != 0)
+                    {
+                        pKernelBus->bar2[gfid].cpuInvisibleBase = pKernelBus->bar2[gfid].cpuVisibleLimit + 1;
+                    }
+                }
+                else
+                {
+                    pKernelBus->bar2[gfid].cpuInvisibleBase =
+                        kbusGetCpuInvisibleBar2BaseAdjust_HAL(pGpu, pKernelBus);
+                }
+            }
+            NV_PRINTF(LEVEL_INFO, "base: 0x%llx size: 0x%x\n",
+                      pKernelBus->bar2[gfid].cpuInvisibleBase, cpuInvisibleSize + 1);
+            pKernelBus->bar2[gfid].cpuInvisibleLimit = pKernelBus->bar2[gfid].cpuInvisibleBase + cpuInvisibleSize;
+        }
+    }
+}
