@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -420,10 +420,18 @@ kceIsGenXorHigherSupported_GH100
     NV_STATUS  status = NV_OK;
     NvBool     bIsGenXorHigher = NV_FALSE;
 
-    status = clPcieGetRootGenSpeed(pGpu, pCl, &genSpeed);
-    if (status != NV_OK)
+    if (!IS_PASSTHRU(pGpu))
+    {
+        status = clPcieGetRootGenSpeed(pGpu, pCl, &genSpeed);
+
+        if (status != NV_OK)
+        {
+            NV_PRINTF(LEVEL_ERROR, "Could not get root gen speed - check for GPU gen speed!\n");
+        }
+    }
+
+    if (IS_PASSTHRU(pGpu) || status != NV_OK)
     {
-        NV_PRINTF(LEVEL_ERROR, "Could not get root gen speed - check for GPU gen speed!\n");
         // Check for GPU gen speed
         if (GPU_BUS_CFG_CYCLE_RD32(pGpu, NV_EP_PCFG_GPU_LINK_CONTROL_STATUS, &busSpeed) != NV_OK)
         {
@@ -432,6 +440,7 @@ kceIsGenXorHigherSupported_GH100
         }
         genSpeed = GPU_DRF_VAL(_EP_PCFG_GPU, _LINK_CONTROL_STATUS, _CURRENT_LINK_SPEED, busSpeed);
     }
+
     NV_PRINTF(LEVEL_INFO, "Gen Speed = %d\n", genSpeed);
 
     if ((genSpeed >= checkGen))
@@ -1158,8 +1167,9 @@ NV_STATUS kceGetP2PCes_GH100(KernelCE *pKCe, OBJGPU *pGpu, NvU32 gpuMask, NvU32
         // there is a tie, then LCE with the lower index is preferred.
         //
         KCE_ITER_BEGIN(pGpu, pKCe, pKCeLoop, minP2PLce)
+
             NvU32 localMaxPcePerHshub = 0;
-            KernelCE *localMaxLcePerHshub;
+            KernelCE *localMaxLcePerHshub = NULL;
             NvU32 localMaxHshub = NV_CE_MAX_HSHUBS;
 
             // if LCE is stubbed or LCE is already assigned to another peer
@@ -1245,3 +1255,31 @@ NV_STATUS kceGetP2PCes_GH100(KernelCE *pKCe, OBJGPU *pGpu, NvU32 gpuMask, NvU32
 
     return NV_OK;
 }
+
+/*! Determine if CE support confidential compute secure copy */
+NvBool kceIsSecureCe_GH100(OBJGPU *pGpu, KernelCE *pKCe)
+{
+    NV2080_CTRL_CE_GET_CAPS_V2_PARAMS physicalCaps;
+
+    // Only check LCEs mapped to a physical CE
+    // TODO Bug 5132562
+    if (pKCe->bStubbed)
+    {
+        NV_PRINTF(LEVEL_INFO, "Skipping stubbed CE %d\n", pKCe->publicID);
+        return NV_FALSE;
+    }
+
+    portMemSet(&physicalCaps, 0, sizeof(physicalCaps));
+    physicalCaps.ceEngineType = NV2080_ENGINE_TYPE_COPY(pKCe->publicID);
+
+    RM_API *pRmApi = GPU_GET_PHYSICAL_RMAPI(pGpu);
+    NV_ASSERT_OK_OR_RETURN(pRmApi->Control(pRmApi,
+                           pGpu->hInternalClient,
+                           pGpu->hInternalSubdevice,
+                           NV2080_CTRL_CMD_CE_GET_PHYSICAL_CAPS,
+                           &physicalCaps,
+                           sizeof(physicalCaps)));
+
+    return (NV2080_CTRL_CE_GET_CAP(physicalCaps.capsTbl, NV2080_CTRL_CE_CAPS_CE_CC_SECURE) != 0) ?
+               NV_TRUE : NV_FALSE;
+};
