@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -40,6 +40,17 @@
 #endif
 
 #define ASYNC_FSP_POLL_PERIOD_MS 50
+//
+// FSP communication uses two headers: MCTP per packet and NVDM per message.
+// Packet size is based on the underlying transport mechanism which can be
+// queried with kfspGetMax[Send,Recv]PacketSize. If a message is larger than
+// can fit in a single packet, only the first packet contains the NVDM header
+// while all packets contain the MCTP header. Each of these headers is one
+// 32-bit DWORD and these indexes are used when placing the DWORD in the packet.
+//
+#define HEADER_DWORD_MCTP 0
+#define HEADER_DWORD_NVDM 1
+#define HEADER_DWORD_MAX  2
 
 /*!
 * Local object related functions
@@ -143,8 +154,45 @@ kfspInitRegistryOverrides
                                "during boot is disabled using the regkey.\n");
         pKernelFsp->setProperty(pKernelFsp, PDB_PROP_KFSP_FSP_FUSE_ERROR_CHECK_ENABLED, NV_FALSE);
     }
+
 }
 
+void
+kfspReleaseProxyImage_IMPL
+(
+    OBJGPU    *pGpu,
+    KernelFsp *pKernelFsp
+)
+{
+    if (pKernelFsp->pSysmemFrtsMemdesc != NULL)
+    {
+        kfspFrtsSysmemLocationClear_HAL(pGpu, pKernelFsp);
+        memdescUnmap(pKernelFsp->pSysmemFrtsMemdesc, NV_TRUE,
+            memdescGetKernelMapping(pKernelFsp->pSysmemFrtsMemdesc),
+            memdescGetKernelMappingPriv(pKernelFsp->pSysmemFrtsMemdesc));
+        memdescFree(pKernelFsp->pSysmemFrtsMemdesc);
+        memdescDestroy(pKernelFsp->pSysmemFrtsMemdesc);
+        pKernelFsp->pSysmemFrtsMemdesc = NULL;
+    }
+
+    // With Keep WPR feature, keep the GSP-FMC and BootArgsMemdesc.
+    if (!(pGpu->getProperty(pGpu, PDB_PROP_GPU_KEEP_WPR_ACROSS_GC6_SUPPORTED) && IS_GPU_GC6_STATE_ENTERING(pGpu)))
+    {
+        if (pKernelFsp->pGspFmcMemdesc != NULL)
+        {
+            memdescFree(pKernelFsp->pGspFmcMemdesc);
+            memdescDestroy(pKernelFsp->pGspFmcMemdesc);
+            pKernelFsp->pGspFmcMemdesc = NULL;
+        }
+
+        if (pKernelFsp->pGspBootArgsMemdesc != NULL)
+        {
+            memdescFree(pKernelFsp->pGspBootArgsMemdesc);
+            memdescDestroy(pKernelFsp->pGspBootArgsMemdesc);
+            pKernelFsp->pGspBootArgsMemdesc = NULL;
+        }
+    }
+}
 
 /*!
  * @brief Clean up objects used when sending GSP-FMC and FRTS info to FSP
@@ -165,30 +213,7 @@ kfspCleanupBootState_IMPL
     portMemFree(pKernelFsp->pCotPayload);
     pKernelFsp->pCotPayload = NULL;
 
-    if (pKernelFsp->pSysmemFrtsMemdesc != NULL)
-    {
-        kfspFrtsSysmemLocationClear_HAL(pGpu, pKernelFsp);
-        memdescUnmap(pKernelFsp->pSysmemFrtsMemdesc, NV_TRUE, 0,
-            memdescGetKernelMapping(pKernelFsp->pSysmemFrtsMemdesc),
-            memdescGetKernelMappingPriv(pKernelFsp->pSysmemFrtsMemdesc));
-        memdescFree(pKernelFsp->pSysmemFrtsMemdesc);
-        memdescDestroy(pKernelFsp->pSysmemFrtsMemdesc);
-        pKernelFsp->pSysmemFrtsMemdesc = NULL;
-    }
-
-    if (pKernelFsp->pGspFmcMemdesc != NULL)
-    {
-        memdescFree(pKernelFsp->pGspFmcMemdesc);
-        memdescDestroy(pKernelFsp->pGspFmcMemdesc);
-        pKernelFsp->pGspFmcMemdesc = NULL;
-    }
-
-    if (pKernelFsp->pGspBootArgsMemdesc != NULL)
-    {
-        memdescFree(pKernelFsp->pGspBootArgsMemdesc);
-        memdescDestroy(pKernelFsp->pGspBootArgsMemdesc);
-        pKernelFsp->pGspBootArgsMemdesc = NULL;
-    }
+    kfspReleaseProxyImage(pGpu, pKernelFsp);
 
     if (pKernelFsp->bClockBoostSupported)
     {
@@ -202,6 +227,24 @@ kfspCleanupBootState_IMPL
     }
 }
 
+/*!
+ * @brief Unload FSP state
+ *
+ * @param[in]  pGpu        GPU object pointer
+ * @param[in]  pKernelFsp  FSP object pointer
+ * @param[in]  flags
+ */
+NV_STATUS
+kfspStateUnload_IMPL
+(
+    OBJGPU    *pGpu,
+    KernelFsp *pKernelFsp,
+    NvU32      flags
+)
+{
+    return NV_OK;
+}
+
 /*!
  * @brief Destroy FSP state
  *
@@ -223,33 +266,32 @@ kfspStateDestroy_IMPL
         memdescDestroy(pKernelFsp->pVidmemFrtsMemdesc);
         pKernelFsp->pVidmemFrtsMemdesc = NULL;
     }
+
 }
 
-/*!
- * @brief Check if FSP RM command queue is empty
+/*
+ * @brief GpuWaitConditionFunc for FSP ready
  *
- * @param[in] pGpu       OBJGPU pointer
- * @param[in] pKernelFsp KernelFsp pointer
+ * @param[in] pGpu          GPU object pointer
+ * @param[in] pCondData     KernelFsp object pointer
  *
- * @return NV_TRUE if queue is empty, NV_FALSE otherwise
+ * @returns   NvBool        NV_TRUE if command and message fsp
+ *                          queues are empty
  */
-NvBool
-kfspIsQueueEmpty_IMPL
+static NvBool
+_kfspWaitForCanSend
 (
-    OBJGPU    *pGpu,
-    KernelFsp *pKernelFsp
+    OBJGPU *pGpu,
+    void   *pCondData
 )
 {
-    NvU32 cmdqHead, cmdqTail;
+    KernelFsp *pKernelFsp = (KernelFsp*) pCondData;
 
-    kfspGetQueueHeadTail_HAL(pGpu, pKernelFsp, &cmdqHead, &cmdqTail);
-
-    // FSP will set QUEUE_HEAD = TAIL after each packet is received
-    return (cmdqHead == cmdqTail);
+    return kfspCanSendPacket_HAL(pGpu, pKernelFsp);
 }
 
 /*!
- * @brief Wait for FSP RM command queue to be empty
+ * @brief Wait until RM can send to FSP
  *
  * @param[in] pGpu       OBJGPU pointer
  * @param[in] pKernelFsp KernelFsp pointer
@@ -257,7 +299,7 @@ kfspIsQueueEmpty_IMPL
  * @return NV_OK, or NV_ERR_TIMEOUT
  */
 NV_STATUS
-kfspPollForQueueEmpty_IMPL
+kfspPollForCanSend_IMPL
 (
     OBJGPU    *pGpu,
     KernelFsp *pKernelFsp
@@ -267,71 +309,20 @@ kfspPollForQueueEmpty_IMPL
     RMTIMEOUT timeout;
 
     gpuSetTimeout(pGpu, GPU_TIMEOUT_DEFAULT, &timeout,
-        GPU_TIMEOUT_FLAGS_OSTIMER |
-        GPU_TIMEOUT_FLAGS_BYPASS_THREAD_STATE);
+        GPU_TIMEOUT_FLAGS_OSTIMER);
 
-    while (!kfspIsQueueEmpty(pGpu, pKernelFsp))
+    status = gpuTimeoutCondWait(pGpu, _kfspWaitForCanSend, pKernelFsp, &timeout);
+    if (status != NV_OK)
     {
-        //
-        // For now we assume that any response from FSP before RM message
-        // send is complete indicates an error and we should abort.
-        //
-        // Ongoing dicussion on usefullness of this check. Bug to be filed.
-        //
-        if (!kfspIsMsgQueueEmpty(pGpu, pKernelFsp))
-        {
-            kfspReadMessage(pGpu, pKernelFsp, NULL, 0);
-            NV_PRINTF(LEVEL_ERROR,
-                "Received error message from FSP while waiting for CMDQ to be empty.\n");
-            status = NV_ERR_GENERIC;
-            break;
-        }
-
-        osSpinLoop();
-
-        status = gpuCheckTimeout(pGpu, &timeout);
-        if (status != NV_OK)
-        {
-            if ((status == NV_ERR_TIMEOUT) &&
-                kfspIsQueueEmpty(pGpu, pKernelFsp))
-            {
-                status = NV_OK;
-            }
-            else
-            {
-                NV_PRINTF(LEVEL_ERROR,
-                    "Timed out waiting for FSP command queue to be empty.\n");
-            }
-            break;
-        }
+        NV_PRINTF(LEVEL_ERROR,
+            "Timed out waiting for FSP queues to be empty.\n");
     }
 
     return status;
 }
 
 /*!
- * @brief Check if FSP RM message queue is empty
- *
- * @param[in] pGpu       OBJGPU pointer
- * @param[in] pKernelFsp KernelFsp pointer
- *
- * @return NV_TRUE if queue is empty, NV_FALSE otherwise
- */
-NvBool
-kfspIsMsgQueueEmpty_IMPL
-(
-    OBJGPU    *pGpu,
-    KernelFsp *pKernelFsp
-)
-{
-    NvU32 msgqHead, msgqTail;
-
-    kfspGetMsgQueueHeadTail_HAL(pGpu, pKernelFsp, &msgqHead, &msgqTail);
-    return (msgqHead == msgqTail);
-}
-
-/*!
- * @brief Poll for response from FSP via RM message queue
+ * @brief Poll for response from FSP
  *
  * @param[in] pGpu       OBJGPU pointer
  * @param[in] pKernelFsp KernelFsp pointer
@@ -367,7 +358,7 @@ kfspWaitForResponse
     NV_STATUS status = NV_OK;
 
     // Poll for message queue to wait for FSP's reply
-    while (kfspIsMsgQueueEmpty(pGpu, pKernelFsp))
+    while (!kfspIsResponseAvailable_HAL(pGpu, pKernelFsp))
     {
         osSpinLoop();
 
@@ -375,7 +366,7 @@ kfspWaitForResponse
         if (status != NV_OK)
         {
             if ((status == NV_ERR_TIMEOUT) &&
-                !kfspIsMsgQueueEmpty(pGpu, pKernelFsp))
+                kfspIsResponseAvailable_HAL(pGpu, pKernelFsp))
             {
                 status = NV_OK;
             }
@@ -404,7 +395,7 @@ kfspSetResponseTimeout
 )
 {
     gpuSetTimeout(pGpu, GPU_TIMEOUT_DEFAULT, &(pKernelFsp->rpcTimeout),
-                  GPU_TIMEOUT_FLAGS_OSTIMER | GPU_TIMEOUT_FLAGS_BYPASS_THREAD_STATE);
+                  GPU_TIMEOUT_FLAGS_OSTIMER);
 }
 
 /*!
@@ -446,42 +437,42 @@ kfspSendMessage
     NvU32      nvdmType
 )
 {
-    NvU32 dataSent, dataRemaining;
+    NvU32 dataSent;
+    NvU32 dataRemaining;
     NvU32 packetPayloadCapacity;
     NvU32 curPayloadSize;
     NvU32 headerSize;
-    NvU32 fspEmemRmChannelSize;
+    NvU32 *pHeader;
+    NvU32 maxPacketSize;
     NvBool bSinglePacket;
     NV_STATUS status;
     NvU8 *pBuffer = NULL;
-    NvU8  seq = 0;
-    NvU8  seid = 0;
-
-    // Allocate buffer of same size as channel
-    fspEmemRmChannelSize = kfspGetRmChannelSize_HAL(pGpu, pKernelFsp);
-    pBuffer = portMemAllocNonPaged(fspEmemRmChannelSize);
-    NV_CHECK_OR_RETURN(LEVEL_ERROR, pBuffer != NULL, NV_ERR_NO_MEMORY);
-    portMemSet(pBuffer, 0, fspEmemRmChannelSize);
+    NvU8 seq = 0;
+    NvU8 seid = 0;
 
     //
     // Check if message will fit in single packet
     // We lose 2 DWORDS to MCTP and NVDM headers
     //
-    headerSize = 2 * sizeof(NvU32);
-    packetPayloadCapacity = fspEmemRmChannelSize - headerSize;
+    headerSize = sizeof(NvU32) * HEADER_DWORD_MAX;
+    maxPacketSize = kfspGetMaxSendPacketSize_HAL(pGpu, pKernelFsp);
+    packetPayloadCapacity = maxPacketSize - headerSize;
     bSinglePacket = (size <= packetPayloadCapacity);
-
+    // Allocate buffer to hold a full packet
+    pBuffer = portMemAllocNonPaged(maxPacketSize);
+    NV_CHECK_OR_RETURN(LEVEL_ERROR, pBuffer != NULL, NV_ERR_NO_MEMORY);
+    portMemSet(pBuffer, 0, maxPacketSize);
+    pHeader = (NvU32*)pBuffer;
     // First packet
     seid = kfspNvdmToSeid_HAL(pGpu, pKernelFsp, nvdmType);
     // SOM=1,EOM=?,SEID,SEQ=0
-    ((NvU32 *)pBuffer)[0] = kfspCreateMctpHeader_HAL(pGpu, pKernelFsp, 1,
+    pHeader[HEADER_DWORD_MCTP] = kfspCreateMctpHeader_HAL(pGpu, pKernelFsp, 1,
                                                      (NvU8)bSinglePacket, seid, seq);
-    ((NvU32 *)pBuffer)[1] = kfspCreateNvdmHeader_HAL(pGpu, pKernelFsp, nvdmType);
+    pHeader[HEADER_DWORD_NVDM] = kfspCreateNvdmHeader_HAL(pGpu, pKernelFsp, nvdmType);
 
     curPayloadSize = NV_MIN(size, packetPayloadCapacity);
     portMemCopy(pBuffer + headerSize, packetPayloadCapacity, pPayload, curPayloadSize);
-
-    status = kfspSendPacket(pGpu, pKernelFsp, pBuffer, curPayloadSize + headerSize);
+    status = kfspSendPacket_HAL(pGpu, pKernelFsp, pBuffer, curPayloadSize + headerSize);
     if (status != NV_OK)
     {
         goto failed;
@@ -493,22 +484,20 @@ kfspSendMessage
         dataSent = curPayloadSize;
         dataRemaining = size - dataSent;
         headerSize = sizeof(NvU32); // No longer need NVDM header
-        packetPayloadCapacity = fspEmemRmChannelSize - headerSize;
+        packetPayloadCapacity = maxPacketSize - headerSize;
 
         while (dataRemaining > 0)
         {
             NvBool bLastPacket = (dataRemaining <= packetPayloadCapacity);
             curPayloadSize = (bLastPacket) ? dataRemaining : packetPayloadCapacity;
 
-            portMemSet(pBuffer, 0, fspEmemRmChannelSize);
-            ((NvU32 *)pBuffer)[0] = kfspCreateMctpHeader_HAL(pGpu, pKernelFsp, 0,
+            pHeader[HEADER_DWORD_MCTP] = kfspCreateMctpHeader_HAL(pGpu, pKernelFsp, 0,
                                                              (NvU8)bLastPacket, seid,
                                                              (++seq) % 4);
-
             portMemCopy(pBuffer + headerSize, packetPayloadCapacity,
                         pPayload + dataSent, curPayloadSize);
 
-            status = kfspSendPacket(pGpu, pKernelFsp, pBuffer, curPayloadSize + headerSize);
+            status = kfspSendPacket_HAL(pGpu, pKernelFsp, pBuffer, curPayloadSize + headerSize);
             if (status != NV_OK)
             {
                 goto failed;
@@ -517,7 +506,6 @@ kfspSendMessage
             dataSent += curPayloadSize;
             dataRemaining -= curPayloadSize;
         }
-
     }
 
 failed:
@@ -557,20 +545,21 @@ kfspReadMessage
     NvU8             *pPacketBuffer;
     NV_STATUS         status;
     NvU32             totalPayloadSize = 0;
+    NvU32             recvBufferSize;
     MCTP_PACKET_STATE packetState = MCTP_PACKET_STATE_START;
 
-    if (kfspIsMsgQueueEmpty(pGpu, pKernelFsp))
+    if (!kfspIsResponseAvailable_HAL(pGpu, pKernelFsp))
     {
-        NV_PRINTF(LEVEL_WARNING, "Tried to read FSP response but MSG queue is empty\n");
+        NV_PRINTF(LEVEL_WARNING, "Tried to read FSP response but none is available\n");
         return NV_OK;
     }
 
-    pPacketBuffer = portMemAllocNonPaged(kfspGetRmChannelSize_HAL(pGpu, pKernelFsp));
+    recvBufferSize = kfspGetMaxRecvPacketSize(pGpu, pKernelFsp);
+    pPacketBuffer = portMemAllocNonPaged(recvBufferSize);
     NV_CHECK_OR_RETURN(LEVEL_ERROR, pPacketBuffer != NULL, NV_ERR_NO_MEMORY);
 
     while ((packetState != MCTP_PACKET_STATE_END) && (packetState != MCTP_PACKET_STATE_SINGLE_PACKET))
     {
-        NvU32 msgqHead, msgqTail;
         NvU32 packetSize;
         NvU32 curPayloadSize;
         NvU8  curHeaderSize;
@@ -583,20 +572,9 @@ kfspReadMessage
             goto done;
         }
 
-        kfspGetMsgQueueHeadTail_HAL(pGpu, pKernelFsp, &msgqHead, &msgqTail);
-
-        // Tail points to last DWORD in packet, not DWORD immediately following it
-        packetSize = (msgqTail - msgqHead) + sizeof(NvU32);
-
-        if ((packetSize < sizeof(NvU32)) ||
-            (packetSize > kfspGetRmChannelSize_HAL(pGpu, pKernelFsp)))
-        {
-            NV_PRINTF(LEVEL_ERROR, "FSP response packet is invalid size: size=0x%x bytes\n", packetSize);
-            status = NV_ERR_INVALID_DATA;
-            goto done;
-        }
-
-        kfspReadFromEmem_HAL(pGpu, pKernelFsp, pPacketBuffer, packetSize);
+        NV_CHECK_OK_OR_GOTO(status, LEVEL_ERROR,
+                kfspReadPacket_HAL(pGpu, pKernelFsp, pPacketBuffer, recvBufferSize, &packetSize),
+                done);
 
         status = kfspGetPacketInfo_HAL(pGpu, pKernelFsp, pPacketBuffer, packetSize, &packetState, &tag);
         if (status != NV_OK)
@@ -632,13 +610,10 @@ kfspReadMessage
                 status = NV_ERR_INSUFFICIENT_RESOURCES;
                 goto done;
             }
-            portMemCopy(pPayloadBuffer + totalPayloadSize, payloadBufferSize - totalPayloadSize ,
+            portMemCopy(pPayloadBuffer + totalPayloadSize, payloadBufferSize - totalPayloadSize,
                         pPacketBuffer + curHeaderSize, curPayloadSize);
         }
         totalPayloadSize += curPayloadSize;
-
-        // Set TAIL = HEAD to indicate CPU received message
-        kfspUpdateMsgQueueHeadTail_HAL(pGpu, pKernelFsp, msgqHead, msgqHead);
     }
 
     NvU8 *pMessagePayload = (pPayloadBuffer == NULL) ? (pPacketBuffer + sizeof(MCTP_HEADER)) : pPayloadBuffer;
@@ -651,53 +626,7 @@ done:
 }
 
 /*!
- * @brief Send one MCTP packet to FSP via EMEM
- *
- * @param[in] pGpu          OBJGPU pointer
- * @param[in] pKernelFsp    KernelFsp pointer
- * @param[in] pPacket       MCTP packet
- * @param[in] packetSize    MCTP packet size in bytes
- *
- * @return NV_OK, or NV_ERR_INSUFFICIENT_RESOURCES
- */
-NV_STATUS
-kfspSendPacket_IMPL
-(
-    OBJGPU      *pGpu,
-    KernelFsp   *pKernelFsp,
-    NvU8        *pPacket,
-    NvU32        packetSize
-)
-{
-    NvU32 paddedSize;
-    NvU8 *pBuffer = NULL;
-    NV_STATUS status = NV_OK;
-
-    // Check that queue is ready to receive data
-    status = kfspPollForQueueEmpty(pGpu, pKernelFsp);
-    if (status != NV_OK)
-    {
-        return NV_ERR_INSUFFICIENT_RESOURCES;
-    }
-
-    // Pad to align size to 4-bytes boundary since EMEMC increments by DWORDS
-    paddedSize = NV_ALIGN_UP(packetSize, sizeof(NvU32));
-    pBuffer = portMemAllocNonPaged(paddedSize);
-    NV_CHECK_OR_RETURN(LEVEL_ERROR, pBuffer != NULL, NV_ERR_NO_MEMORY);
-    portMemSet(pBuffer, 0, paddedSize);
-    portMemCopy(pBuffer, paddedSize, pPacket, paddedSize);
-
-    kfspWriteToEmem_HAL(pGpu, pKernelFsp, pBuffer, paddedSize);
-
-    // Update HEAD and TAIL with new EMEM offset; RM always starts at offset 0.
-    kfspUpdateQueueHeadTail_HAL(pGpu, pKernelFsp, 0, paddedSize - sizeof(NvU32));
-
-    portMemFree(pBuffer);
-    return status;
-}
-
-/*!
- * @brief Send a MCTP message to FSP via EMEM, and read response
+ * @brief Send a MCTP message to FSP and read response
  *
  *
  * Response payload buffer is optional if response fits in a single packet.
@@ -840,11 +769,14 @@ kfspPollForAsyncResponse
 
     status = kfspCheckResponseTimeout(pGpu, pKernelFsp);
 
-    if (!kfspIsMsgQueueEmpty(pGpu, pKernelFsp))
+    if (kfspIsResponseAvailable_HAL(pGpu, pKernelFsp))
     {
-        status = osQueueWorkItemWithFlags(pGpu, kfspProcessAsyncResponseCallback, NULL,
-                                          OS_QUEUE_WORKITEM_FLAGS_LOCK_SEMA |
-                                          OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_DEVICE);
+        status = osQueueWorkItem(
+            pGpu,
+            kfspProcessAsyncResponseCallback,
+            NULL,
+            OS_QUEUE_WORKITEM_FLAGS_LOCK_SEMA |
+                OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_DEVICE);
         if (status != NV_OK)
         {
             NV_PRINTF(LEVEL_ERROR, "Failed to schedule work item, status=%x\n", status);
