@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -23,17 +23,40 @@
 
 #include "gpu/hwpm/profiler_v2.h"
 #include "gpu/hwpm/kern_hwpm.h"
+#include "rmapi/rs_utils.h"
 #include "vgpu/rpc.h"
+#include "kernel/gpu/fifo/kernel_channel_group_api.h"
+#include "kernel/gpu/fifo/kernel_channel_group.h"
 
 static NV_INLINE NvBool
-_isDeviceProfilingPermitted(OBJGPU *pGpu, ProfilerBase *pProf, API_SECURITY_INFO *pSecInfo)
+_isNonAdminProfilingPermitted(OBJGPU *pGpu)
 {
+    // Any non-priv clients with RS_ACCESS_PERFMON capability are allowed
+    if (osCheckAccess(RS_ACCESS_PERFMON))
+    {
+        return NV_TRUE;
+    }
+
+    // Otherwise, allowed only if profiling is deprivileged for all users
+    if (!gpuIsRmProfilingPrivileged(pGpu))
+    {
+        return NV_TRUE;
+    }
+
+    return NV_FALSE;
+}
+
+static NV_INLINE NvBool
+_isProfilingPermitted(OBJGPU *pGpu, ProfilerBase *pProf, API_SECURITY_INFO *pSecInfo)
+{
+    // Admins are always allowed to access device profiling
     if (pSecInfo->privLevel >= RS_PRIV_LEVEL_USER_ROOT)
     {
         return NV_TRUE;
     }
 
-    if (!gpuIsRmProfilingPrivileged(pGpu))
+    // Non-priv clients may device profile only in special cases
+    if (_isNonAdminProfilingPermitted(pGpu))
     {
         return NV_TRUE;
     }
@@ -254,7 +277,7 @@ profilerBaseDestructState_VF
 
         if (pProf->pPmaStreamList[pmaChIdx].pNumBytesCpuAddr != NvP64_NULL )
         {
-            memdescUnmap(pProf->pPmaStreamList[pmaChIdx].pNumBytesBufDesc, NV_TRUE, osGetCurrentProcess(),
+            memdescUnmap(pProf->pPmaStreamList[pmaChIdx].pNumBytesBufDesc, NV_TRUE,
                          pProf->pPmaStreamList[pmaChIdx].pNumBytesCpuAddr,
                          pProf->pPmaStreamList[pmaChIdx].pNumBytesCpuAddrPriv);
         }
@@ -280,9 +303,23 @@ profilerDevConstruct_IMPL
     RS_RES_ALLOC_PARAMS_INTERNAL *pParams
 )
 {
+    OBJSYS                      *pSys             = SYS_GET_INSTANCE();
+    ProfilerBase                *pProfBase        = staticCast(pProfDev, ProfilerBase);
     PROFILER_CLIENT_PERMISSIONS clientPermissions = {0};
+    NVB2CC_ALLOC_PARAMETERS *pAllocParams = pParams->pAllocParams;
 
-    if (!profilerDevQueryCapabilities_HAL(pProfDev, pCallContext, pParams,
+    if (pSys->getProperty(pSys, PDB_PROP_SYS_VALIDATE_CLIENT_HANDLE) &&
+        ((pParams->pSecInfo)->privLevel < RS_PRIV_LEVEL_USER_ROOT) && 
+        (pAllocParams->hClientTarget != 0))
+    {
+        NV_STATUS rmStatus;
+
+        rmStatus = osValidateClientTokens((void*)rmclientGetSecurityTokenByHandle(pCallContext->pClient->hClient),
+                                          (void*)rmclientGetSecurityTokenByHandle(pAllocParams->hClientTarget));
+        NV_CHECK_OR_RETURN(LEVEL_NOTICE, rmStatus == NV_OK, rmStatus);
+    }
+
+    if (!profilerBaseQueryCapabilities_HAL(pProfBase, pCallContext, pParams,
                                             &clientPermissions))
     {
         return NV_ERR_INSUFFICIENT_PERMISSIONS;
@@ -292,22 +329,27 @@ profilerDevConstruct_IMPL
 }
 
 NvBool
-profilerDevQueryCapabilities_IMPL
+profilerBaseQueryCapabilities_IMPL
 (
-    ProfilerDev *pProfDev,
+    ProfilerBase *pProfBase,
     CALL_CONTEXT *pCallContext,
     RS_RES_ALLOC_PARAMS_INTERNAL *pParams,
     PROFILER_CLIENT_PERMISSIONS *pClientPermissions
 )
 {
-    OBJGPU              *pGpu                   = GPU_RES_GET_GPU(pProfDev);
-    ProfilerBase        *pProfBase              = staticCast(pProfDev, ProfilerBase);
+    OBJGPU              *pGpu                   = GPU_RES_GET_GPU(pProfBase);
     API_SECURITY_INFO   *pSecInfo               = pParams->pSecInfo;
     NvBool               bAnyProfilingPermitted = NV_FALSE;
+    KernelHwpm *pKernelHwpm = GPU_GET_KERNEL_HWPM(pGpu);
 
-    pClientPermissions->bMemoryProfilingPermitted =
-        _isMemoryProfilingPermitted(pGpu, pProfBase);
+    // SYS memory profiling is permitted if video memory profiling is permitted.
+    pClientPermissions->bVideoMemoryProfilingPermitted = _isMemoryProfilingPermitted(pGpu, pProfBase);
+    pClientPermissions->bSysMemoryProfilingPermitted = pClientPermissions->bVideoMemoryProfilingPermitted;
 
+    //
+    // bAdminProfilingPermitted controls access to privileged profiling registers.
+    // This is admin-only; no exceptions, not even CAP_PERFMON
+    //
     pClientPermissions->bAdminProfilingPermitted = NV_FALSE;
     if (pSecInfo->privLevel >= RS_PRIV_LEVEL_USER_ROOT)
     {
@@ -315,10 +357,15 @@ profilerDevQueryCapabilities_IMPL
         pClientPermissions->bAdminProfilingPermitted = NV_TRUE;
     }
 
-    pClientPermissions->bDevProfilingPermitted =
-        _isDeviceProfilingPermitted(pGpu, pProfBase, pSecInfo);
+    pClientPermissions->bDevProfilingPermitted = _isProfilingPermitted(pGpu, pProfBase, pSecInfo);
+    pClientPermissions->bCtxProfilingPermitted = _isProfilingPermitted(pGpu, pProfBase, pSecInfo);
 
-    if (pClientPermissions->bDevProfilingPermitted)
+    if (pClientPermissions->bDevProfilingPermitted || pClientPermissions->bCtxProfilingPermitted)
+    {
+        bAnyProfilingPermitted = NV_TRUE;
+    }
+
+    if (pKernelHwpm->getProperty(pKernelHwpm, PDB_PROP_KHWPM_HES_CWD_SUPPORTED) && !IS_VIRTUAL(pGpu))
     {
         bAnyProfilingPermitted = NV_TRUE;
     }
@@ -458,7 +505,36 @@ profilerDevConstructStateInterlude_IMPL
 
     params.bDevProfilingPermitted = clientPermissions.bDevProfilingPermitted;
     params.bAdminProfilingPermitted = clientPermissions.bAdminProfilingPermitted;
-    params.bMemoryProfilingPermitted = clientPermissions.bMemoryProfilingPermitted;
+    params.bVideoMemoryProfilingPermitted = clientPermissions.bVideoMemoryProfilingPermitted;
+    params.bSysMemoryProfilingPermitted = clientPermissions.bSysMemoryProfilingPermitted;
+
+    return pRmApi->Control(pRmApi,
+                           hClient,
+                           hObject,
+                           NVB0CC_CTRL_CMD_INTERNAL_PERMISSIONS_INIT,
+                           &params, sizeof(params));
+}
+
+NV_STATUS
+profilerCtxConstructStateInterlude_IMPL
+(
+    ProfilerCtx *pProfCtx,
+    CALL_CONTEXT *pCallContext,
+    RS_RES_ALLOC_PARAMS_INTERNAL *pAllocParams,
+    PROFILER_CLIENT_PERMISSIONS clientPermissions
+)
+{
+    OBJGPU         *pGpu        = GPU_RES_GET_GPU(pProfCtx);
+    RM_API         *pRmApi      = GPU_GET_PHYSICAL_RMAPI(pGpu);
+    NvHandle        hClient     = RES_GET_CLIENT_HANDLE(pProfCtx);
+    NvHandle        hObject     = RES_GET_HANDLE(pProfCtx);
+
+    NVB0CC_CTRL_INTERNAL_PERMISSIONS_INIT_PARAMS params = {0};
+
+    params.bCtxProfilingPermitted = clientPermissions.bCtxProfilingPermitted;
+    params.bAdminProfilingPermitted = clientPermissions.bAdminProfilingPermitted;
+    params.bVideoMemoryProfilingPermitted = clientPermissions.bVideoMemoryProfilingPermitted;
+    params.bSysMemoryProfilingPermitted = clientPermissions.bSysMemoryProfilingPermitted;
 
     return pRmApi->Control(pRmApi,
                            hClient,
@@ -475,13 +551,10 @@ profilerDevConstructStateEpilogue_FWCLIENT
     RS_RES_ALLOC_PARAMS_INTERNAL *pParams
 )
 {
-    ProfilerBase *pProfBase = staticCast(pProfDev, ProfilerBase);
     RsResourceRef *pParentRef = pCallContext->pResourceRef->pParentRef;
 
     NV_ASSERT_OR_RETURN((pParentRef->internalClassId == classId(Subdevice)), NV_ERR_INVALID_OBJECT_PARENT);
 
-    pProfBase->hSubDevice = pParentRef->hResource;
-
     return NV_OK;
 }
 
@@ -515,3 +588,118 @@ profilerDevDestructState_FWCLIENT
 
     NV_RM_RPC_FREE(pGpu, hClient, hParent, hObject, status);
 }
+
+NV_STATUS
+profilerCtxConstructState_IMPL
+(
+    ProfilerCtx *pProfCtx,
+    CALL_CONTEXT *pCallContext,
+    RS_RES_ALLOC_PARAMS_INTERNAL *pAllocParams,
+    PROFILER_CLIENT_PERMISSIONS clientPermissions
+)
+{
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR, profilerCtxConstructStatePrologue_HAL(pProfCtx,
+                            pCallContext, pAllocParams));
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR, profilerCtxConstructStateInterlude_HAL(pProfCtx,
+                            pCallContext, pAllocParams, clientPermissions));
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR, profilerCtxConstructStateEpilogue_HAL(pProfCtx,
+                            pCallContext, pAllocParams));
+    return NV_OK;
+}
+
+NV_STATUS
+profilerCtxConstruct_IMPL
+(
+    ProfilerCtx *pProfCtx,
+    CALL_CONTEXT *pCallContext,
+    RS_RES_ALLOC_PARAMS_INTERNAL *pParams
+)
+{
+    OBJGPU            *pGpu        = GPU_RES_GET_GPU(pProfCtx);
+    KernelHwpm        *pKernelHwpm = GPU_GET_KERNEL_HWPM(pGpu);
+    ProfilerBase      *pProfBase   = staticCast(pProfCtx, ProfilerBase);
+    RsResourceRef     *pParentRef  = pCallContext->pResourceRef->pParentRef;
+    RmClient          *pClient     = serverutilGetClientUnderLock(pCallContext->pClient->hClient);
+    PROFILER_CLIENT_PERMISSIONS clientPermissions = {0};
+
+    if (!pKernelHwpm->getProperty(pKernelHwpm, PDB_PROP_KHWPM_PROFILING_B1CC_SUPPORTED))
+    {
+        NV_PRINTF(LEVEL_ERROR, "Context level profiler is not supported\n");
+        return NV_ERR_NOT_SUPPORTED;
+    }
+
+    if (!profilerBaseQueryCapabilities_HAL(pProfBase, pCallContext, pParams, &clientPermissions))
+    {
+        return NV_ERR_INSUFFICIENT_PERMISSIONS;
+    }
+
+    //
+    // Bug: 5225901
+    // Restrict B1CC HES the profiler session to user with profiling permission
+    // when TSG is shared between multiple processes (MPS mode).
+    //
+    if (pParentRef->internalClassId == classId(KernelChannelGroupApi) && pClient)
+    {
+        API_SECURITY_INFO   *pSecInfo   = pParams->pSecInfo;
+
+        if (!_isProfilingPermitted(pGpu, pProfBase, pSecInfo))
+        {
+            CHANNEL_NODE   *pChanNode;
+            KernelChannelGroupApi *pKernelChannelGroupApi = dynamicCast(pParentRef->pResource,
+                                                                KernelChannelGroupApi);
+            KernelChannelGroup *pKernelChannelGroup = pKernelChannelGroupApi->pKernelChannelGroup;
+
+            for (pChanNode = pKernelChannelGroup->pChanList->pHead; pChanNode; pChanNode = pChanNode->pNext)
+            {
+                KernelChannel *pChannel = pChanNode->pKernelChannel;
+
+                NV_CHECK_OR_RETURN(LEVEL_NOTICE, (pClient->ProcID == pChannel->ProcessID),  NV_ERR_INSUFFICIENT_PERMISSIONS);
+            }
+        }
+    }
+
+    return profilerCtxConstructState(pProfCtx, pCallContext, pParams, clientPermissions);
+}
+
+NV_STATUS
+profilerCtxConstructStatePrologue_FWCLIENT
+(
+    ProfilerCtx *pProfCtx,
+    CALL_CONTEXT *pCallContext,
+    RS_RES_ALLOC_PARAMS_INTERNAL *pParams
+)
+{
+    OBJGPU     *pGpu        = GPU_RES_GET_GPU(pProfCtx);
+    NvHandle    hClient     = RES_GET_CLIENT_HANDLE(pProfCtx);
+    NvHandle    hParent     = RES_GET_PARENT_HANDLE(pProfCtx);
+    NvHandle    hObject     = RES_GET_HANDLE(pProfCtx);
+    NvU32       class       = RES_GET_EXT_CLASS_ID(pProfCtx);
+    NV_STATUS   status      = NV_OK;
+
+    NV_RM_RPC_ALLOC_OBJECT(pGpu, hClient, hParent, hObject, class,
+        pParams->pAllocParams, pParams->paramsSize, status);
+
+    return status;
+}
+
+void
+profilerCtxDestruct_FWCLIENT
+(
+    ProfilerCtx *pProfCtx
+)
+{
+    NvHandle                     hClient;
+    NvHandle                     hParent;
+    NvHandle                     hObject;
+    RS_RES_FREE_PARAMS_INTERNAL *pParams;
+    CALL_CONTEXT                *pCallContext;
+    OBJGPU                      *pGpu            = GPU_RES_GET_GPU(pProfCtx);
+    NV_STATUS                    status          = NV_OK;
+
+    resGetFreeParams(staticCast(pProfCtx, RsResource), &pCallContext, &pParams);
+    hClient = pCallContext->pClient->hClient;
+    hParent = pCallContext->pResourceRef->pParentRef->hResource;
+    hObject = pCallContext->pResourceRef->hResource;
+
+    NV_RM_RPC_FREE(pGpu, hClient, hParent, hObject, status);
+}
