@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -26,12 +26,15 @@
 #include "gpu/mem_mgr/mem_mgr.h"
 #include "gpu/mem_sys/kern_mem_sys.h"
 #include "gpu/mem_mgr/heap.h"
+#include "gpu/uvm/uvm.h"
+#include "gpu/mem_mgr/phys_mem_allocator/phys_mem_allocator.h"
 #include "gpu/bus/kern_bus.h"
 #include "kernel/gpu/mig_mgr/kernel_mig_manager.h"
 #include "gpu/mem_mgr/mem_desc.h"
 #include "gpu/mem_mgr/virt_mem_allocator.h"
 #include "gpu/gpu_resource_desc.h"
-#include "gpu/subdevice/subdevice.h"
+#include "kernel/gpu/device/device.h"
+#include "kernel/gpu/subdevice/subdevice.h"
 #include "platform/chipset/chipset.h"
 #include "ctrl/ctrl0080/ctrl0080fb.h"
 #include "ctrl/ctrl2080/ctrl2080fb.h"
@@ -127,6 +130,9 @@ kmemsysGetFbInfos_VF(OBJGPU *pGpu, KernelMemorySystem *pKernelMemorySystem, RsCl
     return NV_OK;
 }
 
+// Clients expect to be able to pass indices 0..NV2080_CTRL_FB_INFO_INDEX_MAX in one call
+ct_assert(NV2080_CTRL_FB_INFO_INDEX_MAX < NV2080_CTRL_FB_INFO_MAX_LIST_SIZE);
+
 // Common logic for all runtimes to populate NV2080_CTRL_FB_INFO array
 static NV_STATUS
 _kmemsysGetFbInfos
@@ -163,7 +169,10 @@ _kmemsysGetFbInfos
 
     if (bIsMIGInUse)
     {
-        bIsClientMIGMonitor = !RMCFG_FEATURE_PLATFORM_GSP && rmclientIsCapableByHandle(pClient->hClient, NV_RM_CAP_SYS_SMC_MONITOR);
+        RmClient *pRmClient = dynamicCast(pClient, RmClient);
+        NV_ASSERT_OR_RETURN(pRmClient != NULL, NV_ERR_INVALID_CLIENT);
+
+        bIsClientMIGMonitor = !RMCFG_FEATURE_PLATFORM_GSP && rmclientIsCapable(pRmClient, NV_RM_CAP_SYS_SMC_MONITOR);
         bIsClientMIGProfiler = kmigmgrIsDeviceUsingDeviceProfiling(pGpu, pKernelMIGManager, pDevice);
     }
 
@@ -176,9 +185,12 @@ _kmemsysGetFbInfos
     // Populate indices with runtime-specific logic
     // Return early if this computed all requested indices
     //
-    status = kmemsysGetFbInfos_HAL(pGpu, pKernelMemorySystem, pClient, pDevice, hObject, pParams, &fbInfoListIndicesUnset);
-    if (status == NV_OK && nvPopCount64(fbInfoListIndicesUnset) == 0)
-        return NV_OK;
+    if (pKernelMemorySystem != NULL)
+    {
+        status = kmemsysGetFbInfos_HAL(pGpu, pKernelMemorySystem, pClient, pDevice, hObject, pParams, &fbInfoListIndicesUnset);
+        if (status == NV_OK && nvPopCount64(fbInfoListIndicesUnset) == 0)
+            return NV_OK;
+    }
 
     // Load the per-GPU instance heap if MIG is enabled.
     FOR_EACH_INDEX_IN_MASK(64, i, fbInfoListIndicesUnset)
@@ -335,7 +347,7 @@ _kmemsysGetFbInfos
                     NvU32 heapSizeKb;
                     if (bIsPmaEnabled)
                     {
-                        pmaGetTotalMemory(&pHeap->pmaObject, &bytesTotal);
+                        pmaGetTotalMemory(pHeap->pPmaObject, &bytesTotal);
                         NV_ASSERT(NvU64_HI32(bytesTotal >> 10) == 0);
                         heapSizeKb = NvU64_LO32(bytesTotal >> 10);
                     }
@@ -353,7 +365,7 @@ _kmemsysGetFbInfos
                 else
                 {
                     NV_ASSERT(0 == NvU64_HI32(pMemoryManager->Ram.fbTotalMemSizeMb << 10));
-                    data = NvU64_LO32(NV_MIN((pMemoryManager->Ram.fbTotalMemSizeMb << 10), 
+                    data = NvU64_LO32(NV_MIN((pMemoryManager->Ram.fbTotalMemSizeMb << 10),
                                              (pMemoryManager->Ram.fbOverrideSizeMb << 10))
                                              - pKernelMemorySystem->fbOverrideStartKb);
                     break;
@@ -374,7 +386,7 @@ _kmemsysGetFbInfos
                     NvU32 heapSizeKb;
                     if (bIsPmaEnabled)
                     {
-                        pmaGetTotalMemory(&pHeap->pmaObject, &bytesTotal);
+                        pmaGetTotalMemory(pHeap->pPmaObject, &bytesTotal);
                         NV_ASSERT(NvU64_HI32(bytesTotal >> 10) == 0);
                         heapSizeKb = NvU64_LO32(bytesTotal >> 10);
                     }
@@ -410,7 +422,7 @@ _kmemsysGetFbInfos
                     NvU32 heapSizeKb;
                     if (bIsPmaEnabled)
                     {
-                        pmaGetTotalMemory(&pHeap->pmaObject, &bytesTotal);
+                        pmaGetTotalMemory(pHeap->pPmaObject, &bytesTotal);
                         NV_ASSERT(NvU64_HI32(bytesTotal >> 10) == 0);
                         heapSizeKb = NvU64_LO32(bytesTotal >> 10);
                     }
@@ -435,7 +447,7 @@ _kmemsysGetFbInfos
             {
                 if (bIsPmaEnabled)
                 {
-                    pmaGetTotalMemory(&pHeap->pmaObject, &bytesTotal);
+                    pmaGetTotalMemory(pHeap->pPmaObject, &bytesTotal);
                     NV_ASSERT(NvU64_HI32(bytesTotal >> 10) == 0);
                     data = NvU64_LO32(bytesTotal >> 10);
                 }
@@ -464,7 +476,7 @@ _kmemsysGetFbInfos
                 {
                     if (bIsPmaEnabled)
                     {
-                        pmaGetLargestFree(&pHeap->pmaObject, &largestFree, &heapBase, &largestOffset);
+                        pmaGetLargestFree(pHeap->pPmaObject, &largestFree, &heapBase, &largestOffset);
                     }
                     else
                     {
@@ -489,7 +501,7 @@ _kmemsysGetFbInfos
                             data = NvU64_LO32(pKernelMemorySystem->fbOverrideStartKb);
                             NV_ASSERT_OR_ELSE((NvU64) data == pKernelMemorySystem->fbOverrideStartKb,
                                               status = NV_ERR_INVALID_DATA);
-                            
+
                         }
                         else
                         {
@@ -514,7 +526,7 @@ _kmemsysGetFbInfos
                     bytesFree = 0;
 
                     if (bIsPmaEnabled)
-                        pmaGetFreeMemory(&pHeap->pmaObject, &val);
+                        pmaGetFreeMemory(pHeap->pPmaObject, &val);
                     else
                         heapGetFree(pHeap, &val);
 
@@ -534,7 +546,7 @@ _kmemsysGetFbInfos
                         NvU32 config = PMA_QUERY_NUMA_ENABLED;
 
                         memmgrGetFreeMemoryForAllMIGGPUInstances(pGpu, pMemoryManager, &partTotalBytesFree);
-                        
+
                         //
                         // In the case of MIG+NUMA case(self hosted GPUs), NVOS32_ALLOC_FLAGS_FIXED_ADDRESS_ALLOCATE
                         // is not supported and hence the partition's memory is not accounted in the global PMA.
@@ -544,11 +556,11 @@ _kmemsysGetFbInfos
                         // partitions' free memory + (global total memory - all created partitions' total memory).
                         //
                         if (bIsPmaEnabled &&
-                            (pmaQueryConfigs(&pHeap->pmaObject, &config) == NV_OK) &&
+                            (pmaQueryConfigs(pHeap->pPmaObject, &config) == NV_OK) &&
                             (config & PMA_QUERY_NUMA_ENABLED))
                         {
                             memmgrGetTotalMemoryForAllMIGGPUInstances(pGpu, pMemoryManager, &partTotalBytes);
-                            pmaGetTotalMemory(&pHeap->pmaObject, &val);
+                            pmaGetTotalMemory(pHeap->pPmaObject, &val);
                             bytesFree = partTotalBytesFree + (val - partTotalBytes);
                         }
                         else
@@ -562,7 +574,7 @@ _kmemsysGetFbInfos
                 }
                 else if (bIsPmaEnabled)
                 {
-                    pmaGetFreeMemory(&pHeap->pmaObject, &bytesFree);
+                    pmaGetFreeMemory(pHeap->pPmaObject, &bytesFree);
 
                     NV_ASSERT(NvU64_HI32(bytesFree >> 10) == 0);
                     data = NvU64_LO32(bytesFree >> 10);
@@ -611,7 +623,7 @@ _kmemsysGetFbInfos
                 {
                     NvU32 heapSizeKb;
 
-                    pmaGetTotalMemory(&pHeap->pmaObject, &bytesTotal);
+                    pmaGetTotalMemory(pHeap->pPmaObject, &bytesTotal);
                     NV_ASSERT(NvU64_HI32(bytesTotal >> 10) == 0);
                     heapSizeKb = NvU64_LO32(bytesTotal >> 10);
 
@@ -655,7 +667,7 @@ _kmemsysGetFbInfos
             {
                 if (bIsPmaEnabled)
                 {
-                    pmaGetLargestFree(&pHeap->pmaObject, &largestFree, &heapBase, &largestOffset);
+                    pmaGetLargestFree(pHeap->pPmaObject, &largestFree, &heapBase, &largestOffset);
                 }
                 else
                 {
@@ -670,7 +682,7 @@ _kmemsysGetFbInfos
             {
                 if (bIsPmaEnabled)
                 {
-                    pmaGetLargestFree(&pHeap->pmaObject, &largestFree, &heapBase, &largestOffset);
+                    pmaGetLargestFree(pHeap->pPmaObject, &largestFree, &heapBase, &largestOffset);
                 }
                 else
                 {
@@ -685,7 +697,7 @@ _kmemsysGetFbInfos
             {
                 if (bIsPmaEnabled)
                 {
-                    pmaGetLargestFree(&pHeap->pmaObject, &largestFree, &heapBase, &largestOffset);
+                    pmaGetLargestFree(pHeap->pPmaObject, &largestFree, &heapBase, &largestOffset);
                 }
                 else
                 {
@@ -796,7 +808,7 @@ _kmemsysGetFbInfos
                 {
                     if (bIsPmaEnabled)
                     {
-                        pmaGetTotalProtectedMemory(&pHeap->pmaObject, &bytesTotal);
+                        pmaGetTotalProtectedMemory(pHeap->pPmaObject, &bytesTotal);
                         NV_ASSERT(NvU64_HI32(bytesTotal >> 10) == 0);
                         data = NvU64_LO32(bytesTotal >> 10);
                     }
@@ -823,7 +835,7 @@ _kmemsysGetFbInfos
                 {
                     if (bIsPmaEnabled)
                     {
-                        pmaGetFreeProtectedMemory(&pHeap->pmaObject, &bytesFree);
+                        pmaGetFreeProtectedMemory(pHeap->pPmaObject, &bytesFree);
                         NV_ASSERT(NvU64_HI32(bytesFree >> 10) == 0);
                         data = NvU64_LO32(bytesFree >> 10);
                     }
@@ -856,6 +868,64 @@ _kmemsysGetFbInfos
                 }
                 break;
             }
+            case NV2080_CTRL_FB_INFO_INDEX_ACCESS_COUNTER_BUFFER_COUNT:
+            {
+                OBJUVM *pUvm = GPU_GET_UVM(pGpu);
+
+                if (pUvm != NULL)
+                {
+                    data = pUvm->accessCounterBufferCount;
+                }
+                else
+                {
+                    data = 0;
+                }
+                break;
+            }
+
+            case NV2080_CTRL_FB_INFO_INDEX_COHERENCE_INFO:
+            {
+                MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
+
+                if (pMemoryManager != NULL)
+                {
+                    data = pMemoryManager->bPlatformFullyCoherent ?
+                        NV2080_CTRL_FB_INFO_INDEX_COHERENCE_INFO_FULLY_COHERENT :
+                        NV2080_CTRL_FB_INFO_INDEX_COHERENCE_INFO_NON_FULLY_COHERENT;
+                }
+                else
+                {
+                    data = 0;
+                }
+                break;
+            }
+
+            case NV2080_CTRL_FB_INFO_INDEX_NUMA_NODE_ID:
+            {
+                NvU32 nodeId = NV0000_CTRL_NO_NUMA_NODE;
+
+                if (bIsMIGInUse)
+                {
+                    MIG_INSTANCE_REF ref;
+
+                    if (kmigmgrGetInstanceRefFromDevice(pGpu, pKernelMIGManager, pDevice, &ref) == NV_OK)
+                    {
+                        NvU32 swizzId = ref.pKernelMIGGpuInstance->swizzId;
+
+                        if (pKernelMemorySystem->memPartitionNumaInfo[swizzId].bInUse)
+                        {
+                            nodeId = pKernelMemorySystem->memPartitionNumaInfo[swizzId].numaNodeId;
+                        }
+                    }
+                }
+                else
+                {
+                    nodeId = pGpu->numaNodeId;
+                }
+
+                data = nodeId;
+                break;
+            }
 
             default:
             {
@@ -945,7 +1015,8 @@ subdeviceCtrlCmdFbGetInfoV2_IMPL
     NvHandle hObject = RES_GET_HANDLE(pSubdevice);
     Device *pDevice = GPU_RES_GET_DEVICE(pSubdevice);
 
-    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmDeviceGpuLockIsOwner(pGpu->gpuInstance));
+    NV_ASSERT_OR_RETURN(rmapiLockIsOwner() && rmDeviceGpuLockIsOwner(pGpu->gpuInstance),
+        NV_ERR_INVALID_LOCK_STATE);
 
     if ((pFbInfoParams->fbInfoListSize > NV2080_CTRL_FB_INFO_MAX_LIST_SIZE) ||
         (pFbInfoParams->fbInfoListSize == 0))
@@ -956,117 +1027,6 @@ subdeviceCtrlCmdFbGetInfoV2_IMPL
     return _kmemsysGetFbInfos(pGpu, pClient, pDevice, hObject, pFbInfoParams);
 }
 
-//
-// subdeviceCtrlCmdFbGetCarveoutAddressInfo
-//
-// Lock Requirements:
-//      Assert that API and GPUs lock held on entry
-//
-NV_STATUS
-subdeviceCtrlCmdFbGetCarveoutAddressInfo_IMPL
-(
-    Subdevice *pSubdevice,
-    NV2080_CTRL_FB_GET_SYSTEM_CARVEOUT_ADDRESS_SPACE_INFO *pParams
-)
-{
-
-    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
-
-    pParams->StartAddr = 0x0;
-    pParams->SpaceSize = 0x0;
-
-    return NV_ERR_GENERIC;
-}
-
-//
-// subdeviceCtrlCmdFbSetGpuCacheAllocPolicy
-//
-// Lock Requirements:
-//      Assert that GPUs lock held on entry
-//      Called from SW method w/o API lock
-//
-NV_STATUS
-subdeviceCtrlCmdFbSetGpuCacheAllocPolicy_IMPL
-(
-    Subdevice *pSubdevice,
-    NV2080_CTRL_FB_GPU_CACHE_ALLOC_POLICY_PARAMS *pParams
-)
-{
-    LOCK_ASSERT_AND_RETURN(rmGpuLockIsOwner());
-
-    // Map engine to FBBA client
-    return NV_ERR_NOT_SUPPORTED;
-}
-
-//
-// subdeviceCtrlCmdFbGetGpuCacheAllocPolicy
-//
-// Lock Requirements:
-//      Assert that API and GPUs lock held on entry
-//
-NV_STATUS
-subdeviceCtrlCmdFbGetGpuCacheAllocPolicy_IMPL
-(
-    Subdevice *pSubdevice,
-    NV2080_CTRL_FB_GPU_CACHE_ALLOC_POLICY_PARAMS *pGpuCacheAllocPolicyParams
-)
-{
-
-    NV_STATUS status = NV_ERR_NOT_SUPPORTED;
-    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
-
-    pGpuCacheAllocPolicyParams->allocPolicy = 0;
-
-    NV_PRINTF(LEVEL_ERROR, "Failed to read Client reads ALLOC policy\n");
-    return status;
-}
-
-//
-// subdeviceCtrlCmdFbSetGpuCacheAllocPolicyV2
-//
-// Lock Requirements:
-//      Assert that GPUs lock held on entry
-//      Called from SW method w/o API lock
-//
-NV_STATUS
-subdeviceCtrlCmdFbSetGpuCacheAllocPolicyV2_IMPL
-(
-    Subdevice *pSubdevice,
-    NV2080_CTRL_FB_GPU_CACHE_ALLOC_POLICY_V2_PARAMS *pParams
-)
-{
-    CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
-    RmCtrlParams *pRmCtrlParams = pCallContext->pControlParams;
-    NV_STATUS status = NV_ERR_NOT_SUPPORTED;
-
-    // Bug 724186 -- Skip this check for deferred API
-    LOCK_ASSERT_AND_RETURN(pRmCtrlParams->bDeferredApi || rmGpuLockIsOwner());
-
-    NV_PRINTF(LEVEL_ERROR, "Failed to set alloc policy\n");
-    return status;
-}
-
-//
-// subdeviceCtrlCmdFbGetGpuCacheAllocPolicyV2
-//
-// Lock Requirements:
-//      Assert that API and GPUs lock held on entry
-//
-NV_STATUS
-subdeviceCtrlCmdFbGetGpuCacheAllocPolicyV2_IMPL
-(
-    Subdevice *pSubdevice,
-    NV2080_CTRL_FB_GPU_CACHE_ALLOC_POLICY_V2_PARAMS *pParams
-)
-{
-    NV_STATUS status = NV_ERR_NOT_SUPPORTED;
-
-    NV_ASSERT_OR_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner(), NV_ERR_INVALID_LOCK_STATE);
-
-    NV_PRINTF(LEVEL_ERROR, "Failed to get alloc policy.\n");
-    return status;
-}
-
 //
 // subdeviceCtrlCmdFbGetCliManagedOfflinedPages
 //
@@ -1093,13 +1053,13 @@ subdeviceCtrlCmdFbGetCliManagedOfflinedPages_IMPL
     NvU64               pageSize;
     NvU32               numChunks           = 0;
 
-    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
+    NV_ASSERT_OR_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner(), NV_ERR_INVALID_LOCK_STATE);
     if (!IsSLIEnabled(pGpu))
     {
         if (memmgrIsPmaInitialized(pMemoryManager))
         {
             // If PMA is enabled Client pages are located here.
-            pmaGetClientBlacklistedPages(&pHeap->pmaObject, chunks, &pageSize, &numChunks);
+            pmaGetClientBlacklistedPages(pHeap->pPmaObject, chunks, &pageSize, &numChunks);
 
             NV_ASSERT(numChunks <= NV2080_CTRL_FB_OFFLINED_PAGES_MAX_PAGES);
 
@@ -1154,13 +1114,13 @@ subdeviceCtrlCmdFbUpdateNumaStatus_IMPL
     OBJGPU             *pGpu                = GPU_RES_GET_GPU(pSubdevice);
     KernelMemorySystem *pKernelMemorySystem = GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu);
     Heap               *pHeap               = GPU_GET_HEAP(pGpu);
-    PMA                *pPma                = &pHeap->pmaObject;
+    PMA                *pPma                = pHeap->pPmaObject;
     NV_STATUS          status               = NV_OK;
 
     if (!RMCFG_FEATURE_PMA)
         return NV_ERR_NOT_SUPPORTED;
 
-    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
+    NV_ASSERT_OR_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner(), NV_ERR_INVALID_LOCK_STATE);
 
     if (pParams->bOnline)
     {
@@ -1206,7 +1166,8 @@ subdeviceCtrlCmdFbGetNumaInfo_IMPL
     NvU64                numaMemOffset = 0;
     NvU64                numaMemSize = 0;
 
-    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
+    NV_ASSERT_OR_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner(),
+        NV_ERR_INVALID_LOCK_STATE);
 
     if (pParams->numaOfflineAddressesCount >
         NV_ARRAY_ELEMENTS(pParams->numaOfflineAddresses))
@@ -1305,14 +1266,62 @@ subdeviceCtrlCmdFbGetNumaInfo_IMPL
     return status;
 }
 
+/*!
+ * @brief This call can be used to get static Bar1 related information.
+ *
+ * Lock Requirements:
+ *      Assert that API and GPUs lock held on entry
+ *
+ * @param[in] pSubdevice Subdevice
+ * @param[in] pParams    pointer to control parameters
+ *
+ * @return NV_OK When successful
+ *         NV_ERR_INVALID_STATE Otherwise
+ */
 NV_STATUS
-subdeviceCtrlCmdFbSetZbcReferenced_IMPL
+subdeviceCtrlCmdFbGetStaticBar1Info_IMPL
 (
     Subdevice *pSubdevice,
-    NV2080_CTRL_INTERNAL_MEMSYS_SET_ZBC_REFERENCED_PARAMS *pParams
+    NV2080_CTRL_FB_GET_STATIC_BAR1_INFO_PARAMS *pParams
+)
+{
+    OBJGPU             *pGpu        = GPU_RES_GET_GPU(pSubdevice);
+    KernelBus          *pKernelBus  = GPU_GET_KERNEL_BUS(pGpu);
+    OBJSYS             *pSys        = SYS_GET_INSTANCE();
+    OBJCL              *pCl         = pSys->pCl;
+    NV_STATUS           status      = NV_OK;
+    NvU32               gfid;
+
+    NV_ASSERT_OK_OR_RETURN(vgpuGetCallingContextGfid(pGpu, &gfid));
+
+    NV_ASSERT_OR_RETURN(rmapiLockIsOwner() && rmDeviceGpuLockIsOwner(pGpu->gpuInstance),
+        NV_ERR_INVALID_LOCK_STATE);
+
+    pParams->bStaticBar1Enabled = pKernelBus->bar1[gfid].bStaticBar1Enabled;
+    pParams->bStaticBar1WriteCombined = !pCl->getProperty(pCl, PDB_PROP_CL_DISABLE_IOMAP_WC);
+
+    if (pParams->bStaticBar1Enabled)
+    {
+        pParams->staticBar1StartOffset = pKernelBus->bar1[gfid].staticBar1.startOffset;
+        pParams->staticBar1Size        = pKernelBus->bar1[gfid].staticBar1.size;
+    }
+    else
+    {
+        pParams->staticBar1StartOffset = 0;
+        pParams->staticBar1Size = 0;
+    }
+
+    return status;
+}
+
+static NV_STATUS
+_kmemsysSetZbcReferenced
+(
+    OBJGPU *pGpu,
+    Device *pDevice,
+    NvU32 bZbcSurfacesExist
 )
 {
-    OBJGPU       *pGpu = GPU_RES_GET_GPU(pSubdevice);
     CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
     NvU32         gfid;
     NV_STATUS     status = NV_OK;
@@ -1339,45 +1348,53 @@ subdeviceCtrlCmdFbSetZbcReferenced_IMPL
     return status;
 }
 
-/*!
- * @brief This call can be used to flush L2 followed by FB or
- * just the FB.
- *
- * If L2 ops are needed, either _INVALIDATE or _WRITE_BACK
- * or both flags set to _YES is required. Specifying both
- * to _YES implies EVICT.
- *
- * If only the FB flush is needed, only the
- * _APERTURE and _FB_FLUSH_YES are needed.
- *
- * If only L2 ops are needed (i.e. no FB flush following
- * it), _FB_FLUSH_NO is needed in addition to the other
- * L2 ops flags.
- *
- * Lock Requirements:
- *      Assert that API and GPUs lock held on entry
- *
- * @param[in] pSubdevice Subdevice
- * @param[in] pCacheFlushParams Various flush flags
- *
- */
 NV_STATUS
-subdeviceCtrlCmdFbFlushGpuCache_IMPL
+deviceCtrlCmdFbSetZbcReferenced_IMPL
+(
+    Device *pDevice,
+    NV0080_CTRL_INTERNAL_MEMSYS_SET_ZBC_REFERENCED_PARAMS *pParams
+)
+{
+    OBJGPU *pGpu;
+    Subdevice *pSubdevice;
+
+    NV_ASSERT_OK_OR_RETURN(subdeviceGetByInstance(RES_GET_CLIENT(pDevice), RES_GET_HANDLE(pDevice), pParams->subdevInstance, &pSubdevice));
+
+    pGpu = GPU_RES_GET_GPU(pSubdevice);
+
+    return _kmemsysSetZbcReferenced(pGpu, pDevice, pParams->bZbcSurfacesExist);
+}
+
+NV_STATUS
+subdeviceCtrlCmdFbSetZbcReferenced_IMPL
 (
     Subdevice *pSubdevice,
+    NV2080_CTRL_INTERNAL_MEMSYS_SET_ZBC_REFERENCED_PARAMS *pParams
+)
+{
+    OBJGPU *pGpu = GPU_RES_GET_GPU(pSubdevice);
+    Device *pDevice = GPU_RES_GET_DEVICE(pSubdevice);
+
+    return _kmemsysSetZbcReferenced(pGpu, pDevice, pParams->bZbcSurfacesExist);
+}
+
+NV_STATUS
+kmemsysFlushGpuCache_IMPL
+(
+    OBJGPU *pGpu,
+    KernelMemorySystem *pKernelMemorySystem,
+    KernelBus *pKernelBus,
     NV2080_CTRL_FB_FLUSH_GPU_CACHE_PARAMS *pCacheFlushParams
 )
 {
-    OBJGPU             *pGpu = GPU_RES_GET_GPU(pSubdevice);
-    KernelMemorySystem *pKernelMemorySystem = GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu);
     NV_STATUS           status = NV_OK;
     FB_CACHE_MEMTYPE    memType = FB_CACHE_MEM_UNDEFINED;
     FB_CACHE_OP         cacheOp = FB_CACHE_OP_UNDEFINED;
-    KernelBus          *pKernelBus = GPU_GET_KERNEL_BUS(pGpu);
     NvBool              bWriteback = NV_FALSE;
     NvBool              bInvalidate = NV_FALSE;
 
-    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmDeviceGpuLockIsOwner(pGpu->gpuInstance));
+    NV_ASSERT_OR_RETURN(rmapiLockIsOwner() && rmDeviceGpuLockIsOwner(pGpu->gpuInstance),
+        NV_ERR_INVALID_LOCK_STATE);
 
     // Either WriteBack or Invalidate are required for Cache Ops
     if (FLD_TEST_DRF(2080, _CTRL_FB_FLUSH_GPU_CACHE_FLAGS, _WRITE_BACK,
@@ -1476,6 +1493,41 @@ subdeviceCtrlCmdFbFlushGpuCache_IMPL
     return status;
 }
 
+/*!
+ * @brief This call can be used to flush L2 followed by FB or
+ * just the FB.
+ *
+ * If L2 ops are needed, either _INVALIDATE or _WRITE_BACK
+ * or both flags set to _YES is required. Specifying both
+ * to _YES implies EVICT.
+ *
+ * If only the FB flush is needed, only the
+ * _APERTURE and _FB_FLUSH_YES are needed.
+ *
+ * If only L2 ops are needed (i.e. no FB flush following
+ * it), _FB_FLUSH_NO is needed in addition to the other
+ * L2 ops flags.
+ *
+ * Lock Requirements:
+ *      Assert that API and GPUs lock held on entry
+ *
+ * @param[in] pSubdevice Subdevice
+ * @param[in] pCacheFlushParams Various flush flags
+ *
+ */
+NV_STATUS
+subdeviceCtrlCmdFbFlushGpuCache_IMPL
+(
+    Subdevice *pSubdevice,
+    NV2080_CTRL_FB_FLUSH_GPU_CACHE_PARAMS *pCacheFlushParams
+)
+{
+    OBJGPU             *pGpu = GPU_RES_GET_GPU(pSubdevice);
+    KernelMemorySystem *pKernelMemorySystem = GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu);
+    KernelBus *pKernelBus = GPU_GET_KERNEL_BUS(pGpu);
+    return kmemsysFlushGpuCache(pGpu, pKernelMemorySystem, pKernelBus, pCacheFlushParams);
+}
+
 //
 // subdeviceCtrlCmdFbGetGpuCacheInfo
 //
@@ -1495,7 +1547,7 @@ subdeviceCtrlCmdFbGetGpuCacheInfo_IMPL
     FB_CACHE_WRITE_MODE writeMode = pKernelMemorySystem->l2WriteMode;
     FB_CACHE_BYPASS_MODE bypassMode = FB_CACHE_BYPASS_MODE_DISABLED;
 
-    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
+    NV_ASSERT_OR_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner(), NV_ERR_INVALID_LOCK_STATE);
 
     switch (writeMode)
     {
