@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2020-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2020-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -208,14 +208,16 @@ knvlinkCoreAddDevice_IMPL
     dev->pciInfo.device           = gpuGetDevice(pGpu);
     dev->pciInfo.function         = 0;
     dev->pciInfo.pciDeviceId      = pGpu->idInfo.PCIDeviceID;
-    dev->pciInfo.bars[0].baseAddr = pGpu->pKernelBus->pciBars[0];
-    dev->pciInfo.bars[0].barSize  = pGpu->pKernelBus->pciBarSizes[0];
+    dev->pciInfo.bars[0].baseAddr = GPU_GET_KERNEL_BUS(pGpu)->pciBars[0];
+    dev->pciInfo.bars[0].barSize  = GPU_GET_KERNEL_BUS(pGpu)->pciBarSizes[0];
     dev->initialized              = 1;
     dev->enableALI                = pKernelNvlink->bEnableAli;
     dev->numIoctrls               = nvPopCount32(pKernelNvlink->ioctrlMask);
     dev->numActiveLinksPerIoctrl  = knvlinkGetNumActiveLinksPerIoctrl(pGpu, pKernelNvlink);
     dev->numLinksPerIoctrl        = knvlinkGetTotalNumLinksPerIoctrl(pGpu, pKernelNvlink);
     dev->bReducedNvlinkConfig     = knvlinkIsGpuReducedNvlinkConfig_HAL(pGpu, pKernelNvlink);
+    dev->linkStateSupportedMask   = knvlinkGetSupportedCoreLinkStateMask_HAL(pGpu, pKernelNvlink);
+    dev->bLinkStatesSymmetric     = pKernelNvlink->getProperty(pKernelNvlink, PDB_PROP_KNVLINK_UNILATERAL_LINK_STATE_CHANGE_SUPPORTED);
 
     // Register the GPU in nvlink core
     if (nvlink_lib_register_device(dev) != 0)
@@ -588,7 +590,7 @@ knvlinkIsGpuConnectedToNvswitch_IMPL
     NvU32 i;
     KNVLINK_CONN_INFO remoteEndInfo;
 
-    FOR_EACH_INDEX_IN_MASK(32, i, pKernelNvlink->enabledLinks)
+    FOR_EACH_INDEX_IN_MASK(32, i, KNVLINK_GET_MASK(pKernelNvlink, enabledLinks, 32))
     {
         remoteEndInfo = pKernelNvlink->nvlinkLinks[i].remoteEndInfo;
 
@@ -667,7 +669,7 @@ _knvlinkUpdateRemoteEndUuidInfo
     unsigned      remoteLinkId;
     unsigned      i, j;
 
-    FOR_EACH_INDEX_IN_MASK(32, i, pKernelNvlink->enabledLinks)
+    FOR_EACH_INDEX_IN_MASK(32, i, KNVLINK_GET_MASK(pKernelNvlink, enabledLinks, 32))
     {
         if (pKernelNvlink->nvlinkLinks[i].remoteEndInfo.bConnected &&
             (pKernelNvlink->nvlinkLinks[i].remoteEndInfo.deviceType == NVLINK_DEVICE_TYPE_GPU))
