@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -41,6 +41,7 @@
 #include "core/locks.h"
 #include "gpu/gpu.h"
 #include "gpu/gpu_access.h"
+#include "kernel/diagnostics/xid_context.h"
 #include "nv_ref.h"
 #include "virtualization/hypervisor/hypervisor.h"
 
@@ -226,8 +227,12 @@ void vgpuDevWriteReg032(
         return;
     }
 
-    NV_ASSERT_OK(kbifGetPciConfigSpacePriMirror_HAL(pGpu, GPU_GET_KERNEL_BIF(pGpu),
-                                                    &configSpaceMirrorBase, &configSpaceMirrorSize));
+    if (NV_OK != kbifGetPciConfigSpacePriMirror_HAL(pGpu, GPU_GET_KERNEL_BIF(pGpu),
+                                                    &configSpaceMirrorBase, &configSpaceMirrorSize))
+    {
+        *vgpuHandled = NV_FALSE;
+        return;
+    }
 
     if (IS_VIRTUAL_WITH_SRIOV(pGpu))
     {
@@ -302,8 +307,12 @@ NvU32 vgpuDevReadReg032(
         return 0;
     }
 
-    NV_ASSERT_OK(kbifGetPciConfigSpacePriMirror_HAL(pGpu, GPU_GET_KERNEL_BIF(pGpu),
-                                                    &configSpaceMirrorBase, &configSpaceMirrorSize));
+    if (NV_OK != kbifGetPciConfigSpacePriMirror_HAL(pGpu, GPU_GET_KERNEL_BIF(pGpu),
+                                                    &configSpaceMirrorBase, &configSpaceMirrorSize))
+    {
+        *vgpuHandled = NV_FALSE;
+        return 0;
+    }
 
     if (IS_VIRTUAL_WITH_SRIOV(pGpu))
     {
@@ -376,109 +385,6 @@ osGetCpuVaAddrShift(void)
     return (64 - portUtilCountLeadingZeros64(maxUserVa - 1)) + 1;
 }
 
-/**
- * @brief Adds a filter to trap a certain CPU virtual address range
- *
- * Sets up a filter so all accesses to an address range are sent through the
- * specified callback.
- *
- * Only one filter is allowed for any given address.
- *
- * @param[in] rangeStart  start of CPU address range (inclusive)
- * @param[in] rangeEnd end of CPU address range (inclusive)
- * @param[in] pCb Callback function
- * @param[in] pPriv opaque point to data pass to callback
- *
- * @return NV_OK is success, appropriate error otherwise.
- */
-NV_STATUS
-osMemAddFilter
-(
-    NvU64           rangeStart,
-    NvU64           rangeEnd,
-    OSMemFilterCb  *pCb,
-    void           *pPriv
-)
-{
-    OBJSYS             *pSys = SYS_GET_INSTANCE();
-    POSMEMFILTERDATA    pFilterData = NULL;
-
-    pFilterData = portMemAllocNonPaged(sizeof(OSMEMFILTERDATA));
-    if (pFilterData == NULL)
-    {
-        NV_PRINTF(LEVEL_ERROR,
-                  "Failed to alloc mem for os mem filter data!\n");
-        return NV_ERR_INSUFFICIENT_RESOURCES;
-    }
-
-    pFilterData->node.keyStart = rangeStart;
-    pFilterData->node.keyEnd   = rangeEnd;
-    pFilterData->pPriv         = pPriv;
-    pFilterData->pFilterCb     = pCb;
-    pFilterData->node.Data     = (void *)pFilterData;
-
-    return btreeInsert(&pFilterData->node, &pSys->pMemFilterList);
-}
-
-/**
- * @brief Remove a filter added with @ref osMemAddFilter
- *
- * @param[in] rangeStart memory address to remove filter from.
- */
-NV_STATUS
-osMemRemoveFilter
-(
-    NvU64       rangeStart
-)
-{
-    OBJSYS     *pSys = SYS_GET_INSTANCE();
-    PNODE       pNode = NULL;
-
-    if (btreeSearch(rangeStart, &pNode, pSys->pMemFilterList) != NV_OK)
-    {
-        NV_PRINTF(LEVEL_INFO,
-                  "Failed to find filter data for the given range start address!\n");
-        return NV_ERR_INVALID_ARGUMENT;
-    }
-
-    NV_ASSERT(pNode);
-    NV_ASSERT(pNode->keyStart == rangeStart);
-
-    if (btreeUnlink(pNode, &pSys->pMemFilterList) != NV_OK)
-    {
-        NV_PRINTF(LEVEL_ERROR, "Failed to unlink filter data!\n");
-        return NV_ERR_INVALID_STATE;
-    }
-
-    portMemFree(pNode->Data);
-    pNode = NULL;
-
-    return NV_OK;
-}
-
-/**
- * @brief Retrieves a filter added with @ref osMemAddFilter.
- *
- * @param[in] address Address to search for
- *
- * @return Appropriate filter data if a filter exists, NULL otherwise.
- */
-POSMEMFILTERDATA
-osMemGetFilter(NvUPtr address)
-{
-    OBJSYS     *pSys;
-    PNODE       pNode = NULL;
-
-    pSys = SYS_GET_INSTANCE();
-    if (!pSys)
-        return NULL;
-
-    if (btreeSearch(address, &pNode, pSys->pMemFilterList) != NV_OK)
-        return NULL;
-
-    return pNode->Data;
-}
-
 /*!
  * Some data such as Bindata array are placed on paged memory.  Access to paged segment
  * on high IRQL is not allowed on some platform (e.g. Windows).  The issue could be
@@ -575,7 +481,7 @@ NV_STATUS osReadRegistryString
     return status;
 }
 
-void nvErrorLog(void *pVoid, NvU32 num, const char *pFormat, va_list arglist)
+static void nvErrorLog2(void *pVoid, XidContext context, NvBool oobLogging, const char *pFormat, va_list arglist)
 {
     if ((pFormat == NULL) || (*pFormat == '\0'))
     {
@@ -584,7 +490,7 @@ void nvErrorLog(void *pVoid, NvU32 num, const char *pFormat, va_list arglist)
 
     OBJGPU    *pGpu    = reinterpretCast(pVoid, OBJGPU *);
 
-#if RMCFG_MODULE_SMBPBI || \
+#if RMCFG_MODULE_OOB || \
     (RMCFG_MODULE_KERNEL_RC && !RMCFG_FEATURE_PLATFORM_GSP)
     char *errorString = portMemAllocNonPaged(MAX_ERROR_STRING);
     if (errorString == NULL)
@@ -600,25 +506,39 @@ void nvErrorLog(void *pVoid, NvU32 num, const char *pFormat, va_list arglist)
     if (msglen == 0)
         goto done;
 
+    if (pGpu != NULL && oobLogging)
+    {
+        gpuLogOobXidMessage(pGpu, context.xid, errorString, msglen);
+    }
+
     {
         KernelRc *pKernelRc = GPU_GET_KERNEL_RC(pGpu);
         if (pKernelRc != NULL)
-            krcReportXid(pGpu, pKernelRc, num, errorString);
+        {
+            krcReportXid(pGpu, pKernelRc,
+                         context,
+                         errorString);
+        }
     }
 
 done:
     portMemFree(errorString);
-#endif // RMCFG_MODULE_SMBPBI || (RMCFG_MODULE_KERNEL_RC &&
+#endif // RMCFG_MODULE_OOB || (RMCFG_MODULE_KERNEL_RC &&
        // !RMCFG_FEATURE_PLATFORM_GSP)
 
-    osErrorLogV(pGpu, num, pFormat, arglist);
+    osErrorLogV(pGpu, context, pFormat, arglist);
+}
+
+void nvErrorLog(void *pVoid, XidContext context, const char *pFormat, va_list arglist)
+{
+    nvErrorLog2(pVoid, context, NV_TRUE, pFormat, arglist);
 }
 
 void
 nvErrorLog_va
 (
-    void * pVoid,
-    NvU32 num,
+    void       * pVoid,
+    NvU32        num,
     const char * pFormat,
     ...
 )
@@ -626,6 +546,23 @@ nvErrorLog_va
     va_list arglist;
 
     va_start(arglist, pFormat);
-    nvErrorLog(pVoid, num, pFormat, arglist);
+    nvErrorLog2(pVoid, (XidContext){.xid = num}, NV_TRUE, pFormat, arglist);
+    va_end(arglist);
+}
+
+void
+nvErrorLog2_va
+(
+    void       * pVoid,
+    XidContext   context,
+    NvBool       oobLogging,
+    const char * pFormat,
+    ...
+)
+{
+    va_list arglist;
+
+    va_start(arglist, pFormat);
+    nvErrorLog2(pVoid, context, oobLogging, pFormat, arglist);
     va_end(arglist);
 }
