@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2004-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2004-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -103,6 +103,28 @@ BRINFO upstreamPortInfo[] =
     {0, 0, NULL}
 };
 
+static NV_STATUS
+AMD_14D8_setupFunc
+(
+    OBJCL *pCl
+)
+{
+    pCl->setProperty(pCl, PDB_PROP_CL_WAR_4802761_ENABLED, NV_TRUE);
+
+    return NV_OK;
+}
+
+static NV_STATUS
+Intel_A70D_setupFunc
+(
+    OBJCL *pCl
+)
+{
+    pCl->setProperty(pCl, PDB_PROP_CL_WAR_4802761_ENABLED, NV_TRUE);
+
+    return NV_OK;
+}
+
 static NV_STATUS
 Intel_25XX_setupFunc
 (
@@ -1115,6 +1137,9 @@ AMD_X370_setupFunc
 )
 {
 
+    // WAR for bug 5107271 handling
+    pCl->setProperty(pCl, PDB_PROP_CL_WAR_AMD_5107271, NV_TRUE);
+
     // Set ASPM L0S\L1 properties
     _Set_ASPM_L0S_L1(pCl, NV_FALSE, NV_FALSE);
 
@@ -1292,6 +1317,40 @@ Arm_NeoverseN1_setupFunc
     return NV_OK;
 }
 
+static NV_STATUS
+Riscv_generic_setupFunc
+(
+    OBJCL *pCl
+)
+{
+    pCl->setProperty(pCl, PDB_PROP_CL_IS_CHIPSET_IO_COHERENT, NV_TRUE);
+    return NV_OK;
+}
+
+static NV_STATUS
+PLDA_XpressRichAXI_setupFunc
+(
+    OBJCL *pCl
+)
+{
+#if NVCPU_IS_RISCV64
+    pCl->setProperty(pCl, PDB_PROP_CL_IS_CHIPSET_IO_COHERENT, NV_TRUE);
+#endif
+    return NV_OK;
+}
+
+// Ampere AmpereOne Setup Function
+static NV_STATUS
+Ampere_AmpereOne_setupFunc
+(
+    OBJCL *pCl
+)
+{
+    // TODO Need to check if any more PDB properties should be set
+    pCl->setProperty(pCl, PDB_PROP_CL_IS_CHIPSET_IO_COHERENT, NV_TRUE);
+    return NV_OK;
+}
+
 void
 csGetInfoStrings
 (
