@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -35,11 +35,13 @@
 #include "virtualization/hypervisor/hypervisor.h"
 #include "gpu_mgr/gpu_mgr.h"
 #include "platform/sli/sli.h"
+#include "kernel/gpu/gsp/gsp_trace_rats_macro.h"
 
 #include "kernel/gpu/mig_mgr/kernel_mig_manager.h"
 
 #include "gpu/device/device.h"
 #include "class/cl0080.h"
+#include "class/clc372sw.h"
 
 #include "class/cl83de.h" // GT200_DEBUGGER
 #include "gpu/gr/kernel_sm_debugger_session.h"
@@ -293,6 +295,35 @@ serverTopLock_Epilogue
     }
 }
 
+static NvU32
+_resGetBackRefGpusMask(RsResourceRef *pResourceRef)
+{
+    NvU32 gpuMask = 0x0;
+    RsInterMapping *pBackRefItem;
+
+    if (pResourceRef == NULL)
+    {
+        return 0x0;
+    }
+
+    pBackRefItem = listHead(&pResourceRef->interBackRefsContext);
+    while (pBackRefItem != NULL)
+    {
+        RsResourceRef *pDeviceRef = pBackRefItem->pContextRef;
+        GpuResource *pGpuResource = dynamicCast(pDeviceRef->pResource, GpuResource);
+
+        if (pGpuResource != NULL)
+        {
+            OBJGPU *pGpu = GPU_RES_GET_GPU(pGpuResource);
+            gpuMask |= gpumgrGetGpuMask(pGpu);
+        }
+
+        pBackRefItem = listNext(&pResourceRef->interBackRefsContext, pBackRefItem);
+    }
+
+    return gpuMask;
+}
+
 NV_STATUS
 serverResLock_Prologue
 (
@@ -410,8 +441,15 @@ serverResLock_Prologue
         }
         else
         {
-            status = rmGpuGroupLockAcquire(pParentGpu->gpuInstance,
-                                           GPU_LOCK_GRP_DEVICE,
+            //
+            // Lock the parent GPU and if specified any GPUs that resource
+            // may backreference via mappings.
+            //
+            pLockInfo->gpuMask = gpumgrGetGpuMask(pParentGpu) |
+                                 _resGetBackRefGpusMask(pLockInfo->pResRefToBackRef);
+
+            status = rmGpuGroupLockAcquire(0,
+                                           GPU_LOCK_GRP_MASK,
                                            GPUS_LOCK_FLAGS_NONE,
                                            RM_LOCK_MODULES_CLIENT,
                                            &pLockInfo->gpuMask);
@@ -557,12 +595,6 @@ _serverAlloc_ValidateVgpu
     const NvU32 flags
 )
 {
-    //
-    // Get any GPU to check environmental properties
-    // Assume: In multi-vGPU systems, no mix and matching SRIOV and non-SRIOV GPUs.
-    // Assume: This OBJGPU will only used to read system properties.
-    //
-    OBJGPU *pGpu = gpumgrGetSomeGpu();
 
     // Check whether context is already sufficiently privileged
     if (flags & RS_FLAGS_ALLOC_PRIVILEGED)
@@ -573,15 +605,6 @@ _serverAlloc_ValidateVgpu
         }
     }
 
-    // Check whether object is allowed in current environment
-    if
-    (
-        (IS_SRIOV_WITH_VGPU_GSP_ENABLED(pGpu) && (flags & RS_FLAGS_ALLOC_CPU_PLUGIN_FOR_VGPU_GSP))
-    )
-    {
-        return NV_TRUE;
-    }
-
     return NV_FALSE;
 }
 
@@ -614,7 +637,7 @@ _serverAllocValidatePrivilege
         if (!_serverAlloc_ValidateVgpu(pClient, pParams->hParent, pParams->externalClassId,
                                       pParams->pSecInfo->privLevel, pResDesc->flags))
         {
-            NV_PRINTF(LEVEL_WARNING,
+            NV_PRINTF(LEVEL_NOTICE,
                       "hClient: 0x%08x, externalClassId: 0x%08x: CPU hypervisor does not have permission to allocate object\n",
                       pParams->hClient, pParams->externalClassId);
             return NV_ERR_INSUFFICIENT_PERMISSIONS;
@@ -629,7 +652,7 @@ _serverAllocValidatePrivilege
         {
             if (privLevel < RS_PRIV_LEVEL_USER_ROOT)
             {
-                NV_PRINTF(LEVEL_WARNING,
+                NV_PRINTF(LEVEL_NOTICE,
                           "hClient: 0x%08x, externalClassId: 0x%08x: non-privileged context tried to allocate privileged object\n",
                           pParams->hClient, pParams->externalClassId);
                 return NV_ERR_INSUFFICIENT_PERMISSIONS;
@@ -640,7 +663,7 @@ _serverAllocValidatePrivilege
         {
             if (privLevel < RS_PRIV_LEVEL_KERNEL)
             {
-                NV_PRINTF(LEVEL_WARNING,
+                NV_PRINTF(LEVEL_NOTICE,
                           "hClient: 0x%08x, externalClassId: 0x%08x: non-privileged context tried to allocate kernel privileged object\n",
                           pParams->hClient, pParams->externalClassId);
                 return NV_ERR_INSUFFICIENT_PERMISSIONS;
@@ -739,7 +762,7 @@ serverAllocResourceUnderLock
     tmpStatus = gpuGetByRef(pParentRef, NULL, &pGpu);
 
     // Override locking flags if we'll need to RPC to GSP
-    if (pGpu != NULL && IS_GSP_CLIENT(pGpu) &&
+    if (pGpu != NULL && IS_FW_CLIENT(pGpu) &&
         (pResDesc->flags & RS_FLAGS_ALLOC_RPC_TO_PHYS_RM))
     {
         resLockAccess = LOCK_ACCESS_WRITE; // always write as we're RPCing to GSP
@@ -752,7 +775,6 @@ serverAllocResourceUnderLock
         {
             NV_PRINTF(LEVEL_INFO, "Overriding flags for alloc of class %04x\n",
                 pRmAllocParams->externalClassId);
-            pLockInfo->flags &= ~RM_LOCK_FLAGS_NO_GPUS_LOCK;
             pLockInfo->flags |= RM_LOCK_FLAGS_GPU_GROUP_LOCK;
             if ((pLockInfo->state & RM_LOCK_STATES_ALLOW_RECURSIVE_LOCKS) == 0)
             {
@@ -860,7 +882,7 @@ serverAllocResourceUnderLock
                 goto done;
             }
 
-            if (!IS_VIRTUAL(pGpu) && !IS_GSP_CLIENT(pGpu))
+            if (!IS_VIRTUAL(pGpu) && !IS_FW_CLIENT(pGpu))
             {
                 status = NV_OK;
                 goto done;
@@ -868,7 +890,7 @@ serverAllocResourceUnderLock
 
             // if physical RM RPC make sure we're a GSP client otherwise skip
             if (((pResDesc->flags & (RS_FLAGS_ALLOC_RPC_TO_VGPU_HOST | RS_FLAGS_ALLOC_RPC_TO_PHYS_RM)) == RS_FLAGS_ALLOC_RPC_TO_PHYS_RM) &&
-                (!IS_GSP_CLIENT(pGpu)))
+                (!IS_FW_CLIENT(pGpu)))
             {
                 status = NV_OK;
                 goto done;
@@ -951,7 +973,7 @@ serverFreeResourceRpcUnderLock
     pRmResource = dynamicCast(pResourceRef->pResource, RmResource);
     status = gpuGetByRef(pResourceRef, &bBcResource, &pGpu);
     if ((status != NV_OK) ||
-        (!IS_VIRTUAL(pGpu) && !IS_GSP_CLIENT(pGpu)) ||
+        (!IS_VIRTUAL(pGpu) && !IS_FW_CLIENT(pGpu)) ||
         (pRmResource == NULL) ||
         (pRmResource->bRpcFree == NV_FALSE))
     {
@@ -1063,7 +1085,7 @@ serverUpdateLockFlagsForFree
     if (gpuGetByRef(pLockInfo->pContextRef, NULL, &pGpu) == NV_OK)
     {
         RmResource *pRmResource = dynamicCast(pRmFreeParams->pResourceRef->pResource, RmResource);
-        if (pGpu != NULL && IS_GSP_CLIENT(pGpu) && pRmResource != NULL && pRmResource->bRpcFree)
+        if (pGpu != NULL && IS_FW_CLIENT(pGpu) && pRmResource != NULL && pRmResource->bRpcFree)
         {
             //
             // If the resource desc says no need for GPU locks, we still need to lock
@@ -1073,7 +1095,6 @@ serverUpdateLockFlagsForFree
             {
                 NV_PRINTF(LEVEL_INFO, "Overriding flags for free of class %04x\n",
                     pRmFreeParams->pResourceRef->externalClassId);
-                pLockInfo->flags &= ~RM_LOCK_FLAGS_NO_GPUS_LOCK;
                 pLockInfo->flags |= RM_LOCK_FLAGS_GPU_GROUP_LOCK;
                 pLockInfo->state |= RM_LOCK_STATES_ALLOW_RECURSIVE_LOCKS;
             }
@@ -1254,7 +1275,6 @@ rmapiAllocWithSecInfo
                       pRightsRequested,
                       *pSecInfo);
 
-
     //
     // If hClient is allocated behind GPU locks, client is marked as internal
     //
@@ -1273,9 +1293,9 @@ rmapiAllocWithSecInfo
     }
     else
     {
-        NV_PRINTF(LEVEL_WARNING, "allocation failed; status: %s (0x%08x)\n",
+        NV_PRINTF(LEVEL_INFO, "allocation failed; status: %s (0x%08x)\n",
                   nvstatusToString(status), status);
-        NV_PRINTF(LEVEL_WARNING,
+        NV_PRINTF(LEVEL_INFO,
                   "client:0x%x parent:0x%x object:0x%x class:0x%x\n", hClient,
                   hParent, *phObject, hClass);
     }
@@ -1360,6 +1380,18 @@ resservResourceFactory
         }
     }
 
+    if (pGpu != NULL &&
+        !RMCFG_FEATURE_PLATFORM_MODS &&
+        !gpuIsClassSupported(pGpu, pParams->externalClassId))
+    {
+        if (!IsGM107(pGpu) || // Allow unsupported classes on GM107 on VGPU: bug 5354490 WAR
+            !hypervisorIsVgxHyper())
+        {
+            NV_PRINTF(LEVEL_INFO, "Skipping unsupported class 0x%x\n", pParams->externalClassId);
+            return NV_ERR_NOT_SUPPORTED;
+        }
+    }
+
     status = objCreateDynamicWithFlags(&pDynamic,
                                        (Object*)pGpu,
                                        pResDesc->pClassInfo,
@@ -1374,9 +1406,17 @@ resservResourceFactory
     if (pResource == NULL)
         return NV_ERR_INSUFFICIENT_RESOURCES;
 
-    if (pResDesc->internalClassId == classId(Subdevice) || pResDesc->internalClassId == classId(Device))
+    if (pResDesc->internalClassId == classId(Subdevice)  || pResDesc->internalClassId == classId(Device) ||
+        pResDesc->internalClassId == classId(DispCommon))
     {
-        pGpu = GPU_RES_GET_GPU(dynamicCast(pDynamic, GpuResource));
+        //
+        // DispCommon and DispSwObj's pGpu will be retrieved at the beginning of the function,
+        // since their parent is Device.
+        //
+        if (!(pResDesc->internalClassId == classId(DispCommon)) && !(pResDesc->internalClassId == classId(DispSwObj)))
+        {
+            pGpu = GPU_RES_GET_GPU(dynamicCast(pDynamic, GpuResource));
+        }
 
         if (pGpu && !IS_MIG_IN_USE(pGpu))
         {
@@ -1496,11 +1536,12 @@ rmapiFreeWithSecInfo
     }
     else
     {
-        NV_PRINTF(LEVEL_WARNING,
-                  "Nv01Free: free failed; status: %s (0x%08x)\n",
-                  nvstatusToString(status), status);
-        NV_PRINTF(LEVEL_WARNING, "Nv01Free:  client:0x%x object:0x%x\n",
-                  hClient, hObject);
+       NV_PRINTF_COND(status == NV_ERR_GPU_IN_FULLCHIP_RESET, LEVEL_INFO, LEVEL_WARNING,
+                "Nv01Free: free failed; status: %s (0x%08x)\n",
+                nvstatusToString(status), status);
+       NV_PRINTF_COND(status == NV_ERR_GPU_IN_FULLCHIP_RESET, LEVEL_INFO, LEVEL_WARNING,
+                "Nv01Free:  client:0x%x object:0x%x\n",
+                hClient, hObject);
     }
 
     return status;
@@ -1618,18 +1659,13 @@ serverAllocResourceLookupLockFlags
     LOCK_ACCESS_TYPE *pAccess
 )
 {
+    OBJSYS *pSys = SYS_GET_INSTANCE();
     NV_ASSERT_OR_RETURN(pAccess != NULL, NV_ERR_INVALID_ARGUMENT);
 
     if (lock == RS_LOCK_TOP)
     {
         RS_RESOURCE_DESC *pResDesc;
 
-        if (!serverSupportsReadOnlyLock(&g_resServ, RS_LOCK_TOP, RS_API_ALLOC_RESOURCE))
-        {
-            *pAccess = LOCK_ACCESS_WRITE;
-            return NV_OK;
-        }
-
         pResDesc = RsResInfoByExternalClassId(pParams->externalClassId);
 
         if (pResDesc == NULL)
@@ -1642,6 +1678,23 @@ serverAllocResourceLookupLockFlags
         else
             *pAccess = LOCK_ACCESS_WRITE;
 
+        if ((pResDesc->flags & RS_FLAGS_FORCE_ACQUIRE_RO_API_LOCK_ON_ALLOC_FREE) != 0 &&
+            pSys->getProperty(pSys, PDB_PROP_SYS_ENABLE_FORCE_SHARED_LOCK))
+        {
+            //
+            // If the force acquire RO flag is set then ignore module parameter
+            // setting and always use RO.
+            //
+            *pAccess = LOCK_ACCESS_READ;
+            return NV_OK;
+        }
+
+        if (!serverSupportsReadOnlyLock(&g_resServ, RS_LOCK_TOP, RS_API_ALLOC_RESOURCE))
+        {
+            *pAccess = LOCK_ACCESS_WRITE;
+            return NV_OK;
+        }
+
         return NV_OK;
     }
 
@@ -1660,13 +1713,35 @@ serverFreeResourceLookupLockFlags
     RsServer *pServer,
     RS_LOCK_ENUM lock,
     RS_RES_FREE_PARAMS_INTERNAL *pParams,
-    LOCK_ACCESS_TYPE *pAccess
+    LOCK_ACCESS_TYPE *pAccess,
+    NvBool *pbSupportForceROLock
 )
 {
+    OBJSYS *pSys = SYS_GET_INSTANCE();
+    NvU32 flags;
     NV_ASSERT_OR_RETURN(pAccess != NULL, NV_ERR_INVALID_ARGUMENT);
 
     *pAccess = (serverSupportsReadOnlyLock(pServer, lock, RS_API_FREE_RESOURCE))
         ? LOCK_ACCESS_READ
         : LOCK_ACCESS_WRITE;
+
+    *pbSupportForceROLock = pSys->getProperty(pSys, PDB_PROP_SYS_ENABLE_FORCE_SHARED_LOCK);
+
+    if (pParams->pResourceRef != NULL)
+    {
+        //
+        // If pResourceRef is set, check for the explicit RO opt-in flag that
+        // some resources set as a transition before enabling RO across the
+        // board.
+        //
+        // bug 4283710 - [RM][Locking] Allow RMAPI to take API lock in reader's mode by default.
+        //
+        flags = pParams->pResourceRef->pResourceDesc->flags;
+        if ((flags & RS_FLAGS_FORCE_ACQUIRE_RO_API_LOCK_ON_ALLOC_FREE) != 0 &&
+            (*pbSupportForceROLock))
+        {
+            *pAccess = LOCK_ACCESS_READ;
+        }
+    }
     return NV_OK;
 }
