@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2018-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2018-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -249,28 +249,6 @@ NV_STATUS embeddedParamCopyIn(RMAPI_PARAM_COPY *paramCopies, RmCtrlParams *pRmCt
                             ((NV2080_CTRL_GPU_GET_ENGINES_PARAMS*)pParams)->engineCount, sizeof(NvU32));
             break;
         }
-        case NV2080_CTRL_CMD_BUS_GET_INFO:
-        {
-            CHECK_PARAMS_OR_RETURN(pRmCtrlParams, NV2080_CTRL_BUS_GET_INFO_PARAMS);
-
-            RMAPI_PARAM_COPY_INIT(paramCopies[0],
-                            ((NV2080_CTRL_BUS_GET_INFO_PARAMS*)pParams)->busInfoList,
-                            ((NV2080_CTRL_BUS_GET_INFO_PARAMS*)pParams)->busInfoList,
-                            ((NV2080_CTRL_BUS_GET_INFO_PARAMS*)pParams)->busInfoListSize,
-                            sizeof(NV2080_CTRL_BUS_INFO));
-            break;
-        }
-        case NV2080_CTRL_CMD_FB_GET_INFO:
-        {
-            CHECK_PARAMS_OR_RETURN(pRmCtrlParams, NV2080_CTRL_FB_GET_INFO_PARAMS);
-
-            RMAPI_PARAM_COPY_INIT(paramCopies[0],
-                            ((NV2080_CTRL_FB_GET_INFO_PARAMS*)pParams)->fbInfoList,
-                            ((NV2080_CTRL_FB_GET_INFO_PARAMS*)pParams)->fbInfoList,
-                            ((NV2080_CTRL_FB_GET_INFO_PARAMS*)pParams)->fbInfoListSize,
-                            sizeof(NV2080_CTRL_FB_INFO));
-            break;
-        }
 #ifdef USE_AMAPLIB
         case NV2080_CTRL_CMD_FB_GET_AMAP_CONF:
         {
@@ -382,19 +360,6 @@ NV_STATUS embeddedParamCopyIn(RMAPI_PARAM_COPY *paramCopies, RmCtrlParams *pRmCt
 
             break;
         }
-        case NV0080_CTRL_CMD_MSENC_GET_CAPS:
-        {
-            CHECK_PARAMS_OR_RETURN(pRmCtrlParams, NV0080_CTRL_MSENC_GET_CAPS_PARAMS);
-
-            RMAPI_PARAM_COPY_INIT(paramCopies[0],
-                            ((NV0080_CTRL_MSENC_GET_CAPS_PARAMS*)pParams)->capsTbl,
-                            ((NV0080_CTRL_MSENC_GET_CAPS_PARAMS*)pParams)->capsTbl,
-                            ((NV0080_CTRL_MSENC_GET_CAPS_PARAMS*)pParams)->capsTblSize, 1);
-            paramCopies[0].flags |= RMAPI_PARAM_COPY_FLAGS_SKIP_COPYIN;
-            paramCopies[0].flags |= RMAPI_PARAM_COPY_FLAGS_ZERO_BUFFER;
-
-            break;
-        }
         case NV2080_CTRL_CMD_BIOS_GET_INFO:
         {
             CHECK_PARAMS_OR_RETURN(pRmCtrlParams, NV2080_CTRL_BIOS_GET_INFO_PARAMS);
@@ -443,17 +408,6 @@ NV_STATUS embeddedParamCopyIn(RMAPI_PARAM_COPY *paramCopies, RmCtrlParams *pRmCt
                             sizeof(NV0080_CTRL_GR_INFO));
             break;
         }
-        case NV0080_CTRL_CMD_FIFO_START_SELECTED_CHANNELS:
-        {
-            CHECK_PARAMS_OR_RETURN(pRmCtrlParams, NV0080_CTRL_FIFO_START_SELECTED_CHANNELS_PARAMS);
-
-            RMAPI_PARAM_COPY_INIT(paramCopies[0],
-                            ((NV0080_CTRL_FIFO_START_SELECTED_CHANNELS_PARAMS*)pParams)->fifoStartChannelList,
-                            ((NV0080_CTRL_FIFO_START_SELECTED_CHANNELS_PARAMS*)pParams)->fifoStartChannelList,
-                            ((NV0080_CTRL_FIFO_START_SELECTED_CHANNELS_PARAMS*)pParams)->fifoStartChannelListCount,
-                            sizeof(NV0080_CTRL_FIFO_CHANNEL));
-            break;
-        }
         case NV0080_CTRL_CMD_FIFO_GET_CAPS:
         {
             CHECK_PARAMS_OR_RETURN(pRmCtrlParams, NV0080_CTRL_FIFO_GET_CAPS_PARAMS);
@@ -631,6 +585,21 @@ NV_STATUS embeddedParamCopyIn(RMAPI_PARAM_COPY *paramCopies, RmCtrlParams *pRmCt
                             ((NV0000_CTRL_SYSTEM_GET_P2P_CAPS_PARAMS*)pParams)->busPeerIds,
                             numEntries, sizeof(NvU32));
             paramCopies[0].flags |= RMAPI_PARAM_COPY_FLAGS_SKIP_COPYIN;
+            numEntries = 0;
+
+            if (NvP64_VALUE(((NV0000_CTRL_SYSTEM_GET_P2P_CAPS_PARAMS*)pParams)->busEgmPeerIds) != NULL)
+            {
+                // The handler will check gpuCount * gpuCount against overflow
+                numEntries = ((NV0000_CTRL_SYSTEM_GET_P2P_CAPS_PARAMS*)pParams)->gpuCount *
+                             ((NV0000_CTRL_SYSTEM_GET_P2P_CAPS_PARAMS*)pParams)->gpuCount;
+            }
+
+            RMAPI_PARAM_COPY_INIT(paramCopies[1],
+                                  ((NV0000_CTRL_SYSTEM_GET_P2P_CAPS_PARAMS*)pParams)->busEgmPeerIds,
+                                  ((NV0000_CTRL_SYSTEM_GET_P2P_CAPS_PARAMS*)pParams)->busEgmPeerIds,
+                                  numEntries, sizeof(NvU32));
+            paramCopies[1].flags |= RMAPI_PARAM_COPY_FLAGS_SKIP_COPYIN;
+            paramsCnt++;
             break;
         }
         case NV0080_CTRL_CMD_FB_GET_CAPS:
@@ -753,6 +722,18 @@ NV_STATUS embeddedParamCopyIn(RMAPI_PARAM_COPY *paramCopies, RmCtrlParams *pRmCt
             break;
         }
 
+        case NV2080_CTRL_CMD_GPU_RPC_GSP_TEST:
+        {
+            CHECK_PARAMS_OR_RETURN(pRmCtrlParams, NV2080_CTRL_GPU_RPC_GSP_TEST_PARAMS);
+            RMAPI_PARAM_COPY_INIT(paramCopies[0],
+                            ((NV2080_CTRL_GPU_RPC_GSP_TEST_PARAMS*)pParams)->data,
+                            ((NV2080_CTRL_GPU_RPC_GSP_TEST_PARAMS*)pParams)->data,
+                            ((NV2080_CTRL_GPU_RPC_GSP_TEST_PARAMS*)pParams)->dataSize,
+                            sizeof(NvU32));
+            break;
+        }
+
+
         default:
         {
             return status;
@@ -813,22 +794,6 @@ NV_STATUS embeddedParamCopyOut(RMAPI_PARAM_COPY *paramCopies, RmCtrlParams *pRmC
             ((NV2080_CTRL_GPU_GET_ENGINES_PARAMS*)pParams)->engineList = paramCopies[0].pUserParams;
             break;
         }
-        case NV2080_CTRL_CMD_BUS_GET_INFO:
-        {
-            CHECK_PARAMS_OR_RETURN(pRmCtrlParams, NV2080_CTRL_BUS_GET_INFO_PARAMS);
-
-            status = rmapiParamsRelease(&paramCopies[0]);
-            ((NV2080_CTRL_BUS_GET_INFO_PARAMS*)pParams)->busInfoList = paramCopies[0].pUserParams;
-            break;
-        }
-        case NV2080_CTRL_CMD_FB_GET_INFO:
-        {
-            CHECK_PARAMS_OR_RETURN(pRmCtrlParams, NV2080_CTRL_FB_GET_INFO_PARAMS);
-
-            status = rmapiParamsRelease(&paramCopies[0]);
-            ((NV2080_CTRL_FB_GET_INFO_PARAMS*)pParams)->fbInfoList = paramCopies[0].pUserParams;
-            break;
-        }
 #ifdef USE_AMAPLIB
         case NV2080_CTRL_CMD_FB_GET_AMAP_CONF:
         {
@@ -940,14 +905,6 @@ NV_STATUS embeddedParamCopyOut(RMAPI_PARAM_COPY *paramCopies, RmCtrlParams *pRmC
             ((NV0080_CTRL_HOST_GET_CAPS_PARAMS*)pParams)->capsTbl = paramCopies[0].pUserParams;
             break;
         }
-        case NV0080_CTRL_CMD_MSENC_GET_CAPS:
-        {
-            CHECK_PARAMS_OR_RETURN(pRmCtrlParams, NV0080_CTRL_MSENC_GET_CAPS_PARAMS);
-
-            status = rmapiParamsRelease(&paramCopies[0]);
-            ((NV0080_CTRL_MSENC_GET_CAPS_PARAMS*)pParams)->capsTbl = paramCopies[0].pUserParams;
-            break;
-        }
         case NV2080_CTRL_CMD_BIOS_GET_INFO:
         {
             CHECK_PARAMS_OR_RETURN(pRmCtrlParams, NV2080_CTRL_BIOS_GET_INFO_PARAMS);
@@ -985,14 +942,6 @@ NV_STATUS embeddedParamCopyOut(RMAPI_PARAM_COPY *paramCopies, RmCtrlParams *pRmC
             ((NV0080_CTRL_GR_GET_INFO_PARAMS*)pParams)->grInfoList = paramCopies[0].pUserParams;
             break;
         }
-        case NV0080_CTRL_CMD_FIFO_START_SELECTED_CHANNELS:
-        {
-            CHECK_PARAMS_OR_RETURN(pRmCtrlParams, NV0080_CTRL_FIFO_START_SELECTED_CHANNELS_PARAMS);
-
-            status = rmapiParamsRelease(&paramCopies[0]);
-            ((NV0080_CTRL_FIFO_START_SELECTED_CHANNELS_PARAMS*)pParams)->fifoStartChannelList = paramCopies[0].pUserParams;
-            break;
-        }
         case NV0080_CTRL_CMD_FIFO_GET_CAPS:
         {
             CHECK_PARAMS_OR_RETURN(pRmCtrlParams, NV0080_CTRL_FIFO_GET_CAPS_PARAMS);
@@ -1068,9 +1017,18 @@ NV_STATUS embeddedParamCopyOut(RMAPI_PARAM_COPY *paramCopies, RmCtrlParams *pRmC
 #endif
         case NV0000_CTRL_CMD_SYSTEM_GET_P2P_CAPS:
         {
+            NV_STATUS peerIdsStatus;
             CHECK_PARAMS_OR_RETURN(pRmCtrlParams, NV0000_CTRL_SYSTEM_GET_P2P_CAPS_PARAMS);
-            status = rmapiParamsRelease(&paramCopies[0]);
+
+            peerIdsStatus = rmapiParamsRelease(&paramCopies[0]);
             ((NV0000_CTRL_SYSTEM_GET_P2P_CAPS_PARAMS*)pParams)->busPeerIds = paramCopies[0].pUserParams;
+
+            status = rmapiParamsRelease(&paramCopies[1]);
+            ((NV0000_CTRL_SYSTEM_GET_P2P_CAPS_PARAMS*)pParams)->busEgmPeerIds = paramCopies[1].pUserParams;
+
+            if (peerIdsStatus != NV_OK)
+                status = peerIdsStatus;
+
             break;
         }
         case NV0080_CTRL_CMD_FB_GET_CAPS:
@@ -1155,6 +1113,14 @@ NV_STATUS embeddedParamCopyOut(RMAPI_PARAM_COPY *paramCopies, RmCtrlParams *pRmC
             break;
         }
 
+        case NV2080_CTRL_CMD_GPU_RPC_GSP_TEST:
+        {
+            CHECK_PARAMS_OR_RETURN(pRmCtrlParams, NV2080_CTRL_GPU_RPC_GSP_TEST_PARAMS);
+            status = rmapiParamsRelease(&paramCopies[0]);
+            ((NV2080_CTRL_GPU_RPC_GSP_TEST_PARAMS*)pParams)->data = paramCopies[0].pUserParams;
+            break;
+        }
+
         default:
         {
             return NV_OK;
